from .download import download_zip, download_file
from pathlib import Path
down_path = Path(__file__).parent
xz_url = "addons_file"+"/"+down_path.name+"/"
download_zip(xz_url + "medusaNodesAsset.zip",down_path)

bl_info = {
    "name": "Medusa Nodes",
    "description": "Procedural hair system featuring custom-made Geometry Nodes nodegroups.",
    "author": "Nodes: I. Kupunia, Scripting: L. Boulbayam",
    "version": (1, 1, 0),
    "blender": (4, 0, 2),
    "location": "Node Editors > Sidebar",
    "warning": "The addon is in it’s beta development phase and may exhibit some bugs and not yet production-ready!",
    "doc_url": "https://medusa-nodes-documentation.readthedocs.io",
    "tracker_url": "https://github.com/ikakupa/medusa-nodes-bug-reports",
    "support": "COMMUNITY",
    "category": "MedusaNodes",
}

import bpy
import blf

#Auto-updater module
from . import addon_updater_ops



from bpy.props import (
    CollectionProperty,
    IntProperty,
    BoolProperty,
    StringProperty,
    PointerProperty,
    EnumProperty,
)
from .medusaNodes import *


classes = (
    SCENE_PT_MedusaNodesSaveOptions,
    SCENE_PT_MedusaNodesSaveOptionsIS,
    RENDER_PG_bn_image_settings,
    SCENE_PT_MedusaNodesPrefs,
    MedusaNodesPaintMode,
    MedusaNodesInvokeMaskMenu,
    medusaNodesAddInputGroup,
    MedusaNodesMainDuplGroomFunc,
    MedusaNodesMainDelGroomFunc,
    MedusaNodesOrder,
    MedusaNodesEditMaskGroup,
    MedusaNodesDeleteGroup,
    MedusaNodesViewGroup,
    MedusaNodesMuteLink,
    MedusaNodesCopyInputLinks,
    MedusaNodesPasteInputLinks,
    MedusaNodesCopyInputNodes,
    MedusaNodesPasteInputNodes,
    MedusaNodesCreateMatAtt,
    MedusaNodesRenameMatAtt,
    MedusaNodesRemoveMatAtt,
    MedusaNodeTreeMaskGroupMenu,
    MedusaNodesCreateGuiCurve,
    MedusaNodesAddExsGuiCurve,
    MedusaNodesResInhGuidCurve,
    MedusaNodeTreeGuidesMenu,
    MedusaNodesGuides,
    MedusaNodesNodeProp_Expand,
    MedusaNodesSculptToggle,
    MedusaNodesHide,
    MedusaNodesDisable,
    MedusaNodesDupDeformer,
    MedusaNodesDeleteDeformer,
    MedusaNodesApplyAllDeformers,
    MedusaNodesApplyDeformer,
    MedusaNodesResDeformer,
    MedusaNodeTreeOptionsMenu,
    MedusaNodesAddNode,
    MedusaNodesHideViewport,
    MedusaNodeTreeDeformersMenu,
    MedusaNodesSaveOptions,
    MedusaNodesTree,
    MedusaNodesTreeItem,
    MedusaNodesTreeItem_Expand,
    MedusaNodesAddTemplates,
    MEDUSANODES_UL_TreeItem,
    MedusaNodesMainNewGroom,
    MedusaNodesMainDuplGroom,
    MedusaNodesMainDelGroom,
    MedusaNodesMainNewGroomFunc,
    MedusaNodeTreeOptionsDupMenu,
    MedusaNodesCache,
    MedusaNodesNode_Documentation,
    MedusaNodes_OP_DuplciateDeformerDConnections,
    MedusaNodes_OP_DuplciateDeformerIConnections,

    curveObjectPropGroup,
    MEDUSANODES_UL_CurveObjectCollection,
    MedusaNodesAddCurveObject,
    MedusaNodesRemoveCurveObject,
    MedusaNodesEditCurveObject,
     
    SCENE_PT_MedusaNodesMain_Viewport,
    # SCENE_PT_MedusaNodesMainRiggingView,
    SCENE_PT_MedusaNodesMainRenVieView,
    SCENE_PT_MedusaNodesMainRenVie_curvesView,
    SCENE_PT_MedusaNodesMainRenVie_viewport_displayView,

    SCENE_PT_MedusaNodesMain,
    # SCENE_PT_MedusaNodesMainRigging,
    SCENE_PT_MedusaNodesMainRenVie,
    SCENE_PT_MedusaNodesMainRenVie_curves,
    SCENE_PT_MedusaNodesMainRenVie_viewport_display,
    SCENE_PT_MedusaNodesHierarchyOptionsView,
    SCENE_PT_MedusaNodesHierarchyOptions,
    

    SCENE_PT_MedusaNodesHierarchy_Viewport,
    SCENE_PT_MedusaNodesHierarchy,
    SCENE_PT_MedusaNodesHierarchy_Properties,
    SCENE_PT_MedusaNodesProperties,
    MedusaNodesNotifications,
    MedusaNodesChangeResolution
)

# Rules for the existing guides picker
def poll(self, obj):
    return obj.type == 'CURVES' and "MedusaHair" in obj

def draw_curves_amount():
    spline_count = 0
    point_count = 0
    activeOb = bpy.context.active_object
    contextScreen = bpy.context.screen
    ui_scale = bpy.context.preferences.view.ui_scale

    # int(font_size * ui_scale)
    display = True
    for area in contextScreen.areas:
        if area.type == 'VIEW_3D':
            for space in area.spaces:
                 if space.type == 'VIEW_3D':
                    if space.overlay.show_overlays == False:
                        display = False
                    elif space.overlay.show_text == False:
                        display = False

    if activeOb and display:
        if activeOb.type == "CURVES":
            for mod in activeOb.modifiers:
                if mod.type == "NODES" and mod.name == "MedusaNodes":
                    spline_count, point_count = medusaNodes.getHairCount()
                    font_id = 0
                    lineHeight = blf.dimensions(font_id, 'M')[1] #* 2.0
                    yPosFac = (lineHeight - lineHeight*ui_scale)*-1
                    xPos = 65*ui_scale 

                    blf.position(font_id, xPos, (80+yPosFac)*ui_scale, 0)
                    # blf.size(font_id, 12, 72)#Blender 3.6
                    blf.size(font_id, int(14 * ui_scale))#Blender 4.0
                    blf.color(font_id, 0.462746, 0.858824, 0.666667, 1)#color: #76DBAA
                    #blf.color(font_id, 0.1, 1, 0.670591, 1)
                    blf.draw(font_id, "Medusa Nodes" )

                    blf.position(font_id, xPos, (60+yPosFac)*ui_scale, 0)
                    blf.size(font_id, int(14 * ui_scale))
                    blf.color(font_id, 1, 1, 1, 1)
                    blf.draw(font_id, activeOb.name)

                    blf.position(font_id, xPos, (40+yPosFac)*ui_scale, 0)
                    #blf.size(font_id, 12, 72)#Blender 3.6
                    blf.size(font_id, int(14 * ui_scale))#Blender 4.0
                    blf.color(font_id, 1, 1, 1, 1)
                    blf.draw(font_id, "Points : {:,}".format(point_count) )

                    blf.position(font_id, xPos, (20+yPosFac)*ui_scale, 0) 
                    # blf.size(font_id, 12, 72)#Blender 3.6
                    blf.size(font_id, int(14 * ui_scale))#Blender 4.0
                    blf.color(font_id, 1, 1, 1, 1)
                    blf.draw(font_id, "Splines : {:,}".format(spline_count))


                    break

drawHandle = None
timerHandle = None

def register():
    global drawHandle
    global timerHandle
    # for pcoll in preview_collections.values():
    #     bpy.utils.previews.remove(pcoll)
    # preview_collections.clear()
    # pcoll = bpy.utils.previews.new()
    # my_icons_dir = os.path.join(assetPath, "icons")
    # pcoll.load("temp1", os.path.join(my_icons_dir, "temp1.png"), 'IMAGE')
    # pcoll.load("temp2", os.path.join(my_icons_dir, "temp2.png"), 'IMAGE')
    # pcoll.load("temp3", os.path.join(my_icons_dir, "temp3.png"), 'IMAGE')
    # pcoll.load("temp4", os.path.join(my_icons_dir, "temp4.png"), 'IMAGE')
    # pcoll.load("temp5", os.path.join(my_icons_dir, "temp5.png"), 'IMAGE')
    # preview_collections["main"] = pcoll

    #Auto-updater register
    try:
        addon_updater_ops.register(bl_info)
    except:
        pass
    ###################################

    for cls in classes:
        bpy.utils.register_class(cls)
    bpy.types.Scene.medusaGuidePicker = PointerProperty(
            name = "Guides",
            type=bpy.types.Object,
            poll=poll)
    bpy.types.Scene.medusaNodesSaveOptions = PointerProperty(type=MedusaNodesSaveOptions)
    bpy.types.Scene.medusaNodes = CollectionProperty(type=MedusaNodesTree)
    bpy.types.Scene.use_medusa_hair_object = BoolProperty(name="Solo Guide Hierarchy", description = "Display the Guide object's hierarchy independently whenever the Guide object is active (leave this option unchecked for a better experience)", default=False)
    bpy.types.Scene.lastMedusaHairOb = StringProperty()
    bpy.types.Scene.newGroomName = StringProperty(
        name="",
        description="Groom Name",
        default = "Groom01",
        update = checkExistingPart
    )
    
    bpy.types.Scene.newGroomPartName= StringProperty(
        name="",
        description="Groom Part Name",
        default = "Part01"
    )
    bpy.types.Scene.medusaNodesTree = CollectionProperty(type=MedusaNodesTreeItem)
    bpy.types.Scene.medusaNodesTree_index = IntProperty(default=-1, name = "Custom Medusa Nodes' node")
    bpy.types.Scene.medusaNodesActiveOb = StringProperty(default="")
    bpy.types.Scene.medusaNodesActiveNodeTree = StringProperty(default="")
    bpy.types.Scene.medusaNodesActiveNode = CollectionProperty(type=MedusaNodesTreeItem)

    bpy.msgbus.subscribe_rna(
        key=(bpy.types.Object, 'active_material'),
        owner="medusa_nodes",
        args=(),
        notify=msgbus_callback
    )


    bpy.types.Collection.curve_object_list = bpy.props.CollectionProperty(type = curveObjectPropGroup)
    bpy.types.Collection.curve_object_index = bpy.props.IntProperty()


    for hand in bpy.app.handlers.depsgraph_update_post:
        if nodesUpdate.__name__ == hand.__name__:
            bpy.app.handlers.depsgraph_update_post.remove(hand)
    
    for hand in bpy.app.handlers.save_post:
        if blendFileSaved.__name__ == hand.__name__:
            bpy.app.handlers.save_post.remove(hand)

    bpy.app.handlers.depsgraph_update_post.append(nodesUpdate)
    bpy.app.handlers.save_post.append(blendFileSaved)
    
    drawHandle = bpy.types.SpaceView3D.draw_handler_add(draw_curves_amount, (), 'WINDOW', 'POST_PIXEL')
    timerHandle = bpy.app.timers.register(subpanel_update, persistent=True)

def unregister():
    global timerHandle
    # for pcoll in preview_collections.values():
    #     bpy.utils.previews.remove(pcoll)
    # preview_collections.clear()
    panel_class_names = []
    for cls in bpy.types.Panel.__subclasses__():
        if "MedusaSubPanel" in cls.__name__:
            panel_class_names.append(cls)
    for classO in panel_class_names:
        try:
            bpy.utils.unregister_class(classO)
        except:
            pass

    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)
    for hand in bpy.app.handlers.depsgraph_update_post:
        if nodesUpdate.__name__ == hand.__name__:
            bpy.app.handlers.depsgraph_update_post.remove(hand)
    for hand in bpy.app.handlers.save_post:
        if blendFileSaved.__name__ == hand.__name__:
            bpy.app.handlers.save_post.remove(hand)
    bpy.types.SpaceView3D.draw_handler_remove(drawHandle, 'WINDOW')
    bpy.msgbus.clear_by_owner("medusa_nodes")
    addon_updater_ops.unregister()


    del bpy.types.Collection.curve_object_list
    del bpy.types.Collection.curve_object_index

    if timerHandle:
        bpy.app.timers.unregister(timerHandle)
    else:
        timerHandle = None

