import os
import re
import bpy
import json
import textwrap
import datetime
import numpy as np
import bpy.utils.previews
#Auto-updater module
from . import addon_updater_ops
from bpy.types import PropertyGroup
from collections import defaultdict
from .nodesArrange import nodes_iterate
from bpy.app.handlers import persistent
from bl_ui.properties_render import draw_curves_settings
from bpy.app.translations import pgettext_iface as iface_

lstData = None
lstCount = -1

from bpy.props import (
    CollectionProperty,
    IntProperty,
    BoolProperty,
    StringProperty,
    PointerProperty,
    EnumProperty,
    FloatProperty
)

bl_info = {
    "name": "Medusa Nodes",
    "description": "A procedural hair system featuring custom-made Geometry Nodes' node groups.",
    "author": "Nodes: I. Kupunia, Scripting: L. Boulbayam",
    "version": (1, 1, 0),
    "blender": (4, 0, 2),
    "location": "3D View > Sidebar, Node Editors > Sidebar",
    "warning": "The addon is in it’s beta development phase and may exhibit some bugs and not production-ready yet!",
    "doc_url": "https://medusa-nodes-documentation.readthedocs.io",
    "tracker_url": "https://github.com/ikakupa/medusa-nodes-bug-reports",
    "support": "COMMUNITY",
    "category": "MedusaNodes",
}

activeNodeGroupName = ""
activeNodeGroupSocket = ""
activeNodeTreeName = ""

treeOldList = {}
treeOldCpt = -1
preview_collections = {}
paintModeLstOb = None

sculptModeLstOb = None
sculptMode = False
needToUpdate = False


attrbData = []

script_file = os.path.realpath(__file__)
directory = os.path.dirname(script_file)
assetPath = os.path.join(directory,"medusaNodesAsset")
jsonProfile = os.path.join(directory,"curve_mask_profile.json")
inputsDisable = os.path.join(directory,"inputs_to_disable.json")
with open(inputsDisable, 'r') as f:
    inputsDisableData = json.load(f)
#Icon path added - Irakli
iconPath = os.path.join(assetPath, "icons")
subIconPath = os.path.join(iconPath, "deformer_icons")
addDeformerMenuIconPath = os.path.join(subIconPath, "hair")
guideIconPath = os.path.join(subIconPath, "guide")


brushList = ('Add','Delete','Density','Slide')


pcoll = bpy.utils.previews.new()
guidepcoll = bpy.utils.previews.new()

for entry in os.scandir(addDeformerMenuIconPath):
    if entry.name.endswith(".png"):
        name = os.path.splitext(entry.name)[0]
        pcoll.load(name, entry.path, "IMAGE")
for entry in os.scandir(iconPath):
    if entry.name.endswith(".png"):
        name = os.path.splitext(entry.name)[0]
        pcoll.load(name, entry.path, "IMAGE")

for entry in os.scandir(guideIconPath):
    if entry.name.endswith(".png"):
        name = os.path.splitext(entry.name)[0]
        guidepcoll.load(name, entry.path, "IMAGE")

#--------------------------------------------------------------------
def check_valid(item):
    if item:
        if item.objectName in bpy.data.objects:
            actOb = bpy.data.objects[item.objectName]
            if len(actOb.modifiers)>0:
                if item.nodeTreeName in actOb.modifiers:
                    return True
    try:
        SetupNodeData()
    except:
        pass
    return False

def addApplyUpdateTree(obj,emitter):
    lstMode = bpy.context.object.mode
    bpy.ops.object.mode_set(mode="OBJECT")
    updateNodeTree = bpy.data.node_groups.get("MN Update")
    if updateNodeTree is None:
        blendFile = os.path.join(assetPath, "medusaNodes.blend")
        with bpy.data.libraries.load(blendFile) as (data_from, data_to):
            data_to.node_groups = ["MN Update"]
    updateNodeTree = bpy.data.node_groups.get("MN Update")
    updateModifier = obj.modifiers.new("Temp","NODES")
    updateModifier.node_group = updateNodeTree
    updateModifier["Socket_2"] = emitter

    try:
        bpy.ops.object.modifier_apply(modifier = updateModifier.name)
    except:
        obj.modifiers.remove(updateModifier)
    bpy.ops.object.mode_set(mode=lstMode)


def clear_node(node):
    node_groups = bpy.data.node_groups
    (base, sep, ext) = node.node_tree.name.rpartition('.')
    if ext.isnumeric():
        if base in node_groups:
            oldTree = node.node_tree
            node.node_tree.use_fake_user = False
            node.node_tree = node_groups.get(base)
            if oldTree.users == 0:
                bpy.data.node_groups.remove(oldTree)


def addMedusaNode(nodeGroup,nodeNameToAdd):
    ori_node_groups = bpy.data.node_groups[:].copy()
    blendFile = os.path.join(assetPath, "medusaNodes.blend")
    with bpy.data.libraries.load(blendFile) as (data_from, data_to):
        nodeInScene = None
        nodeToAdd = None
        if "mask group" not in nodeNameToAdd.lower():
            nodeInScene = bpy.data.node_groups.get(nodeNameToAdd)
            nodeToAdd = nodeInScene
        if nodeInScene is None:
            data_to.node_groups = [nodeNameToAdd]

        if "Visualizer" in nodeNameToAdd:
            matsImport = []
            for mat in data_from.materials:
                if mat not in bpy.data.materials:
                    matsImport.append(mat)
            data_to.materials = matsImport
    if nodeToAdd is None:
        nodeToAdd = data_to.node_groups[0]

    grp = None
    if nodeToAdd:
        nodeTreeInScene = nodeToAdd
        if nodeTreeInScene:
            matnum = re.search(".\d\d\d$", nodeTreeInScene.name)
            if matnum:
                try:
                    index = int(matnum.group(0).replace(".",""))
                except:
                    index = 0
                nodeTreeInScene.name = nodeTreeInScene.name.replace(matnum.group(0)," "+str(index))
            grp = nodeGroup.nodes.new("GeometryNodeGroup")
            grp.node_tree = nodeTreeInScene
            grp.label = nodeNameToAdd.replace("MN ","")
            grp.width = 250
            if "visualizer" in nodeTreeInScene.name.lower():
                usedMats = []
                for mat in bpy.data.materials:
                    if "root normal" in mat.name.lower():
                        grp.inputs["Root Normal Material"].default_value = mat
                        usedMats.append(mat)
                    else:
                        if "guide" in nodeTreeInScene.name.lower():
                            if "_guide" in mat.name.lower():
                                grp.inputs["Hair Material"].default_value = mat
                                usedMats.append(mat)
                        else:
                            if "groom" in mat.name.lower() and "_guide" not in mat.name.lower():
                                grp.inputs["Hair Material"].default_value = mat
                                usedMats.append(mat)
                
                for mat in matsImport:
                    if mat not in usedMats:
                        bpy.data.materials.remove(mat)
    #alignMaskGroupNodes(nodeGroup)

    for group in bpy.data.node_groups:
        if group not in ori_node_groups:
            for node in group.nodes:
                if node.type == 'GROUP': 
                    if node:
                        if node.node_tree:
                            if "mask group" not in node.node_tree.name.lower():
                                clear_node(node)


    return grp

def addMedusaProfileNode(nodeGroup,node):
    profileNodes=[]
    for input in node.inputs:
        #ramp Irakli
        if "ramp" in input.name.lower():
            if not input.is_linked:
                groupNode = addMedusaNode(nodeGroup,"MN Ramp Mask Group")
                nodeGroup.links.new(input,groupNode.outputs[0])
                profileNodes.append(groupNode)
                
        #----------------------------------------------------------------
        if "profile" in input.name.lower():
            if not input.is_linked:
                groupNode = addMedusaNode(nodeGroup,"MN Curve Mask Group")
                nodeGroup.links.new(input,groupNode.outputs[0])
                profileNodes.append(groupNode)

                with open(jsonProfile, 'r') as f:
                    profileData = json.load(f)
                for json_key in profileData:
                    if node.node_tree.name in json_key:
                        if input.name in profileData[json_key]:
                            for snode in groupNode.node_tree.nodes:
                                if snode.type == "CURVE_FLOAT":
                                    m = snode.mapping
                                    while len(m.curves[0].points)>2:
                                        p = m.curves[0].points[0]
                                        m.curves[0].points.remove(p)
                                    data = profileData[json_key][input.name]
                                    if len(data) > 1:
                                        m.curves[0].points[0].location.x = data[0][0]
                                        m.curves[0].points[0].location.y = data[0][1]
                                        for i in range(1,len(data)-1):
                                            point = data[i]
                                            p = m.curves[0].points.new(point[0],point[1])
                                        m.curves[0].points[-1].location.x = data[-1][0]
                                        m.curves[0].points[-1].location.y = data[-1][1]
                                    m.update()
    return profileNodes


def alignNodes(nodeTree):
    nodes_iterate(nodeTree)
    doNotMove =['GEOMETRY','OBJECT']
    for node in nodeTree.nodes:
        placement = 5
        for input in node.inputs:
            if input.type not in doNotMove:
                for link in input.links:
                    dimensionY = max(min(link.from_node.bl_height_default, link.from_node.bl_height_max), link.from_node.bl_height_min,link.from_node.dimensions.y,link.from_node.height)
                    link.from_node.location.x = node.location.x
                    link.from_node.location.y = dimensionY + node.location.y + placement
                    placement = link.from_node.location.y + 5

def nodeColor(node):
    deformerBaseColor = [0.24447, 0.607777, 0.470434]
    names = ['Guide', 'Clump', 'Braid', 'Curl', 'Noise', 'Trim', 'Curves to Splines' ,'Mirror','Scale','Resample Hair', 'Children', 'Dynamics', 'Cache']

    if node.label in names:
        index = names.index(node.label)
        factor = 0.04
        result = index * factor
        node.use_custom_color = True
        node.color = [c - result for c in deformerBaseColor]
    elif "visualizer" in node.label.lower():
        node.use_custom_color = True
        node.color = [0.351533, 0.376262, 0.610496]
    elif "generator" in node.label.lower():
        node.use_custom_color = True
        node.color = [0.608, 0.284448, 0.271195]

def areaFactor(emitterGeometry):
    polygons = emitterGeometry.data.polygons
    area = 0.0
    for p in polygons:
        area += p.area
    areaFactor = 1 / area

    return areaFactor


    

class MedusaNodesAddNode(bpy.types.Operator):
    bl_idname = "medusanodes.add_node"
    bl_label = ""
    bl_options = {'UNDO'}

    nodeName : StringProperty(default="")

    enum_items = (('MN Guide', "Create 'Guide' node", "Creates new 'Guide' node", 'Guide',0),
                 ('MN Clump', "Create 'Clump' node", "Creates new 'Clump' node", 'Clump',1),
                 ('MN Braid', "Create 'Braid' node", "Creates new 'Braid' node", 'Braid',2),
                 ('MN Noise', "Create 'Noise' node", "Creates new 'Noise' node", 'Noise',3),
                 ('MN Curl', "Create 'Curl' node", "Creates new 'Curl' node", 'Curl',4),
                 ('MN Trim', "Create 'Trim' node", "Creates new 'Trim' node", 'Trim',5),
                 ('MN Mirror', "Create 'Mirror' node", "Creates new 'Mirror' node", 'Mirror',6),
                 ('MN Curves to Splines', "Create 'Curves to Splines' node", "Creates new 'Curves to Splines' node", 'Curves to Splines',7),
                 ('MN Scale', "Create 'Scale' node", "Creates new 'Scale' node", 'Scale',8),
                 ('MN Resample Hair', "Create 'Resample Hair' node", "Creates new 'Resample Hair' node", 'Resample Hair',9),
                 ('MN Children', "Create 'Children' node", "Creates new 'Children' node", 'Children',10),
                 ('MN Dynamics', "Create 'Simulation' node", "Creates new 'Simulation' node", 'Simulation',11),
                 ('MN Cache', "Create 'Cache' node", "Creates new 'Cache' node", 'Cache',12))

    @classmethod
    def description(cls, context, properties):
        defDescrp = "Add new node"
        for item in cls.enum_items:
            if properties.nodeName.lower() in item[0].lower():
                return item[1]
        return defDescrp
    
    def execute(self, context):
        global processedNode
        item_index = context.scene.medusaNodesTree_index
        item_list = context.scene.medusaNodesTree
        allitemsLists = context.scene.medusaNodes
        
        alreadyExi = False
        if item_list:
            for item in item_list:
                geoNodes0 = bpy.data.objects[item.objectName].modifiers[item.nodeTreeName]
                activeNode0 = geoNodes0.node_group.nodes[item.name]
            
                if "mn guide" in activeNode0.node_tree.name.lower():
                    alreadyExi = True
                    break
            if len(item_list)>item_index and item_index != -1:
                item = item_list[item_index]
                if item.objectName in bpy.data.objects:
                    if len(bpy.data.objects[item.objectName].modifiers)>0:
                        if check_valid(item):
                            geoNodes = bpy.data.objects[item.objectName].modifiers[item.nodeTreeName]
                            if geoNodes:
                                if geoNodes.type == 'NODES':
                                    groupOutput = geoNodes.node_group.nodes['Group Output']
                                    activeNode = geoNodes.node_group.nodes[item.name]
                                    area_factor = areaFactor(geoNodes['Socket_2'])
                                    dimensions = geoNodes['Socket_2'].dimensions
                                    dimension_factor = (dimensions[0]+dimensions[1]+dimensions[2]) / 3

                                    nextItemF = None
                                    if "mn visual" in activeNode.node_tree.name.lower():
                                        nextIndex = item_index + 1
                                    else:
                                        nextIndex = item_index - 1
                                    if not item.expanded and item.childCount > 0 and "mn visual" in activeNode.node_tree.name.lower():
                                        for i in range(0,len(allitemsLists)):
                                            nextItem = allitemsLists[i]
                                            if nextItem.nodeTreeName == item.nodeTreeName and nextItem.objectName == item.objectName and nextItem.name != activeNode.name:
                                                nextNode = geoNodes.node_group.nodes[nextItem.name]
                                                
                                                if nextIndex > item_index:
                                                    if "mn visual" not in nextNode.node_tree.name.lower():
                                                        nextItemF = nextItem
                                                        break
                                                else:
                                                    nextItemF = nextItem
                                                    break
                                    else:
                                        # store all indexs 
                                        sameHeiIndex = []
                                        actiIndex = -1
                                        for i in range(0,len(item_list)):
                                            nextItemT = item_list[i]
                                            if nextItemT.nodeTreeName == item.nodeTreeName and nextItemT.objectName == item.objectName:
                                                sameHeiIndex.append(i)
                                                if nextItemT.name == activeNode.name:
                                                    actiIndex = len(sameHeiIndex)-1
                                        if "mn visual" in activeNode.node_tree.name.lower():
                                            nextItemF = item_list[sameHeiIndex[actiIndex+1]]
                                        else:
                                            nextItemF = item_list[sameHeiIndex[actiIndex-1]]
                                    nodeLinks = geoNodes.node_group.links
                                    newNode = addMedusaNode(geoNodes.node_group,self.nodeName)
                                    profileNodes = addMedusaProfileNode(geoNodes.node_group,newNode)
                                    if nextItemF:
                                        nextNode = geoNodes.node_group.nodes[nextItemF.name]
                                        if newNode:
                                            nodeColor(newNode)
                                            if nextIndex > item_index:
                                                newNode.location.x = activeNode.location.x + newNode.width + 20
                                                newNode.location.y = activeNode.location.y
                                                
                                                for output in newNode.outputs:
                                                    for input in activeNode.inputs:
                                                        if output.name == input.name:
                                                            if input.node != output.node:
                                                                nodeLinks.new(input, output)

                                                for input in newNode.inputs:
                                                    #create curves collection Curves to Splines
                                                    if 'curve collection' in input.name.lower():
                                                        colName = item.objectName+'_Curves'
                                                        collection = bpy.data.collections.get(colName)
                                                        if collection is None:
                                                            collection = bpy.data.collections.new(colName)
                                                        try:
                                                            bpy.data.objects[item.objectName].users_collection[0].children.link(collection)
                                                        except:
                                                            pass
                                                        bpy.ops.medusanodes.add_curve_object() #add first curve
                                                        input.default_value = collection
                                                    emiGeo = 0
                                                    for output in nextNode.outputs:
                                                        if output.name == input.name:
                                                            if "emitter geometry" in input.name.lower():
                                                                emiGeo = 1
                                                            if input.node != output.node:
                                                                nodeLinks.new(input, output)
                                                    if "emitter geometry" in input.name.lower() and emiGeo == 0:
                                                        nodeHier = followLinks(activeNode)
                                                        processedNode = {}
                                                        for node in reversed(nodeHier):
                                                            for out in node.outputs:
                                                                if "emitter geometry" in out.name.lower():
                                                                    if input.node != out.node:
                                                                        nodeLinks.new(input, out)
                                                                        break


                                            else:
                                                newNode.location.x = activeNode.location.x + 250 + 20
                                                newNode.location.y = activeNode.location.y


                                                for input in newNode.inputs:
                                                    emiGeo = 0
                                                    for output in activeNode.outputs:
                                                        if output.name == input.name:
                                                            if "emitter geometry" in input.name.lower():
                                                                emiGeo = 1
                                                            if input.node != output.node:
                                                                nodeLinks.new(input, output)
                                                    if "emitter geometry" in input.name.lower() and emiGeo == 0:
                                                        nodeHier = followLinks(activeNode)
                                                        processedNode = {}
                                                        for node in reversed(nodeHier):
                                                            for out in node.outputs:
                                                                if "emitter geometry" in out.name.lower():
                                                                    if input.node != out.node:
                                                                        nodeLinks.new(input, out)
                                                                        break
                                                    #Dynamic Density for clump and other deformers which have the density value
                                                    if 'density' in input.name.lower() and 'map' not in input.name.lower():
                                                        input.default_value= area_factor * 1000
                                                    if 'scale' in input.name.lower() and 'profile' not in input.name.lower():
                                                        if 'noise' in newNode.node_tree.name.lower():
                                                            input.default_value = dimension_factor * 1 / 2
                                                        elif 'curl' in newNode.node_tree.name.lower():
                                                            input.default_value = dimension_factor * 1 / 7
                                                    #Braid
                                                    if 'braid' in newNode.node_tree.name.lower():
                                                        if 'sides offset from emitter' in input.name.lower():
                                                            input.default_value = dimension_factor * 1 / 2 / 10
                                                        # if 'braid thickness' in input.name.lower():
                                                        #     input.default_value = dimension_factor * 1 / 2 / 10 #dependent on the scale, not needed
                                                        if 'braid scale' == input.name.lower():
                                                            input.default_value = dimension_factor * 1 / 2 / 20
                                                        if 'braid frequency' == input.name.lower():
                                                            input.default_value = (1 / dimension_factor) * 10
                                                            
                                                    #Set 'Volumize' only for the first clump deformer
                                                    ClumpDeformerCount = []
                                                    for node in geoNodes.node_group.nodes:
                                                        if node.type == 'GROUP':
                                                            if node.node_tree.name.lower() == 'mn clump':
                                                                ClumpDeformerCount.append(1)

                                                    if 'volumize' in input.name.lower() and len(ClumpDeformerCount) == 1:
                                                        input.default_value = 1.0

                                                    #create curves collection Curves to Splines
                                                    if 'curve collection' in input.name.lower():
                                                        colName = item.objectName+'_Curves'
                                                        collection = bpy.data.collections.get(colName)
                                                        if collection is None:
                                                            collection = bpy.data.collections.new(colName)
                                                        try:
                                                            bpy.data.objects[item.objectName].users_collection[0].children.link(collection)
                                                        except:
                                                            pass
                                                        bpy.ops.medusanodes.add_curve_object() #add first curve
                                                        input.default_value = collection

                                                for output in newNode.outputs:
                                                    for input in nextNode.inputs:
                                                        if output.name == input.name:
                                                            if input.node != output.node:
                                                                nodeLinks.new(input, output)
                                                nextNode.location.x = newNode.location.x + newNode.width + 20        
                                    else:
                                        if newNode:
                                            newNode.location.x = activeNode.location.x - 250
                                            for output in newNode.outputs:
                                                for input in activeNode.inputs:
                                                    if output.name == input.name:
                                                        if input.node != output.node:
                                                            nodeLinks.new(input, output)

        
                                    groupOutput.location.x = groupOutput.inputs[0].links[0].from_node.location.x + groupOutput.inputs[0].links[0].from_node.width + 20
                                    nodeCpt = 0
                                    for node in geoNodes.node_group.nodes:
                                        if node.type == "GROUP":
                                            if newNode.node_tree.name == node.node_tree.name:# and newNode.label in node.label:
                                                nodeCpt += 1
                                    if nodeCpt>1:
                                        newNode.label = newNode.label + " " + str(nodeCpt-1)

                                    SetupNodeData()
                                    if newNode and not alreadyExi and ("mn guide" in newNode.node_tree.name.lower() or "mn braid" in newNode.node_tree.name.lower()):
                                        try:
                                            if "mn visual" in activeNode.node_tree.name.lower():
                                                ind = nextIndex
                                                if not item.expanded:
                                                    for i in range(0,len(allitemsLists)):
                                                        indexIte = allitemsLists[i]
                                                        if indexIte.name == newNode.name:
                                                            ind = indexIte.selfIndex
                                                bpy.ops.medusanodes.guides_menu(index=ind, guideItems='0')
                                            else:
                                                bpy.ops.medusanodes.guides_menu(index=bpy.context.scene.medusaNodesTree_index, guideItems='0')
                                        except:
                                            pass
                                    elif newNode and alreadyExi and "mn guide" in newNode.node_tree.name.lower():
                                        try:
                                            if "mn visual" in activeNode.node_tree.name.lower():
                                                ind = nextIndex
                                                if not item.expanded:
                                                    for i in range(0,len(allitemsLists)):
                                                        indexIte = allitemsLists[i]
                                                        if indexIte.name == newNode.name:
                                                            ind = indexIte.selfIndex
                                                bpy.ops.medusanodes.guides_menu(index=ind, guideItems='0')
                                            else:
                                                bpy.ops.medusanodes.guides_menu(index=bpy.context.scene.medusaNodesTree_index, guideItems='0')
                                        except:
                                            pass
                                    elif newNode and alreadyExi and ("mn clump" in newNode.node_tree.name.lower() or "mn braid" in newNode.node_tree.name.lower()):
                                        try:
                                            if "mn visual" in activeNode.node_tree.name.lower():
                                                ind = nextIndex
                                                if not item.expanded:
                                                    for i in range(0,len(allitemsLists)):
                                                        indexIte = allitemsLists[i]
                                                        if indexIte.name == newNode.name:
                                                            ind = indexIte.selfIndex
                                                bpy.ops.medusanodes.guides_menu(index=ind, guideItems='2')
                                            else:
                                                bpy.ops.medusanodes.guides_menu(index=bpy.context.scene.medusaNodesTree_index, guideItems='2')
                                        except:
                                            pass
                                    geoNodes.show_viewport = False
                                    geoNodes.show_viewport = True
                                    
            if newNode:
                if len(item_list)>item_index and item_index != -1:
                    item = item_list[item_index]
                    if item.objectName in bpy.data.objects:
                        if len(bpy.data.objects[item.objectName].modifiers)>0:
                            if check_valid(item):
                                for i in range(0,len(item_list)):
                                    itemC = item_list[i]
                                    if check_valid(itemC):
                                        if itemC.label == newNode.label and itemC.name == newNode.name and itemC.objectName == item.objectName:
                                            bpy.context.scene.medusaNodesTree_index  = i
                                            break
            alignNodes(geoNodes.node_group)
        else:
            message = f"To be able to add a node create a new Medusa Nodes's groom first!"
            self.report({'INFO'}, message) 

        return {'FINISHED'}

def align2Nodes(nodeA,nodeB):
    y_locs = nodeA.location.y - nodeA.dimensions.y
    nodeB.location.y = y_locs

def deselectNodes(geoNodes):
    for node in geoNodes.node_group.nodes:
        node.select= False

def deleteSelectedNodes(geoNodes):
    for node in geoNodes.node_group.nodes:
        if node.select == True:
            try:
                geoNodes.node_group.nodes.remove(node)
            except:
                pass

def getSelectedNodes(geoNodes):
    selectedNodes = []
    for node in geoNodes.node_group.nodes:
        if node.select == True:
            selectedNodes.append(node)
    return selectedNodes

def duplicateConnections(geoNodes,node):
    global processedNode
    deselectNodes(geoNodes)
    allNodes = followLinks(node,inp=None)
    processedNode = {}
    finNodes = []
    for nodeS in allNodes:
        if nodeS not in finNodes and nodeS.type != "OBJECT_INFO":
            if nodeS.outputs[0].name.lower() != "splines" and "hair" not in nodeS.outputs[0].name.lower():
                finNodes.append(nodeS)
                nodeS.select = True
            elif nodeS == node:
                finNodes.append(node)
                node.select = True
        for subNode in allNodes[nodeS]:
            if subNode not in finNodes and subNode.type != "OBJECT_INFO":
                if subNode.outputs[0].name.lower() != "splines" and "hair" not in subNode.outputs[0].name.lower():
                    finNodes.append(subNode)
                    subNode.select = True
        break
    finNodes = getSelectedNodes(geoNodes)

    if bpy.context.area.type == "VIEW_3D":
        bpy.ops.screen.userpref_show("INVOKE_DEFAULT")
        addedArea = bpy.context.window_manager.windows[-1].screen.areas[0]
        addedArea.type = "NODE_EDITOR"
        addedArea.spaces.active.tree_type = 'GeometryNodeTree'
        bpy.context.view_layer.update()
        bpy.ops.wm.redraw_timer(type='DRAW_WIN_SWAP', iterations=1)

        for area in bpy.context.screen.areas:
            if area.type == 'NODE_EDITOR' and area.spaces[0].tree_type == 'GeometryNodeTree':
                with bpy.context.temp_override(window=bpy.context.window,area=area,region=[region for region in area.regions if region.type == 'WINDOW'][0]):
                    bpy.ops.node.duplicate_move()
                break
        if addedArea:
            bpy.ops.wm.window_close()
            addedArea = None
    else:
        bpy.ops.node.duplicate_move()
    duplicNodes = getSelectedNodes(geoNodes)
    for nodeIndex in range(0,len(duplicNodes)):
        deselectNodes(geoNodes)
        nodeDu = duplicNodes[nodeIndex]
        r = re.sub('.*?([0-9]*)$',r'\1',nodeDu.label)
        
        if r:
            if r.isdigit():
                nodeDu.label = nodeDu.label.replace(r,str(int(r)+1).zfill(2))
            else:
                nodeDu.label = nodeDu.label.replace(r,str(1).zfill(2))
        else:
            if nodeDu.label == "":
                if nodeDu.type == "GROUP":
                    nodeDu.label = nodeDu.node_tree.name.lower() + "01"
            else:
                nodeDu.label = nodeDu.label + "01"
        
        nodeOri = finNodes[nodeIndex]
        if nodeDu.type == nodeOri.type:
            align2Nodes(nodeOri,nodeDu)
        
        if nodeDu.type == "GROUP":
            if nodeDu.node_tree.name.lower() == node.node_tree.name.lower():
                for out in node.outputs:
                    if out.name.lower() in ["splines","guide curves","emitter geometry"]:
                        if out.is_linked:
                            for link in out.links:
                                geoNodes.node_group.links.new(nodeDu.outputs[out.name],link.to_socket)
                            geoNodes.node_group.links.new(nodeDu.inputs[out.name],out)
                        elif "guide curves" in out.name.lower():
                            for link in out.links:
                                geoNodes.node_group.links.new(nodeDu.outputs[out.name],link.to_socket)
                            geoNodes.node_group.links.new(nodeDu.inputs[out.name],out)

            if "mask" in nodeDu.node_tree.name.lower():
                nodeDu.node_tree = nodeDu.node_tree.copy()
    alignNodes(geoNodes.node_group)

def inheritConnections(geoNodes,node):
    deselectNodes(geoNodes)
    duplicatedInhherNodes = []
    node.select = True
    if bpy.context.area.type == "VIEW_3D":
        bpy.ops.screen.userpref_show("INVOKE_DEFAULT")
        addedArea = bpy.context.window_manager.windows[-1].screen.areas[0]
        addedArea.type = "NODE_EDITOR"
        addedArea.spaces.active.tree_type = 'GeometryNodeTree'
        bpy.context.view_layer.update()
        bpy.ops.wm.redraw_timer(type='DRAW_WIN_SWAP', iterations=1)
        for area in bpy.context.screen.areas:
            if area.type == 'NODE_EDITOR' and area.spaces[0].tree_type == 'GeometryNodeTree':
                with bpy.context.temp_override(window=bpy.context.window,area=area,region=[region for region in area.regions if region.type == 'WINDOW'][0]):
                    node.select = True
                    bpy.ops.node.duplicate_move(NODE_OT_duplicate={"keep_inputs":True})
                    duplicatedInhherNodes = getSelectedNodes(geoNodes)
                break
        if addedArea:
            bpy.ops.wm.window_close()
            addedArea = None
    else:
        bpy.ops.node.duplicate_move(NODE_OT_duplicate={"keep_inputs":True})
        duplicatedInhherNodes = getSelectedNodes(geoNodes)
    for nodeIndex in range(0,len(duplicatedInhherNodes)):
        nodeDu = duplicatedInhherNodes[nodeIndex]
        r = re.sub('.*?([0-9]*)$',r'\1',nodeDu.label)
        if r:
            if r.isdigit():
                nodeDu.label = nodeDu.label.replace(r,str(int(r)+1).zfill(2))
            else:
                nodeDu.label = nodeDu.label.replace(r,str(1).zfill(2))
        else:
            nodeDu.label = nodeDu.label + "01"
        if nodeDu.type == "GROUP":
            if nodeDu.node_tree.name.lower() == node.node_tree.name.lower():
                for out in node.outputs:
                    if out.name.lower() in ["splines","guide curves","emitter geometry"]:
                        if out.is_linked:
                            for link in out.links:
                                geoNodes.node_group.links.new(nodeDu.outputs[out.name],link.to_socket)
                            geoNodes.node_group.links.new(nodeDu.inputs[out.name],out)
                        elif "guide curves" in out.name.lower():
                            for link in out.links:
                                geoNodes.node_group.links.new(nodeDu.outputs[out.name],link.to_socket)
                            geoNodes.node_group.links.new(nodeDu.inputs[out.name],out)
    align2Nodes(node,getSelectedNodes(geoNodes)[-1])
    alignNodes(geoNodes.node_group)

def getHairCount(obj=None):
    '''Returns the spline and point count of the selected Medusa Nodes object'''
    spline_count = 0
    point_count = 0

    if obj is None:
        obj = bpy.context.active_object
    if obj:
        if obj.type == "CURVES":
            deps = bpy.context.evaluated_depsgraph_get()
            eval_obj = obj.evaluated_get(deps)
            spline_count = len(eval_obj.data.curves)
            point_count = len(eval_obj.data.curves.data.points)
    return spline_count, point_count

class MedusaNodes_OP_DuplciateDeformerDConnections(bpy.types.Operator):
    '''All of the connected 'Mask Groups' will be duplicated with the deformer as well'''
    bl_label="Duplicate 'Mask Groups'"
    bl_idname='medusanodes.duplicate_deformer_dupli'
    bl_options = {'UNDO'}

    def execute(self, context):
        activeNode = bpy.context.scene.medusaNodesTree[bpy.context.scene.medusaNodesTree_index]
        if check_valid(activeNode):       
            geoNodes = bpy.data.objects[activeNode.objectName].modifiers[activeNode.nodeTreeName]
            node = geoNodes.node_group.nodes[activeNode.name]

            if 'generator'in node.node_tree.name.lower() or 'visualizer' in node.node_tree.name.lower():
                message = f"Duplicating the base nodes like '{node.node_tree.name[3:]}' is not possible!"
                self.report({'INFO'}, message)   
            else:
                duplicateConnections(geoNodes,node) 
                           
        return {'FINISHED'}

class MedusaNodes_OP_DuplciateDeformerIConnections(bpy.types.Operator):
    '''All of the 'Mask Groups' of the selected deformer will be connected to the duplicated deformer'''
    bl_label="Reuse 'Mask Groups'"
    bl_idname='medusanodes.duplicate_deformer_inh'
    bl_options = {'UNDO'}

    def execute(self, context):
        activeNode = bpy.context.scene.medusaNodesTree[bpy.context.scene.medusaNodesTree_index]
        if check_valid(activeNode):       
            geoNodes = bpy.data.objects[activeNode.objectName].modifiers[activeNode.nodeTreeName]
            node = geoNodes.node_group.nodes[activeNode.name]

            if 'generator'in node.node_tree.name.lower() or 'visualizer' in node.node_tree.name.lower():
                message = f"Duplicating the base nodes like '{node.node_tree.name[3:]}' is not possible!"
                self.report({'INFO'}, message)   
            else:
                inheritConnections(geoNodes,node)

        return {'FINISHED'}

class MedusaNodesDupDeformer(bpy.types.Operator):
    bl_idname = "medusanodes.dup_deformer"
    bl_label = "Duplicate Deformer"
    bl_options = {'UNDO'}
    
    def execute(self, context):
        bpy.ops.medusanodes.duplicate_deformer("INVOKE_DEFAULT")
        return {'FINISHED'}

class MedusaNodesResDeformer(bpy.types.Operator):
    '''Reset the selected deformer. The operator will reset deformer's parameters to their default values'''
    bl_idname = "medusanodes.reset_deformer"
    bl_label = "Reset Deformer"
    bl_options = {'UNDO'}
    
    def execute(self, context):
        item_index = bpy.context.scene.medusaNodesTree_index
        item_list = bpy.context.scene.medusaNodesTree
        if len(item_list)>item_index and item_index != -1:
            activeNode = item_list[item_index]
            if check_valid(activeNode):
                geoNodes = bpy.data.objects[activeNode.objectName].modifiers[activeNode.nodeTreeName]
                node = geoNodes.node_group.nodes[activeNode.name]

                if hasattr(node.node_tree,"inputs"):
                    nodInp = node.node_tree.inputs
                    if node.type == "GROUP":
                        for input,nt_input in zip(node.inputs,nodInp):
                            if hasattr(input, "default_value") and input.type != "MATERIAL" and not input.is_linked:
                                input.default_value = nt_input.default_value
                else:
                    nodInp = node.node_tree.interface.items_tree
                    if node.type == "GROUP":
                        for input in node.inputs:
                            for nt_input in nodInp:
                                if input.name == nt_input.name:
                                    if hasattr(input, "default_value") and input.type != "MATERIAL" and not input.is_linked:
                                        input.default_value = nt_input.default_value
                                    break

        return {'FINISHED'}

class MedusaNodesApplyAllDeformers(bpy.types.Operator):
    '''Apply all deformers. Freeze the 'Generator' to be able to apply the deformers'''
    bl_idname = "medusanodes.apply_all"
    bl_label = "Apply All Deformers"
    bl_options = {'UNDO'}
    
    def execute(self, context):
        item_list = bpy.context.scene.medusaNodesTree
        item_index = bpy.context.scene.medusaNodesTree_index
        if len(item_list)>1:
            firstNode = item_list[0]
            obj = None
            nodeTreeName = None
            for node in item_list:
                temOb = bpy.data.objects.get(node.objectName)
                if temOb:
                    if "MedusaGroom" in temOb:
                        obj = temOb
                        nodeTreeName = node.nodeTreeName
            if obj is None:
                obj = bpy.data.objects[firstNode.objectName]
                nodeTreeName = firstNode.nodeTreeName

            # activeNode = item_list[item_index]
            # obj = bpy.data.objects[activeNode.objectName]
            originalMod = obj.modifiers[nodeTreeName]
            geoNodes = originalMod.node_group
            # node = geoNodes.nodes[activeNode.name]

            selectedObjectName = bpy.context.object.name

            bpy.ops.object.mode_set(mode="OBJECT")
            bpy.ops.object.select_all(action="DESELECT")
            bpy.context.view_layer.objects.active = obj
            obj.select_set(True)

            duplicateModifier = obj.modifiers.new(name = originalMod.name + '_Temp', type = originalMod.type)
            nodeGroupCopy = geoNodes.copy()
            duplicateModifier.node_group = nodeGroupCopy

            for inp in geoNodes.interface.items_tree:
                try:
                    duplicateModifier[inp.identifier] = originalMod[inp.identifier]
                except:
                    pass

            lastNode = item_list[1]
            # Connect to group output
            groupOutNode = None
            nodeToBake = None
            for node in nodeGroupCopy.nodes:
                if node.type == "GROUP":
                    if node.name == lastNode.name:
                        nodeToBake = node
                if node.type == "GROUP_OUTPUT":
                    groupOutNode = node
            
            if nodeToBake:
                for out in nodeToBake.outputs:
                    if out.name == "Splines":
                        nodeGroupCopy.links.new(groupOutNode.inputs[0],out)
            bpy.ops.object.modifier_move_to_index(modifier=duplicateModifier.name,index=0)
            bpy.ops.object.modifier_apply(modifier = duplicateModifier.name)
            bpy.data.node_groups.remove(nodeGroupCopy)
            for node_hie in item_list:
                try:
                    activeNode = geoNodes.nodes[node_hie.name]
                    
                    if "mn visual" not in activeNode.node_tree.name.lower() and "mn generator" not in activeNode.node_tree.name.lower():
                        nodesOutConnectWith = {}
                        nodesInpConnectWith = {}
                        for out in activeNode.outputs:
                            if out.is_linked:
                                for link in out.links:
                                    node = link.to_node
                                    if node.type == "GROUP":
                                        nodesOutConnectWith[link.from_socket]=link.to_socket
                        for inp in activeNode.inputs:
                            if inp.is_linked:
                                for link in inp.links:
                                    node = link.from_node
                                    nodesInpConnectWith[link.to_socket]=link.from_socket
                        processed = []
                        for conx in nodesOutConnectWith:
                            for conxi in nodesInpConnectWith:
                                if conxi.name == conx.name:
                                    geoNodes.links.new(nodesInpConnectWith[conxi],nodesOutConnectWith[conx])
                                    processed.append(nodesInpConnectWith[conxi])
                                    processed.append(nodesOutConnectWith[conx])

                        for conx in nodesInpConnectWith:
                            if nodesInpConnectWith[conx] not in processed:
                                if(len(nodesInpConnectWith[conx].links)) == 1:
                                    if nodesInpConnectWith[conx].node.type != "GROUP":
                                        geoNodes.nodes.remove(nodesInpConnectWith[conx].node)
                                    else:
                                        if "mask group" in nodesInpConnectWith[conx].node.node_tree.name.lower() and "mn " in nodesInpConnectWith[conx].node.node_tree.name.lower():
                                            geoNodes.nodes.remove(nodesInpConnectWith[conx].node)
                                        elif "mn" not in nodesInpConnectWith[conx].node.node_tree.name.lower():
                                            geoNodes.nodes.remove(nodesInpConnectWith[conx].node)
                        
                        try:
                            geoNodes.nodes.remove(activeNode)
                        except:
                            pass
                except:
                    pass
            SetupNodeData()
            
            bpy.ops.object.select_all(action="DESELECT")
            bpy.context.view_layer.objects.active = bpy.data.objects[selectedObjectName]
            bpy.data.objects[selectedObjectName].select_set(True)

        return {'FINISHED'}

class MedusaNodesApplyDeformer(bpy.types.Operator):
    '''Apply selected deformer. Freeze the 'Generator' to be able to apply the deformer'''
    bl_idname = "medusanodes.apply_deformer"
    bl_label = "Apply Selected Deformer"
    bl_options = {'UNDO'}
    
    def execute(self, context):
        item_index = bpy.context.scene.medusaNodesTree_index
        item_list = bpy.context.scene.medusaNodesTree
        if len(item_list)>item_index and item_index != -1:
            activeNode = item_list[item_index]
            obj = bpy.data.objects[activeNode.objectName]
            originalMod = obj.modifiers[activeNode.nodeTreeName]
            geoNodes = originalMod.node_group
            node = geoNodes.nodes[activeNode.name]

            selectedObjectName = bpy.context.object.name

            bpy.ops.object.mode_set(mode="OBJECT")
            bpy.ops.object.select_all(action="DESELECT")
            bpy.context.view_layer.objects.active = obj
            obj.select_set(True)

            duplicateModifier = obj.modifiers.new(name = originalMod.name + '_Temp', type = originalMod.type)
            duplicateModifier.node_group = geoNodes

            for inp in geoNodes.interface.items_tree:
                try:
                    duplicateModifier[inp.identifier] = originalMod[inp.identifier]
                except:
                    pass

            enabeledList = []
            nodeToBake = None
            groupOutNode = None
            previousToSocket = None
            for node in geoNodes.nodes:
                if node.type == "GROUP":
                    if node.name == activeNode.name:
                        nodeToBake = node
                if node.type == "GROUP_OUTPUT":
                    groupOutNode = node
                    previousToSocket= node.inputs[0].links[0].from_socket

            for node in geoNodes.nodes:
                if node != nodeToBake and node.type == "GROUP":
                    for inp in node.inputs:
                        if inp.name.lower() == "disable":
                            enabeledList.append(inp.default_value)
                            inp.default_value = True

            for out in nodeToBake.outputs:
                if out.name == "Splines":
                    geoNodes.links.new(groupOutNode.inputs[0], out)
                    bpy.ops.object.modifier_apply(modifier = duplicateModifier.name)
                    geoNodes.links.new(previousToSocket, groupOutNode.inputs[0])
                    bpy.ops.medusanodes.delete_deformer()
            i=0
            for node in geoNodes.nodes:
                if node.type == "GROUP":
                    for inp in node.inputs:
                        if inp.name.lower() == "disable":
                            inp.default_value = enabeledList[i]
                            i+=1

            bpy.ops.object.select_all(action="DESELECT")
            bpy.context.view_layer.objects.active = bpy.data.objects[selectedObjectName]
            bpy.data.objects[selectedObjectName].select_set(True)

        return {'FINISHED'}


class MedusaNodesCache(bpy.types.Operator):
    """Used to freeze or unfreeze distributed hair. Freezing the distributed hair speeds up deformation calculations"""
    bl_idname = "medusanodes.cache_node"
    bl_label = "Freeze"
    bl_options = {'UNDO'}

    objectName : StringProperty()
    nodeTreeName: StringProperty()
    nodeName: StringProperty()
    socketName: StringProperty()

    @classmethod
    def description(cls, context, properties):
        obj = bpy.data.objects[properties.objectName]
        originalMod = obj.modifiers[properties.nodeTreeName]
        geoNodes = originalMod.node_group
        node = geoNodes.nodes[properties.nodeName]
        if "mn generator" in node.node_tree.name.lower():
            return "Used to freeze or unfreeze distributed hair. Freezing the distributed hair speeds up deformation calculations"
        else:
            return "Freezing the deformer makes it faster. The 'Generator' should be frozen first"

    def execute(self, context):
        obj = bpy.data.objects[self.objectName]
        originalMod = obj.modifiers[self.nodeTreeName]
        geoNodes = originalMod.node_group
        socket = geoNodes.nodes[self.nodeName].inputs[self.socketName]
        selectedObjectName = bpy.context.object.name
        #should only cache if the object has any splines
        spline_count, point_count = getHairCount(obj)
        if "tempCache" in bpy.data.hair_curves:
            bpy.data.hair_curves.remove(bpy.data.hair_curves["tempCache"])
        if spline_count>0:
            if socket.default_value == False:
                oldVis = bpy.context.object.hide_get()
                oldVie = bpy.context.object.hide_viewport
                oldSel = bpy.context.object.select_get()
                bpy.context.object.hide_set(False)
                bpy.context.object.hide_viewport = False
                lstMode = bpy.context.object.mode
                try:
                    bpy.ops.object.mode_set(mode="OBJECT")
                except:
                    pass
                bpy.ops.object.select_all(action="DESELECT")
                bpy.context.view_layer.objects.active = obj
                obj.select_set(True)
                bpy.ops.object.mode_set(mode="OBJECT")

                duplicateModifier = obj.modifiers.new(name = originalMod.name + '_Temp', type = originalMod.type)
                nodeGroupCopy = geoNodes.copy()
                duplicateModifier.node_group = nodeGroupCopy

                for inp in geoNodes.interface.items_tree:
                    try:
                        duplicateModifier[inp.identifier] = originalMod[inp.identifier]
                    except:
                        pass

                nodeToBake = None
                VisualNodemat = None
                groupOutNode = None

                # Connect to group output
                for node in nodeGroupCopy.nodes:
                    if node.type == "GROUP":
                        if node.name == self.nodeName:
                            nodeToBake = node
                            for inp in node.inputs:
                                if inp.name.lower() == "mask":
                                    if not inp.is_linked:
                                        inp.default_value = 0
                                    else:
                                        for link in inp.links:
                                            nodeGroupCopy.links.remove(link)
                                    inp.default_value = 0
                    if node.type == "GROUP_OUTPUT":
                        groupOutNode = node
                
                for node in nodeGroupCopy.nodes:
                    if node != nodeToBake and node.type == "GROUP":
                        for inp in node.inputs:
                            if inp.name.lower() == "disable":
                                inp.default_value = True
                        if "mn visualizer" in node.node_tree.name.lower():
                            VisualNodemat = node.inputs["Hair Material"].default_value

                for out in nodeToBake.outputs:
                    if out.name == "Splines":
                        nodeGroupCopy.links.new(groupOutNode.inputs[0],out)
                
                bpy.ops.object.modifier_apply(modifier = duplicateModifier.name)
                bpy.data.node_groups.remove(nodeGroupCopy)
                if VisualNodemat:
                    obj.data.materials.append(VisualNodemat)
                
                bpy.ops.object.select_all(action="DESELECT")
                bpy.context.view_layer.objects.active = bpy.data.objects[selectedObjectName]
                bpy.data.objects[selectedObjectName].select_set(oldSel)
                bpy.ops.object.mode_set(mode=lstMode)
                bpy.data.objects[selectedObjectName].hide_set(oldVis)
                bpy.data.objects[selectedObjectName].hide_viewport = oldVie
            else:
                node = geoNodes.nodes[self.nodeName]
                if "mn generator" in node.node_tree.name.lower():
                    dataName = obj.data.name
                    obj.data.name = "tempCache"
                    obj.data = bpy.data.hair_curves.new(dataName)
                    obj.data.surface = bpy.data.hair_curves["tempCache"].surface
                    obj.data.surface_uv_map = bpy.data.hair_curves["tempCache"].surface_uv_map

                    VisualNodemat = None
                    for nodeG in geoNodes.nodes:
                        if nodeG != node and node.type == "GROUP":
                            for inp in nodeG.inputs:
                                if inp.name.lower() == "cached":
                                    inp.default_value = False
                                    break
                            if nodeG.type == "GROUP":
                                if "mn visualizer" in nodeG.node_tree.name.lower():
                                    VisualNodemat = nodeG.inputs["Hair Material"].default_value
                    if VisualNodemat:
                        obj.data.materials.append(VisualNodemat)
            socket.default_value = not socket.default_value
            if "tempCache" in bpy.data.hair_curves:
                bpy.data.hair_curves.remove(bpy.data.hair_curves["tempCache"])
        else:
            message=f"'{geoNodes.nodes[self.nodeName].label}' has no splines to freeze! Increase the 'Density'."
            self.report({'WARNING'}, message)
        return {'FINISHED'}

class MedusaNodesDeleteDeformer(bpy.types.Operator):
    '''Delete selected deformer'''
    bl_idname = "medusanodes.delete_deformer"
    bl_label = "Delete Deformer"
    bl_options = {'UNDO'}
    
    def execute(self, context):
        global needToUpdate
        global subPanels
        subPanels = []
        item_index = context.scene.medusaNodesTree_index
        item_list = context.scene.medusaNodesTree
        if len(item_list)>item_index and item_index != -1:
            item = item_list[item_index]
            if check_valid(item):
                geoNodes = bpy.data.objects[item.objectName].modifiers[item.nodeTreeName]
                if geoNodes:
                    if geoNodes.type == 'NODES':
                        activeNode = geoNodes.node_group.nodes[item.name]
                        if "mn visual" not in activeNode.node_tree.name.lower() and "mn generator" not in activeNode.node_tree.name.lower():
                            nodesOutConnectWith = {}
                            nodesInpConnectWith = {}
                            for out in activeNode.outputs:
                                if out.is_linked:
                                    for link in out.links:
                                        node = link.to_node
                                        if node.type == "GROUP":
                                            nodesOutConnectWith[link.from_socket]=link.to_socket
                            for inp in activeNode.inputs:
                                if inp.is_linked:
                                    for link in inp.links:
                                        node = link.from_node
                                        nodesInpConnectWith[link.to_socket]=link.from_socket
                            processed = []
                            needToUpdate = False
                            groupInpToRemove = None
                            for conx in nodesOutConnectWith:
                                for conxi in nodesInpConnectWith:
                                    if conxi.name == conx.name:
                                        geoNodes.node_group.links.new(nodesInpConnectWith[conxi],nodesOutConnectWith[conx])
                                        processed.append(nodesInpConnectWith[conxi])
                                        processed.append(nodesOutConnectWith[conx])
                            for conx in nodesInpConnectWith:
                                if nodesInpConnectWith[conx] not in processed:
                                    if nodesInpConnectWith[conx].node.type == "OBJECT_INFO":
                                        objInfoNode = nodesInpConnectWith[conx].node
                                        for obInLnk in objInfoNode.inputs["Object"].links:
                                            if obInLnk.from_node.type == "GROUP_INPUT":
                                                if len(obInLnk.from_socket.links)==1:
                                                    groupInpToRemove = obInLnk.from_socket
                                                    break
                                    if(len(nodesInpConnectWith[conx].links)) == 1:
                                        if nodesInpConnectWith[conx].node.type != "GROUP":
                                            geoNodes.node_group.nodes.remove(nodesInpConnectWith[conx].node)
                                        else:
                                            if "mask group" in nodesInpConnectWith[conx].node.node_tree.name.lower() and "mn " in nodesInpConnectWith[conx].node.node_tree.name.lower():
                                                geoNodes.node_group.nodes.remove(nodesInpConnectWith[conx].node)
                                            elif "mn" not in nodesInpConnectWith[conx].node.node_tree.name.lower():
                                                geoNodes.node_group.nodes.remove(nodesInpConnectWith[conx].node)
                            geoNodes.node_group.nodes.remove(activeNode)
                            if groupInpToRemove:
                                try:
                                    geoNodes.node_group.interface.remove(geoNodes.node_group.interface.items_tree[groupInpToRemove.name])
                                except:
                                    pass
                            SetupNodeData()
                            needToUpdate = True


        return {'FINISHED'}

class MedusaNodeTreeDeformersMenu(bpy.types.Menu):
    '''Add a new deformer above the selected one'''
    bl_label = "Add Deformer"
    bl_idname= "MEDUSANODE_MT_deformers_menu"
    
    def draw(self, context):
        layout = self.layout
        layout.label(text="Deformers",icon="NODE")
        layout.operator("medusanodes.add_node",text="Guide", icon_value=pcoll["guide"].icon_id).nodeName = "MN Guide"
        layout.operator("medusanodes.add_node",text="Clump", icon_value=pcoll["clump"].icon_id).nodeName = "MN Clump"
        layout.operator("medusanodes.add_node",text="Braid",icon_value=pcoll["braid"].icon_id).nodeName = "MN Braid"
        layout.operator("medusanodes.add_node",text="Noise",icon_value=pcoll["noise"].icon_id).nodeName = "MN Noise"
        layout.operator("medusanodes.add_node",text="Curl", icon_value=pcoll["curl"].icon_id).nodeName = "MN Curl"
        layout.operator("medusanodes.add_node",text="Trim", icon_value=pcoll["trim"].icon_id).nodeName = "MN Trim"
        layout.separator(factor=1.0)
        layout.label(text="Utility",icon="NODE")
        layout.operator("medusanodes.add_node",text="Curves to Splines",icon_value=pcoll["curves to splines"].icon_id).nodeName = "MN Curves to Splines"
        layout.operator("medusanodes.add_node",text="Mirror",icon_value=pcoll["mirror"].icon_id).nodeName = "MN Mirror"
        layout.operator("medusanodes.add_node",text="Scale",icon_value=pcoll["scale"].icon_id).nodeName = "MN Scale"
        layout.operator("medusanodes.add_node",text="Resample Hair",icon_value=pcoll["resample hair"].icon_id).nodeName = "MN Resample Hair"
        layout.operator("medusanodes.add_node",text="Children",icon_value=pcoll["children"].icon_id).nodeName = "MN Children"
        layout.separator(factor=1.0)
        layout.label(text="Simulation (Experimental)",icon="NODE")
        layout.operator("medusanodes.add_node",text="Cache",icon_value=pcoll["cache"].icon_id).nodeName = "MN Cache"
        layout.operator("medusanodes.add_node",text="Dynamics",icon_value=pcoll["dynamics"].icon_id).nodeName = "MN Dynamics"
    

def setupGuide(hairOb,guideOb,scalpOb):
    guideOb.modifiers.clear()
    if len(bpy.data.node_groups)>0:
        grp = bpy.data.node_groups[-1]
        if "surface deform" in grp.name.lower():
            if grp.users == 0:
                bpy.data.node_groups.remove(grp)
    guideOb["MedusaHair"] = hairOb
    nodMod = guideOb.modifiers.new("MedusaNodes","NODES")
    nodeModTree = bpy.data.node_groups.get(guideOb.name)
    if nodeModTree:
        if nodeModTree.users == 0:
            bpy.data.node_groups.remove(nodeModTree)
        elif nodeModTree.users == 1 and nodeModTree.use_fake_user:
            bpy.data.node_groups.remove(nodeModTree)
    nodeModTree = bpy.data.node_groups.new(guideOb.name,"GeometryNodeTree")
    nodeModTree.use_fake_user = True
    nodMod.node_group = nodeModTree

    splineGenNode = addMedusaNode(nodeModTree,"MN Generator")
    nodeColor(splineGenNode)
    VisualNode = addMedusaNode(nodeModTree,"MN Visualizer Guide")
    nodeColor(VisualNode)
    addMedusaProfileNode(nodeModTree,VisualNode)

    scalpObPolygons=scalpOb.data.polygons
    #calculate geometry area
    area = 0.0
    for p in scalpObPolygons:
        area += p.area
    areaFactor = 1 / area

    #calculate dimensions   
    dimensions = scalpOb.dimensions
    dimensions = dimensions[0]+dimensions[1]+dimensions[2]

    hairNodeData = {
        "Spline Length": dimensions * 0.1,
        "Hair Thickness": dimensions * 0.0025,
        "Density": areaFactor * 50,
        "Seed": 1,
    }

    for inp in hairNodeData:
        for nodeInp in splineGenNode.inputs:
            if nodeInp.name.lower() == inp.lower():
                nodeInp.default_value = hairNodeData[inp]
        for nodeInp in VisualNode.inputs:
            if nodeInp.name.lower() == inp.lower():
                nodeInp.default_value = hairNodeData[inp]
                break

    if splineGenNode and VisualNode:
        VisualNode.inputs[0].default_value = False
        groupInput = nodeModTree.nodes.new("NodeGroupInput")
        groupInput.location.x = -200
        groupOutput = nodeModTree.nodes.new("NodeGroupOutput")
        groupOutput.location.x = 700
        objectInfo = nodeModTree.nodes.new("GeometryNodeObjectInfo")
        objectInfo.location.x = groupInput.width + groupInput.location.x + 20
        objectInfo.location.y = groupInput.location.y
        objectInfo.inputs[0].default_value = scalpOb
        splineGenNode.location.x = 120
        VisualNode.location.x = 400

        #nodeModTree.outputs.new(VisualNode.outputs["Hair"].bl_idname,"Hair")
        #nodeModTree.inputs.new(splineGenNode.inputs["Splines"].bl_idname,"Splines")
        #nodeModTree.inputs.new(objectInfo.inputs[0].bl_idname,"Emitter Object")
        if "Hairs" in VisualNode.outputs:
            nodeModTree.interface.new_socket(name ='Hairs', description ='', in_out ='OUTPUT', socket_type=VisualNode.outputs["Hairs"].bl_idname, parent= None)
        else:
            nodeModTree.interface.new_socket(name ='Hair', description ='', in_out ='OUTPUT', socket_type=VisualNode.outputs["Hair"].bl_idname, parent= None)
        nodeModTree.interface.new_socket(name ='Splines', description ='', in_out ='INPUT', socket_type=splineGenNode.inputs["Splines"].bl_idname, parent= None)
        nodeModTree.interface.new_socket(name ='Emitter Object', description ='', in_out ='INPUT', socket_type=objectInfo.inputs[0].bl_idname, parent= None)

        nodeModTree.links.new(splineGenNode.inputs["Splines"],groupInput.outputs[0])
        nodeModTree.links.new(splineGenNode.inputs["Emitter Geometry"],objectInfo.outputs["Geometry"])
        nodeModTree.links.new(VisualNode.inputs["Splines"],splineGenNode.outputs["Splines"])
        nodeModTree.links.new(VisualNode.inputs["Emitter Geometry"],splineGenNode.outputs["Emitter Geometry"])
        if "Hairs" in VisualNode.outputs:
            nodeModTree.links.new(groupOutput.inputs[0],VisualNode.outputs["Hairs"])
        else:
            nodeModTree.links.new(groupOutput.inputs[0],VisualNode.outputs["Hair"])
        nodeModTree.links.new(objectInfo.inputs[0],groupInput.outputs[1])

        nodeModTree.interface.items_tree[2].name = "Emitter Object"
        groupInput.outputs[1].name = "Emitter Object"

        for gInp in nodMod.node_group.interface.items_tree:
            if gInp.name.lower() == "emitter object":
                nodMod[gInp.identifier] = scalpOb
        mat = VisualNode.inputs["Hair Material"].default_value

        if mat:
            mat.name = guideOb.name
        guideOb.data.materials.append(mat)
        hSplineNode = None
        for node in hairOb.modifiers.active.node_group.nodes:
            if node.type == "GROUP":
                if "generator" in node.node_tree.name.lower():
                    hSplineNode = node
                    break
        if hSplineNode:
            d = splineGenNode.inputs["Control Point Count"].driver_add("default_value")
            var1 = d.driver.variables.new()
            var1.name = "AVERAGE"
            var1.targets[0].id_type = 'NODETREE'
            var1.targets[0].id = hairOb.modifiers.active.node_group
            var1.targets[0].data_path = 'nodes["'+hSplineNode.name+'"].inputs["Control Point Count"].default_value'
            d.driver.expression = var1.name

    nodMod.show_viewport = False
    nodMod.show_viewport = True
    alignNodes(nodeModTree)

def setupHairOb(hairOb,scalpOb,hairNodeData):
    hairOb.modifiers.clear()
    if len(bpy.data.node_groups)>0:
        grp = bpy.data.node_groups[-1]
        if "surface deform" in grp.name.lower():
            if grp.users == 0:
                bpy.data.node_groups.remove(grp)
    nodMod = hairOb.modifiers.new("MedusaNodes","NODES")
    nodeModTree = bpy.data.node_groups.get(hairOb.name)

    #nodeModTree.inputs.new('NodeSocketGeometry', "Splines")
    #nodeModTree.outputs.new('NodeSocketGeometry', "Hair")

    if nodeModTree:
        bpy.data.node_groups.remove(nodeModTree)
    nodeModTree = bpy.data.node_groups.new(hairOb.name,"GeometryNodeTree")
    nodeModTree.use_fake_user = True
    nodMod.node_group = nodeModTree

    

    splineGenNode = addMedusaNode(nodeModTree,"MN Generator")
    nodeColor(splineGenNode)
    VisualNode = addMedusaNode(nodeModTree,"MN Visualizer")
    nodeColor(VisualNode)
    addMedusaProfileNode(nodeModTree,VisualNode)

    for inp in hairNodeData:
        for nodeInp in splineGenNode.inputs:
            if nodeInp.name.lower() == inp.lower():
                nodeInp.default_value = hairNodeData[inp]
        for nodeInp in VisualNode.inputs:
            if nodeInp.name.lower() == inp.lower():
                nodeInp.default_value = hairNodeData[inp]
                break

    if splineGenNode and VisualNode:
        groupInput = nodeModTree.nodes.new("NodeGroupInput")
        groupInput.location.x = -200
        groupOutput = nodeModTree.nodes.new("NodeGroupOutput")
        groupOutput.location.x = 700
        objectInfo = nodeModTree.nodes.new("GeometryNodeObjectInfo")
        objectInfo.location.x = objectInfo.width + groupInput.location.x + 20
        objectInfo.location.y = groupInput.location.y
        objectInfo.inputs[0].default_value = scalpOb
        splineGenNode.location.x = 120
        VisualNode.location.x = 400

        #Blender 3.6
        # nodeModTree.outputs.new(VisualNode.outputs["Hair"].bl_idname,"Hair")
        # nodeModTree.inputs.new(splineGenNode.inputs["Splines"].bl_idname,"Splines")
        # nodeModTree.inputs.new(objectInfo.inputs[0].bl_idname,"Emitter Object")

        #Blender 4.0 requires interface method "new_socket" because of the new socket groupping functionality.
        if "Hairs" in VisualNode.outputs:
            nodeModTree.interface.new_socket(name ='Hairs', description ='', in_out ='OUTPUT', socket_type=VisualNode.outputs["Hairs"].bl_idname, parent= None)
        else:
            nodeModTree.interface.new_socket(name ='Hair', description ='', in_out ='OUTPUT', socket_type=VisualNode.outputs["Hair"].bl_idname, parent= None)
        nodeModTree.interface.new_socket(name ='Splines', description ='', in_out ='INPUT', socket_type=splineGenNode.inputs["Splines"].bl_idname, parent= None)
        nodeModTree.interface.new_socket(name ='Emitter Object', description ='', in_out ='INPUT', socket_type=objectInfo.inputs[0].bl_idname, parent= None)


        nodeModTree.links.new(splineGenNode.inputs["Splines"],groupInput.outputs[0])
        nodeModTree.links.new(splineGenNode.inputs["Emitter Geometry"],objectInfo.outputs["Geometry"])
        nodeModTree.links.new(VisualNode.inputs["Splines"],splineGenNode.outputs["Splines"])
        nodeModTree.links.new(VisualNode.inputs["Emitter Geometry"],splineGenNode.outputs["Emitter Geometry"])
        if "Hairs" in VisualNode.outputs:
            nodeModTree.links.new(groupOutput.inputs[0],VisualNode.outputs["Hairs"])
        else:
            nodeModTree.links.new(groupOutput.inputs[0],VisualNode.outputs["Hair"])
        nodeModTree.links.new(objectInfo.inputs[0],groupInput.outputs[1])

        groupInput.outputs[1].name = "Emitter Object"#Blender 3.6

        for gInp in nodMod.node_group.interface.items_tree:
            if gInp.name.lower() == "emitter object":
                nodMod[gInp.identifier] = scalpOb
        
        mat = VisualNode.inputs["Hair Material"].default_value
        if mat:
            mat.name = hairOb.name
        hairOb.data.materials.append(mat)
        hairOb["MedusaGroom"] = True
    alignNodes(nodeModTree)

def createGuidCurve(hairObject,scalpOb):
    lstMode = bpy.context.object.mode
    bpy.ops.object.mode_set(mode="OBJECT")
    bpy.ops.object.select_all(action="DESELECT")
    bpy.context.view_layer.objects.active = scalpOb

    scalpObDisabled = scalpOb.hide_viewport
    scalpObHidden = scalpOb.hide_get()

    scalpOb.hide_set(False)
    scalpOb.hide_viewport = False

    bpy.ops.object.curves_empty_hair_add(align='WORLD', scale=(1, 1, 1))
    
    scalpOb.hide_set(scalpObHidden)
    scalpOb.hide_viewport = scalpObDisabled
    guideOb = bpy.context.object
    for col in guideOb.users_collection:
        try:
            col.objects.unlink(guideOb)
        except Exception as E:
            pass
    for col in hairObject.users_collection:
        try:
            col.objects.link(guideOb)
        except Exception as E:
            pass

    guideName = hairObject.name+"_Guide"
    
    guideCpt = 0
    guideObCpt = 0
    for ob in bpy.data.objects:
        if ob.type == "CURVES":
            if guideName in ob.name:
                guideObCpt = guideObCpt + 1
                nameParts = ob.name.split("_")
                if len(nameParts)>0:
                    if nameParts[-1].lower() != "guide":
                        namePrt = nameParts[-1][-2:]
                        if namePrt.isdigit():
                            if int(namePrt)>guideCpt:
                                guideCpt = int(namePrt)
    
    if guideObCpt > 0:
        guideStrInde = str(guideCpt+1).zfill(2)
        if guideName[-2:] != guideStrInde and not guideName[-2:].isdigit():
            guideName = guideName+guideStrInde
    guideOb.name = guideName
    if "_Guide" not in guideOb.name:
        guideOb.name = guideOb.name + "_Guide"

    setupGuide(hairObject,guideOb,scalpOb)
    bpy.ops.object.mode_set(mode=lstMode)
    return guideOb

class MedusaNodesCreateGuiCurve(bpy.types.Operator):
    '''Create new 'Guide' object, the older ones will remain in the outliner'''
    bl_idname = "medusanodes.create_guide_curves"
    bl_label = "Create Guide Curves"
    bl_options = {'UNDO'}

    index : IntProperty(default=0)
    
    def execute(self, context):
        item_index = self.index
        item_list = context.scene.medusaNodesTree
        allitemsLists = context.scene.medusaNodes
        if len(item_list)>item_index:
            item = item_list[item_index]
        else:
            item = allitemsLists[item_index]
        if check_valid(item):
            geoNodes = bpy.data.objects[item.objectName].modifiers[item.nodeTreeName]
            if geoNodes:
                if geoNodes.type == 'NODES':
                    lstMode = bpy.context.object.mode
                    currOb = bpy.context.object
                    oldVisCurr = currOb.hide_get()
                    oldVieCurr = currOb.hide_viewport
                    currOb.hide_set(False)
                    currOb.hide_viewport = False
                    bpy.ops.object.mode_set(mode="OBJECT")
                    activeNode = geoNodes.node_group.nodes[item.name]
                    nodeLinks = geoNodes.node_group.links
                    for gInp in geoNodes.node_group.interface.items_tree:
                        if gInp.name.lower() == "emitter object" or gInp.name.lower() == "emitter geometry":
                            scalpOb = geoNodes[gInp.identifier]
                            if scalpOb:
                                contextObjectHiddenState = scalpOb.hide_get()
                                if contextObjectHiddenState:
                                    scalpOb.hide_set(False)
                                    scalpOb.hide_viewport = False
                                    scalpOb.select_set(True) 
                                else:
                                    pass
 
                                hairOb = bpy.data.objects[item.objectName]
                                guideOb = createGuidCurve(hairOb,scalpOb)
                                guideInpName = guideOb.name.split("_")[-1]
                                bpy.ops.object.select_all(action="DESELECT")
                                bpy.context.view_layer.objects.active = hairOb
                                hairOb.select_set(True)
                                objectInfo = None
                                for node in geoNodes.node_group.nodes:
                                    if node.type == "OBJECT_INFO" and node.name == activeNode.name+"_ObjectInfo":
                                        objectInfo = node
                                        break
                                guideInp = geoNodes.node_group.interface.items_tree.get(guideInpName)
                                for node in geoNodes.node_group.nodes:
                                    if node.type == "GROUP_INPUT":
                                        if objectInfo is None:
                                            objectInfo = geoNodes.node_group.nodes.new("GeometryNodeObjectInfo")
                                            objectInfo.name = activeNode.name+"_ObjectInfo"
                                            objectInfo.location.x = node.location.x
                                            objectInfo.location.y = node.location.y
                                            node.location.x = node.location.x - 200
                                            if guideInp:
                                                if len(node.outputs[guideInp.index-1].links)>0:
                                                    guideInpName = guideInpName + "_01"
                                                    guideInp = None
                                        objectInfo.inputs[0].default_value = guideOb
                                        if guideInp is None:
                                            geoNodes.node_group.interface.new_socket(name = guideInpName, description ='', in_out ='INPUT', socket_type=objectInfo.inputs[0].bl_idname, parent= None)
                                            guideInp = geoNodes.node_group.interface.items_tree.get(guideInpName)
                                        nodeLinks.new(objectInfo.inputs[0],node.outputs[guideInp.index-1])
                                        node.outputs[guideInp.index-1].name = guideInpName
                                        for gInp in geoNodes.node_group.interface.items_tree:
                                            if gInp.name.lower() == guideInpName.lower():
                                                if geoNodes[gInp.identifier] is None:
                                                    geoNodes[gInp.identifier] = guideOb

                                        for input in activeNode.inputs:
                                            if input.name.lower() == "guide curves":
                                                nodeLinks.new(input,objectInfo.outputs["Geometry"])
                                                break
                                        break
                            scalpOb.hide_set(contextObjectHiddenState)
                            break
                    
                    bpy.ops.object.mode_set(mode=lstMode)
                    currOb.hide_set(oldVisCurr)
                    currOb.hide_viewport = oldVieCurr
        return {'FINISHED'}

class MedusaNodesAddExsGuiCurve(bpy.types.Operator):
    '''Choose an existing 'Guide' object in the scene, and connect it to the deformer'''
    bl_idname = "medusanodes.add_existing_guides"
    bl_label = "Choose Existing 'Guide' Object"
    bl_options = {'UNDO'}

    index : IntProperty(default=0)

    @classmethod
    def poll(cls, context):
        return True

    def execute(self, context):
        if context.scene.medusaGuidePicker:
            for mod in context.object.modifiers:
                if mod.type == "NODES" and mod.name == "MedusaNodes":
                    item_index = self.index
                    item_list = context.scene.medusaNodesTree
                    allitemsLists = context.scene.medusaNodes
                    if len(item_list)>item_index:
                        item = item_list[item_index]
                    else:
                        item = allitemsLists[item_index]
                    activeNode = mod.node_group.nodes[item.name]
                    guideInpName = context.scene.medusaGuidePicker.name.split("_")[-1]
                    guideInp = mod.node_group.interface.items_tree.get(guideInpName)
                    objectInfo = None
                    for node in mod.node_group.nodes:
                        if node.type == "OBJECT_INFO" and node.name == activeNode.name+"_ObjectInfo":
                            objectInfo = node
                    for node in mod.node_group.nodes:
                        if node.type == "GROUP_INPUT":
                            if objectInfo is None:
                                objectInfo = mod.node_group.nodes.new("GeometryNodeObjectInfo")
                                objectInfo.name = activeNode.name+"_ObjectInfo"
                                #objectInfo.inputs[0].default_value = someobject! #GUIDE DEFORMER NOT UPDATING FIX
                                objectInfo.location.x = node.location.x
                                objectInfo.location.y = node.location.y
                                
                            if guideInp is None:
                                mod.node_group.interface.new_socket(name = guideInpName, description ='', in_out ='INPUT', socket_type=objectInfo.inputs[0].bl_idname, parent= None)
                                guideInp = mod.node_group.interface.items_tree.get(guideInpName)
                            mod.node_group.links.new(objectInfo.inputs[0],node.outputs[guideInp.index-1])
                            node.outputs[guideInp.index-1].name = guideInpName
                            
                            for gInp in mod.node_group.interface.items_tree:
                                # if gInp.name.lower() == "guide curves":
                                if gInp.name.lower() == guideInpName.lower():
                                    if mod[gInp.identifier] is None:
                                        mod[gInp.identifier] = context.scene.medusaGuidePicker
                                    else:
                                        try:
                                            if gInp.identifier == objectInfo.inputs["Object"].links[0].from_socket.identifier:
                                                mod[gInp.identifier] = context.scene.medusaGuidePicker
                                        except:
                                            pass
                            
                            for input in activeNode.inputs:
                                if input.name.lower() == "guide curves":
                                    mod.node_group.links.new(input,objectInfo.outputs["Geometry"])
                                    break
                            break
                    nodesUpdate(context.scene)
                    break
        return {'FINISHED'}

    def invoke(self, context, event):
        return context.window_manager.invoke_props_dialog(self)

    def draw(self, context):

        layout = self.layout
        layout.use_property_decorate = False
        #layout.separator(factor=1.5)
       
        row = layout.row()
        #row.alignment = 'CENTER'
        #row.label(text="Pick and Choose the Guide Object", icon_value=guidepcoll["visualizer"].icon_id) # ' icon = 'STRANDS''

        layout.separator(factor=0.5)

        row1 = layout.row(align = True)
        col = row1.column(align = True)

        row = col.row()
        row.separator(factor=2)
        row.prop(context.scene,"medusaGuidePicker", text = "'Guide' Object")
        row.separator(factor=2)
        layout.separator(factor=1)

class MedusaNodesResInhGuidCurve(bpy.types.Operator):
    '''Connects the Guide Curves from the underlying deformer'''
    bl_idname = "medusanodes.inherit_guide_curves"
    bl_label = "Inherit 'Guide' Object"
    bl_options = {'UNDO'}

    index : IntProperty(default=0)
    
    def execute(self, context):
        global processedNode
        item_index = self.index
        item_list = context.scene.medusaNodesTree
        item = item_list[item_index]
        if check_valid(item):
            geoNodes = bpy.data.objects[item.objectName].modifiers[item.nodeTreeName]
            if geoNodes:
                if geoNodes.type == 'NODES':
                    activeNode = geoNodes.node_group.nodes[item.name]
                    allNodes = followLinks(activeNode,inp=None)
                    processedNode = {}

                    for node in reversed(allNodes):
                        for subNode in allNodes[node]:
                            if subNode.type == "GROUP":
                                for out in subNode.outputs:
                                    if "guide" in out.name.lower():
                                        geoNodes.node_group.links.new(activeNode.inputs["Guide Curves"],out)
                                        break
        return {'FINISHED'}

class MedusaNodeTreeGuidesMenu(bpy.types.Operator):
    '''Add, choose, or inherit the 'Guide' object'''
    bl_label = "'Guide' Object's Options"
    bl_idname= "medusanodes.guides_menu"
    bl_options = {'UNDO'}

    index : IntProperty(default=0)

    enum_items =(('0', "Create 'Guide' Object", "Creates new 'Guide' object, the previous ones will remain in the outliner.", 'ADD',0),
            ('1', "Choose Existing 'Guide' Object", "Choose an existing 'Guide' object and connect it with this deformer" ,'EYEDROPPER',1),
            ('2', "Inherit 'Guide' Object", "Connect the 'Guide' object from the underlying deformer",'FILE_PARENT',2))
    guideItems : bpy.props.EnumProperty(name="Guides Curves Options", items=enum_items)

    @classmethod
    def description(cls, context, properties):
        items = {id: desc for id, name, *desc, in cls.enum_items}
        desc = items[properties.guideItems][0]
        return desc

    def execute(self, context):
        if(int(self.guideItems)==0):
            bpy.ops.medusanodes.create_guide_curves(index=self.index)
        elif(int(self.guideItems)==1):
            bpy.ops.medusanodes.add_existing_guides("INVOKE_DEFAULT",index=self.index)
        elif(int(self.guideItems)==2):
            bpy.ops.medusanodes.inherit_guide_curves(index=self.index)
        return{'FINISHED'}

class MedusaNodesCopyInputLinks(bpy.types.Operator):
    bl_idname = "medusanodes.copy_input_links"
    bl_label = "Copy Input Link"
    bl_options = {'UNDO'}
    
    def execute(self, context):
        print("Copy Input Link !")
        return {'FINISHED'}

class MedusaNodesPasteInputLinks(bpy.types.Operator):
    bl_idname = "medusanodes.paste_input_links"
    bl_label = "Paste Input Link"
    bl_options = {'UNDO'}
    
    def execute(self, context):
        print("Paste Input Link !")
        return {'FINISHED'}

class MedusaNodesCopyInputNodes(bpy.types.Operator):
    bl_idname = "medusanodes.copy_input_nodes"
    bl_label = "Copy Input Nodes"
    bl_options = {'UNDO'}
    
    def execute(self, context):
        print("Copy Input Nodes !")
        return {'FINISHED'}

class MedusaNodesPasteInputNodes(bpy.types.Operator):
    bl_idname = "medusanodes.paste_input_nodes"
    bl_label = "Paste Input Link"
    bl_options = {'UNDO'}
    
    def execute(self, context):
        print("Paste Input Link !")
        return {'FINISHED'}

class MedusaNodesCreateMatAtt(bpy.types.Operator):
    bl_idname = "medusanodes.create_material_attrr"
    bl_label = "Create Material Attribute"
    bl_options = {'UNDO'}

    def attChanged(self,context):
        global attrbData
        if len(attrbData)>0:
            atrrSocket = attrbData[1]
            geoNodes = attrbData[2]

            attrbData[0].attribute_name = self.matAtrribute
            atrrSocket.name = self.matAtrribute
            geoNodes[atrrSocket.identifier+'_attribute_name']  = self.matAtrribute

            item_index = context.scene.medusaNodesTree_index
            item_list = context.scene.medusaNodesTree
            if len(item_list)>item_index and item_index != -1:
                item = item_list[item_index]
                if check_valid(item):
                    geoNodes = bpy.data.objects[item.objectName].modifiers[item.nodeTreeName]
                    if geoNodes:
                        if geoNodes.type == 'NODES':
                            activeNode = geoNodes.node_group.nodes[item.name]
                            if len(activeNode.inputs[activeNodeGroupSocket].links)>0:
                                linkSocket = activeNode.inputs[activeNodeGroupSocket].links[0]
                                from_socket = linkSocket.from_socket
                                from_socket["attributeName"] = self.matAtrribute
                                from_socket["attributeOutNodeSock"] = self.matAtrribute

            attrbData = []

    matAtrribute : StringProperty(
        name="Attribute name",
        description="Attribute name",
        default = "",
        update=attChanged
    )

    def draw(self, context):
        layout = self.layout
        layout.use_property_decorate = False
        layout.label(text='Create Material Attribute')
        row = layout.row()
        row.prop(self, 'matAtrribute')
    
    def execute(self, context):
        return {'FINISHED'}
    
    def invoke(self,context,event):
        item_index = context.scene.medusaNodesTree_index
        item_list = context.scene.medusaNodesTree
        if len(item_list)>item_index and item_index != -1:
            item = item_list[item_index]
            if check_valid(item):
                geoNodes = bpy.data.objects[item.objectName].modifiers[item.nodeTreeName]
                if geoNodes:
                    if geoNodes.type == 'NODES':
                        activeNode = geoNodes.node_group.nodes[item.name]
                        fromNode = None
                        if len(activeNode.inputs[activeNodeGroupSocket].links)>0:
                            linkSocket = activeNode.inputs[activeNodeGroupSocket].links[0]
                            from_socket = linkSocket.from_socket
                            fromNode = linkSocket.from_node
                            
                            attriName = ""
                            if fromNode.type == "GROUP":
                                attriName = fromNode.node_tree.name
                            else:
                                attriName = fromNode.name

                            nodeOutput = None
                            nodeVisu   = None

                            for node in geoNodes.node_group.nodes:
                                if node.type == "GROUP_OUTPUT":
                                    nodeOutput = node
                                elif node.type == "GROUP":
                                    if "mn visualizer" in node.node_tree.name.lower():
                                        nodeVisu = node

                            if nodeOutput and nodeVisu:
                                mat = nodeVisu.inputs["Hair Material"].default_value
                                if mat:
                                    y = 0
                                    for shadeNode in mat.node_tree.nodes:
                                        if shadeNode.type == "ATTRIBUTE":
                                            if attriName in shadeNode.attribute_name:
                                                y = y + 1
                                    if y > 0:
                                        attriName = attriName + " " + str(y)
                                try:
                                    atrrSocket = geoNodes.node_group.outputs.new(from_socket.bl_idname,attriName)
                                    geoNodes[atrrSocket.identifier+'_attribute_name'] = attriName
                                    geoNodes.node_group.links.new(from_socket,nodeOutput.inputs[-2])
                                except:
                                    pass

                                if mat:
                                    oldX = 0
                                    for shadeNode in mat.node_tree.nodes:
                                        if shadeNode.location.x < oldX:
                                            oldX = shadeNode.location.x
                                    attrShadeNode = mat.node_tree.nodes.new("ShaderNodeAttribute")
                                    attrShadeNode.attribute_name = attriName
                                    attrShadeNode.location.x = oldX - 150
                                from_socket["attributeName"] = attriName
                                from_socket["attributeShaderNode"] = attrShadeNode.name
                                from_socket["attributeOutNodeSock"] = atrrSocket.name
                                self.matAtrribute = attriName
                                global attrbData
                                attrbData.append(attrShadeNode)
                                attrbData.append(atrrSocket)
                                attrbData.append(geoNodes)
    
        return context.window_manager.invoke_popup(self)

class MedusaNodesRenameMatAtt(bpy.types.Operator):
    bl_idname = "medusanodes.rename_material_attrr"
    bl_label = "Rename Material Attribute"
    bl_options = {'UNDO'}

    def attChanged(self,context):
        global attrbData
        if len(attrbData)>0:
            atrrSocket = attrbData[1]
            geoNodes = attrbData[2]
            attrbData[0].attribute_name = self.matAtrribute
            atrrSocket.name = self.matAtrribute
            geoNodes[atrrSocket.identifier+'_attribute_name']  = self.matAtrribute

            item_index = context.scene.medusaNodesTree_index
            item_list = context.scene.medusaNodesTree
            if len(item_list)>item_index and item_index != -1:
                item = item_list[item_index]
                if check_valid(item):
                    geoNodes = bpy.data.objects[item.objectName].modifiers[item.nodeTreeName]
                    if geoNodes:
                        if geoNodes.type == 'NODES':
                            activeNode = geoNodes.node_group.nodes[item.name]
                            if len(activeNode.inputs[activeNodeGroupSocket].links)>0:
                                linkSocket = activeNode.inputs[activeNodeGroupSocket].links[0]
                                from_socket = linkSocket.from_socket
                                from_socket["attributeName"] = self.matAtrribute
                                from_socket["attributeOutNodeSock"] = self.matAtrribute
        
            attrbData = []

    matAtrribute : StringProperty(
        name="Attribute name",
        description="Attribute name",
        default = "",
        update=attChanged
    )

    def draw(self, context):
        layout = self.layout
        layout.use_property_decorate = False
        layout.label(text='Rename Material Attribute')
        row = layout.row()
        row.prop(self, 'matAtrribute')

    def execute(self, context):
        item_index = context.scene.medusaNodesTree_index
        item_list = context.scene.medusaNodesTree
        if len(item_list)>item_index and item_index != -1:
            item = item_list[item_index]
            if check_valid(item):
                geoNodes = bpy.data.objects[item.objectName].modifiers[item.nodeTreeName]
                if geoNodes:
                    if geoNodes.type == 'NODES':
                        activeNode = geoNodes.node_group.nodes[item.name]
                        if len(activeNode.inputs[activeNodeGroupSocket].links)>0:
                            linkSocket = activeNode.inputs[activeNodeGroupSocket].links[0]
                            from_socket = linkSocket.from_socket

                            if "attributeShaderNode" in from_socket:
                                self.matAtrribute = from_socket["attributeName"]

        return {'FINISHED'}
    
    def invoke(self,context,event):
        item_index = context.scene.medusaNodesTree_index
        item_list = context.scene.medusaNodesTree
        if len(item_list)>item_index and item_index != -1:
            item = item_list[item_index]
            if check_valid(item):
                geoNodes = bpy.data.objects[item.objectName].modifiers[item.nodeTreeName]
                if geoNodes:
                    if geoNodes.type == 'NODES':
                        activeNode = geoNodes.node_group.nodes[item.name]
                        if len(activeNode.inputs[activeNodeGroupSocket].links)>0:
                            linkSocket = activeNode.inputs[activeNodeGroupSocket].links[0]
                            from_socket = linkSocket.from_socket

                            if "attributeShaderNode" in from_socket:
                                nodeVisu   = None

                                for node in geoNodes.node_group.nodes:
                                    if node.type == "GROUP":
                                        if "mn visualizer" in node.node_tree.name.lower():
                                            nodeVisu = node

                                if nodeVisu:
                                    mat = nodeVisu.inputs["Hair Material"].default_value
                                    if mat:
                                        self.matAtrribute = from_socket["attributeName"]
                                        global attrbData
                                        if from_socket["attributeOutNodeSock"] in geoNodes.node_group.outputs and from_socket["attributeShaderNode"] in mat.node_tree.nodes:
                                            outSocket = geoNodes.node_group.outputs[from_socket["attributeOutNodeSock"]]
                                            shaderAttrNode = mat.node_tree.nodes[from_socket["attributeShaderNode"]]
                                            attrbData.append(shaderAttrNode)
                                            attrbData.append(outSocket)
                                            attrbData.append(geoNodes)
                                
        return context.window_manager.invoke_popup(self)

class MedusaNodesRemoveMatAtt(bpy.types.Operator):
    bl_idname = "medusanodes.remove_material_attrr"
    bl_label = "Remove Material Attribute"
    bl_options = {'UNDO'}

    def execute(self, context):
        item_index = context.scene.medusaNodesTree_index
        item_list = context.scene.medusaNodesTree
        if len(item_list)>item_index and item_index != -1:
            item = item_list[item_index]
            if check_valid(item):
                geoNodes = bpy.data.objects[item.objectName].modifiers[item.nodeTreeName]
                if geoNodes:
                    if geoNodes.type == 'NODES':
                        activeNode = geoNodes.node_group.nodes[item.name]
                        if len(activeNode.inputs[activeNodeGroupSocket].links)>0:
                            linkSocket = activeNode.inputs[activeNodeGroupSocket].links[0]
                            from_socket = linkSocket.from_socket

                            if "attributeShaderNode" in from_socket:
                                nodeVisu   = None

                                for node in geoNodes.node_group.nodes:
                                    if node.type == "GROUP":
                                        if "mn visualizer" in node.node_tree.name.lower():
                                            nodeVisu = node

                                if nodeVisu:
                                    mat = nodeVisu.inputs["Hair Material"].default_value
                                    if mat:
                                        self.matAtrribute = from_socket["attributeName"]
                                        global attrbData
                                        if from_socket["attributeOutNodeSock"] in geoNodes.node_group.outputs and from_socket["attributeShaderNode"] in mat.node_tree.nodes:
                                            self.matAtrribute = from_socket["attributeName"]
                                            global attrbData
                                            outSocket = geoNodes.node_group.outputs[from_socket["attributeOutNodeSock"]]
                                            shaderAttrNode = mat.node_tree.nodes[from_socket["attributeShaderNode"]]
                                            if shaderAttrNode:
                                                mat.node_tree.nodes.remove(shaderAttrNode)
                                            if outSocket:
                                                geoNodes.node_group.outputs.remove(outSocket)
        return {"FINISHED"}

class medusaNodesAddInputGroup(bpy.types.Operator):
    bl_idname = "medusanodes.add_group"
    bl_label = ""
    bl_options = {'UNDO'}
    
    groupNodeName: StringProperty(default="")
    groupName : StringProperty(default="")

    def execute(self, context):
        item_index = context.scene.medusaNodesTree_index
        item_list = context.scene.medusaNodesTree
        if len(item_list)>item_index and item_index != -1:
            item = item_list[item_index]
            if check_valid(item):
                geoNodes = bpy.data.objects[item.objectName].modifiers[item.nodeTreeName]
                if geoNodes:
                    if geoNodes.type == 'NODES':
                        activeNode = geoNodes.node_group.nodes[item.name]
                        groupNode = addMedusaNode(geoNodes.node_group,self.groupName)
                        if groupNode and activeNodeGroupSocket!="":
                            geoNodes.node_group.links.new(activeNode.inputs[activeNodeGroupSocket],groupNode.outputs[0])
                            #alignMaskGroupNodes(geoNodes.node_group)
                            #align2Nodes(groupNode,activeNode)
                            bpy.ops.medusanodes.nodeprop_expand(objectName=item.objectName, nodeTreeName="MedusaNodes", nodeName= activeNode.name, socketName=activeNodeGroupSocket)
                            if "curve mask group" in groupNode.node_tree.name.lower():
                                with open(jsonProfile, 'r') as f:
                                    profileData = json.load(f)
                                for json_key in profileData:
                                    if activeNode.node_tree.name in json_key:
                                        if activeNodeGroupSocket in profileData[json_key]:
                                            for snode in groupNode.node_tree.nodes:
                                                if snode.type == "CURVE_FLOAT":
                                                    m = snode.mapping
                                                    while len(m.curves[0].points)>2:
                                                        p = m.curves[0].points[0]
                                                        m.curves[0].points.remove(p)
                                                    data = profileData[json_key][activeNodeGroupSocket]
                                                    if len(data) > 1:
                                                        m.curves[0].points[0].location.x = data[0][0]
                                                        m.curves[0].points[0].location.y = data[0][1]
                                                        for i in range(1,len(data)-1):
                                                            point = data[i]
                                                            p = m.curves[0].points.new(point[0],point[1])
                                                        m.curves[0].points[-1].location.x = data[-1][0]
                                                        m.curves[0].points[-1].location.y = data[-1][1]
                                                    m.update()
                            if "texture mask group" in groupNode.node_tree.name.lower():
                                bpy.context.scene.medusaNodesSaveOptions.image_name = activeNodeGroupSocket.replace(" ","")
        bpy.ops.medusanodes.nodeprop_expand(objectName=item.objectName, nodeTreeName="MedusaNodes", nodeName= activeNode.name, socketName=activeNodeGroupSocket)
        alignNodes(geoNodes.node_group)
        return {'FINISHED'}

class MedusaNodesInvokeMaskMenu(bpy.types.Operator):
    '''Add or remove different Mask Groups, as well as create material attributes'''
    bl_idname = "medusanodes.invokemaskmenu"
    bl_label = "Mask Group Options"
    bl_options = {'UNDO'}

    groupNodeName : StringProperty(default="")
    groupNodeSocketName : StringProperty(default="")
    nodeGroup : StringProperty(default="")

    def execute(self, context):
        global activeNodeGroupName
        global activeNodeGroupSocket
        global activeNodeTreeName
        activeNodeGroupSocket = self.groupNodeSocketName
        activeNodeGroupName = self.groupNodeName
        activeNodeTreeName = self.nodeGroup
        bpy.ops.wm.call_menu(name=MedusaNodeTreeMaskGroupMenu.bl_idname)
        return {'FINISHED'}

class MedusaNodesNode_Documentation(bpy.types.Operator):
    '''Look up the documentation for this parameter (under development).'''
    bl_idname = "medusanodes.documentation"
    bl_label = "Documentation"
    bl_options = {'UNDO'}
    
    inputName : StringProperty()
    nodeGroupName : StringProperty()

    def execute(self, context):
        bpy.ops.wm.url_open(url="https://medusa-nodes.com/docs/"+self.nodeGroupName+"_"+self.inputName)
        return {'FINISHED'}

class MedusaNodeTreeMaskGroupMenu(bpy.types.Menu):
    bl_label = "Mask Groups"
    bl_idname= "MEDUSANODE_MT_masks_group_menu"
    bl_owner_id = "SCENE_PT_medusanodes_main"

    def draw(self, context):
        layout = self.layout
        
        op = layout.operator( "medusanodes.documentation", icon = "QUESTION")
        op.nodeGroupName = activeNodeTreeName
        op.inputName = activeNodeGroupSocket
        layout.separator()
        layout.label(text="Add Masks Group")
        layout.operator("medusanodes.add_group",text="Texture", icon= 'TEXTURE').groupName = "MN Texture Mask Group"
        layout.operator("medusanodes.add_group",text="Random", icon_value=pcoll["random"].icon_id).groupName = "MN Random Mask Group"
        layout.operator("medusanodes.add_group",text="Percent",icon_value=pcoll["percent"].icon_id).groupName = "MN Percent Mask Group"
        layout.operator("medusanodes.add_group",text="Noise", icon ='MOD_NOISE').groupName = "MN Noise Mask Group"
        layout.operator("medusanodes.add_group",text="Curve", icon ='FCURVE').groupName = "MN Curve Mask Group"
        layout.operator("medusanodes.add_group",text="Ramp",icon_value=pcoll["ramp"].icon_id).groupName = "MN Ramp Mask Group"
        layout.operator("medusanodes.add_group",text="White Noise").groupName = "MN White Noise Mask Group"
        layout.operator("medusanodes.add_group",text="Musgrave").groupName = "MN Musgrave Mask Group"
        layout.operator("medusanodes.add_group",text="Magic").groupName = "MN Magic Mask Group"
        layout.operator("medusanodes.add_group",text="Checker").groupName = "MN Checker Mask Group"
        layout.operator("medusanodes.add_group",text="Voronoi").groupName = "MN Voronoi Mask Group"

        # layout.separator()
        # layout.label(text="Options")
        # layout.operator("medusanodes.copy_input_links")
        # layout.operator("medusanodes.paste_input_links")
        # layout.operator("medusanodes.copy_input_nodes")
        # layout.operator("medusanodes.paste_input_nodes")
        layout.separator()
        layout.operator("medusanodes.delete_group",icon = 'TRASH').groupNodeName = activeNodeGroupName
        layout.separator()
        layout.label(text="Material Attribute")
        layout.operator_context = "INVOKE_DEFAULT"
        layout.operator("medusanodes.create_material_attrr")
        layout.operator_context = "INVOKE_DEFAULT"
        layout.operator("medusanodes.rename_material_attrr")
        layout.operator("medusanodes.remove_material_attrr")

class MedusaNodeTreeOptionsDupMenu(bpy.types.Menu):
    '''Duplicate selected deformer'''
    bl_label = 'Duplicate Deformer'
    bl_idname = 'MEDUSANODE_MT_duplicate_menu'

    def draw(self, context):
        layout = self.layout

        layout.operator("medusanodes.duplicate_deformer_dupli")
        layout.operator("medusanodes.duplicate_deformer_inh")

class MedusaNodeTreeOptionsMenu(bpy.types.Menu):
    '''Apply, duplicate, reset, or delete the selected deformer'''
    bl_label = "Deformer Options"
    bl_idname= "MEDUSANODE_MT_options_menu"

    def draw(self, context):
        global activeNodeGroupName
        global activeNodeObjectName
        global activeNodeTreeGroupName
        deformerCached = False
        layout = self.layout

        item_index = context.scene.medusaNodesTree_index
        item_list = context.scene.medusaNodesTree
        if item_index>=0:# and len(item_list)>item_index:
            try:
                item = None
                if len(item_list)>item_index:
                    medusaNodes = context.scene.medusaNodesTree
                    item = medusaNodes[item_index]
                    activeNodeGroupName = item.name
                    activeNodeObjectName = item.objectName
                    activeNodeTreeGroupName = item.nodeTreeName
                else:
                    for barNode in context.scene.medusaNodes:
                        if barNode.name == activeNodeGroupName and barNode.nodeTreeName == activeNodeTreeGroupName and barNode.objectName == activeNodeObjectName:
                            item = barNode
                            break

                if check_valid(item):
                    geoNodes = bpy.data.objects[item.objectName].modifiers[item.nodeTreeName]
                    if geoNodes:
                        if geoNodes.type == 'NODES':
                            for node in geoNodes.node_group.nodes:
                                if node.type == "GROUP":
                                    if "mn generator" in node.node_tree.name.lower():
                                        for inp in node.inputs:
                                            if inp.name.lower() == "cached":
                                                if inp.default_value:
                                                    deformerCached = True
            except Exception as E:
                print(E)

        row = layout.row()
        row.operator("medusanodes.apply_deformer", icon='IMPORT')
        row.enabled = deformerCached
        row = layout.row()
        row.operator("medusanodes.apply_all", icon='IMPORT')
        row.enabled = deformerCached
        layout.menu('MEDUSANODE_MT_duplicate_menu', icon='DUPLICATE')
        layout.operator("medusanodes.reset_deformer", icon='FILE_REFRESH')
        layout.operator("medusanodes.delete_deformer", icon='TRASH')

def nameChanged(self,context):
    if self.name:
        if check_valid(self):
            geoNodes = bpy.data.objects[self.objectName].modifiers[self.nodeTreeName]
            node = geoNodes.node_group.nodes[self.name]
            if "mn visualizer" in node.node_tree.name.lower():
                bpy.data.objects[self.objectName].name = self.label
            else:
                node.label = self.label
            SetupNodeData()

class MedusaNodesTree(bpy.types.PropertyGroup):
    name : bpy.props.StringProperty(default="")
    label : bpy.props.StringProperty(default="")
    expanded: bpy.props.BoolProperty(default=False)
    selfIndex : bpy.props.IntProperty(default=-1)
    parentIndex : bpy.props.IntProperty(default=-1)
    childCount : bpy.props.IntProperty(default=0)
    objectName : bpy.props.StringProperty(default="")
    nodeTreeName : bpy.props.StringProperty(default="")

class MedusaNodesTreeItem(bpy.types.PropertyGroup):
    name : bpy.props.StringProperty(default="")
    label : bpy.props.StringProperty(default="",update=nameChanged)
    indent: bpy.props.IntProperty(default=0)
    expanded: bpy.props.BoolProperty(default=False)
    nodeIndex : bpy.props.IntProperty(default=-1)
    childCount: bpy.props.IntProperty(default=0)
    objectName : bpy.props.StringProperty(default="")
    nodeTreeName : bpy.props.StringProperty(default="")

processedNode = {}
def followLinks(node_in,inp=None):
    global processedNode
    nodeHierarchy = {}
    for n_inputs in node_in.inputs:
        if inp == None:
            for node_links in n_inputs.links:
                if node_links.to_node not in nodeHierarchy:
                    nodeHierarchy[node_links.to_node] = []
                if node_links.from_node not in nodeHierarchy[node_links.to_node]:
                    nodeHierarchy[node_links.to_node].append(node_links.from_node)
                
                nodeHierarchyR = {}
                if node_links.from_node.name not in processedNode:
                    processedNode[node_links.from_node.name] = []
                if n_inputs.name not in processedNode[node_links.from_node.name]:
                    nodeHierarchyR = followLinks(node_links.from_node)
                processedNode[node_links.from_node.name].append(n_inputs.name)
                nodeHierarchy = {**nodeHierarchy, **nodeHierarchyR}
        elif n_inputs.name.lower() == inp.lower():
            for node_links in n_inputs.links:
                if node_links.to_node not in nodeHierarchy:
                    nodeHierarchy[node_links.to_node] = []
                if node_links.from_node not in nodeHierarchy[node_links.to_node]:
                    nodeHierarchy[node_links.to_node].append(node_links.from_node)
                        
                nodeHierarchyR = {}
                if node_links.from_node.name not in processedNode:
                    processedNode[node_links.from_node.name] = []
                if n_inputs.name not in processedNode[node_links.from_node.name]:
                    nodeHierarchyR = followLinks(node_links.from_node)
                processedNode[node_links.from_node.name].append(n_inputs.name)
                nodeHierarchy = {**nodeHierarchy, **nodeHierarchyR}
    return nodeHierarchy

def getNodeHierarchy(geoNodes):
    global processedNode
    nodeHierarchy = {}
    nodeHierarchyCleaned = {}
    finalNodeHierarchy = {}
    
    if geoNodes:
        if geoNodes.type == 'NODES':
            for geo_node in geoNodes.node_group.nodes:
                if geo_node.type == 'GROUP_OUTPUT':
                    nodeHierarchy = followLinks(geo_node)
                    processedNode = {}
                    break
    pastKey = None
    for node in nodeHierarchy:
        finSubNodes = []
        if node.type == "REROUTE":
            subNodes = nodeHierarchy[pastKey]+nodeHierarchy[node]
            for subNode in subNodes:
                if subNode.type != "REROUTE":
                    finSubNodes.append(subNode)
            nodeHierarchyCleaned[pastKey] = finSubNodes
        else:
            subNodes = nodeHierarchy[node]
            for subNode in subNodes:
                if subNode.type != "REROUTE":
                    finSubNodes.append(subNode)
            nodeHierarchyCleaned[node] = finSubNodes
        if node.type != "REROUTE":
            pastKey = node

    for node in nodeHierarchyCleaned:
        if node.type == "GROUP":
            finSubNodes = []
            for subNode in nodeHierarchyCleaned[node]:
                if subNode.type == "GROUP":
                    if "mask" not in subNode.node_tree.name.lower():
                        if subNode not in finSubNodes:
                            finSubNodes.append(subNode)
            finalNodeHierarchy[node] = finSubNodes
    return finalNodeHierarchy

def SetupNodeData():
    global treeOldList
    global treeOldCpt
    global processedNode
    ob = bpy.context.object
    lastMedusaHairOb = bpy.context.scene.lastMedusaHairOb
    if sculptMode and bpy.context.mode == "SCULPT_CURVES":
        ob = sculptModeLstOb
    elif paintModeLstOb and bpy.context.mode == "PAINT_TEXTURE":
        ob = paintModeLstOb
    if not bpy.context.scene.use_medusa_hair_object:
        if "MedusaHair" in ob:
            ob = ob["MedusaHair"]
        if lastMedusaHairOb in bpy.data.objects:
            if ob is None:
                ob = bpy.data.objects[lastMedusaHairOb]
            else:
                if len(ob.modifiers) == 0:
                    ob = bpy.data.objects[lastMedusaHairOb]

    if ob:
        if len(ob.modifiers)>0:
            geoNodes = ob.modifiers.active
            medusaNodes = bpy.context.scene.medusaNodes
            treeList = bpy.context.scene.medusaNodesTree

            bpy.context.scene.medusaNodesActiveOb = ob.name
            bpy.context.scene.medusaNodesActiveNodeTree = ""
            if geoNodes.type == "NODES" and "medusanodes" in geoNodes.name.lower():
                bpy.context.scene.lastMedusaHairOb = ob.name
                for treeItem in treeList:
                    treeOldList[str(treeItem.nodeIndex)+"_"+treeItem.name+"_"+treeItem.objectName+"_"+treeItem.nodeTreeName] = [treeItem.name,treeItem.objectName,treeItem.nodeTreeName,treeItem.indent,treeItem.expanded]

                medusaNodes.clear()
                treeList.clear()
                if geoNodes.node_group is not None:
                    bpy.context.scene.medusaNodesActiveNodeTree = geoNodes.node_group.name
                    finalNodeHierarchy = getNodeHierarchy(geoNodes)
                    finalNodeHierarchyF = {}

                    processedNodes = []
                    mainNode = None
                    for node in finalNodeHierarchy:
                        node["objectName"] = ob.name
                        node["nodeTreeName"] = geoNodes.name
                        if "mn visualizer" in node.node_tree.name.lower():
                            mainNode = node
                            finalNodeHierarchyF[mainNode] = []
                            node["hair_object"] = ob
                        if "mn guide" in node.node_tree.name.lower() or "mn clump" in node.node_tree.name.lower() or "mn braid" in node.node_tree.name.lower():
                            finalNodeHierarchyF[node] = []
                            guideCurveOb = None
                            for inp in node.inputs:
                                if "guide" in inp.name.lower():
                                    guideHier = followLinks(node,inp=inp.name)
                                    processedNode = {}
                                    if len(guideHier)>0:
                                        firsKey = list(guideHier)[0]
                                        if guideHier[firsKey][0].type != "GROUP":
                                            lstKey = list(guideHier)[-1]
                                            if len(guideHier[lstKey])>0:
                                                if guideHier[lstKey][-1].type == "GROUP_INPUT":
                                                    for out in guideHier[lstKey][-1].outputs:
                                                        for lnk in out.links:
                                                            if lnk.to_node == lstKey:
                                                                guideCurveOb=geoNodes[out.identifier]
                                                                break
                                                elif guideHier[lstKey][-1].type == "OBJECT_INFO":
                                                    guideCurveOb=guideHier[lstKey][-1].inputs['Object'].default_value
                                    break
                            node["guide_curve"] = guideCurveOb
                            guideCurveNodeMod = None
                            if guideCurveOb:
                                for mod in guideCurveOb.modifiers:
                                    if mod.type == "NODES":
                                        if mod.node_group:
                                            for inpGuid in mod.node_group.interface.items_tree:
                                                if inpGuid.name.lower() == "emitter geometry" or inpGuid.name.lower() == "emitter object":
                                                    guideCurveNodeMod = mod
                                                    break
                                        break
                                    if guideCurveOb == None:
                                        if "hair" in mod.name.lower() or "guide" in mod.name.lower():
                                            guideCurveNodeMod = mod
                            if guideCurveNodeMod:
                                guideNodeHierarchy = getNodeHierarchy(guideCurveNodeMod)
                                mainNodeG = None
                                guideNodeFinalHier = {}
                                tempTest = {}
                                for subNodeG in guideNodeHierarchy:
                                    subNodeG["objectName"] = guideCurveOb.name
                                    subNodeG["nodeTreeName"] = guideCurveNodeMod.name
                                    if "mn visualizer" in subNodeG.node_tree.name.lower():
                                        mainNodeG = subNodeG
                                        guideNodeFinalHier[mainNodeG] = []
                                        tempTest[mainNodeG.label] = []
                                        mainNodeG["hair_object"] = guideCurveOb
                                    if mainNodeG:
                                        for nodeG in guideNodeHierarchy[subNodeG]:
                                            nodeG["objectName"] = guideCurveOb.name
                                            nodeG["nodeTreeName"] = guideCurveNodeMod.name
                                            if nodeG not in guideNodeFinalHier[mainNodeG]:
                                                guideNodeFinalHier[mainNodeG].append(nodeG)
                                                tempTest[mainNodeG.label].append(nodeG.label)
                                            else:
                                                guideNodeFinalHier[mainNodeG].remove(nodeG)
                                                guideNodeFinalHier[mainNodeG].append(nodeG)
                                if mainNodeG:
                                    finalNodeHierarchyF[node].append(mainNodeG)
                                    finalNodeHierarchyF = {**finalNodeHierarchyF, **guideNodeFinalHier}
                        if mainNode and node not in processedNodes:
                            for subNode in finalNodeHierarchy[node]:
                                if subNode not in processedNodes:
                                    subNode["objectName"] = ob.name
                                    subNode["nodeTreeName"] = geoNodes.name
                                    if subNode not in finalNodeHierarchyF[mainNode]:
                                        finalNodeHierarchyF[mainNode].append(subNode)
                                    else:
                                        finalNodeHierarchyF[mainNode].remove(subNode)
                                        finalNodeHierarchyF[mainNode].append(subNode)
                    t = 0
                    j = 0
                    nodesIndex = {}
                    mainNodeH = None
                    pareIndexGui = 0
                    
                    for nodeH in finalNodeHierarchyF:
                        if t == 0:
                            mainNodeH = nodeH
                            nodesIndex[nodeH] = j
                            node = medusaNodes.add()
                            if "mn visualize" in nodeH.node_tree.name.lower():
                                node.label = nodeH["objectName"]
                            else:
                                if nodeH.label == "":
                                    nodeH.label = nodeH.node_tree.name.replace("MN ","")
                                node.label = nodeH.label
                            node.name = nodeH.name
                            node.selfIndex = len(medusaNodes)-1
                            node.objectName = nodeH["objectName"]
                            node.nodeTreeName = nodeH["nodeTreeName"]
                            t = 1
                            j = j + 1

                        for links in reversed(finalNodeHierarchyF[nodeH]):
                            node = medusaNodes.add()
                            if "mn visualize" in links.node_tree.name.lower():
                                node.label = links["objectName"]
                            else:
                                if links.label == "":
                                    links.label = links.node_tree.name.replace("MN ","")
                                node.label = links.label
                            node.name = links.name
                            node.selfIndex = len(medusaNodes)-1
                            node.parentIndex = nodesIndex[nodeH]
                            node.objectName = links["objectName"]
                            node.nodeTreeName = links["nodeTreeName"]
                            nodesIndex[links] = j
                            j = j + 1
                    
                    v = {}
                    for node in medusaNodes:
                        if node.parentIndex != -1:
                            parent = medusaNodes[node.parentIndex]
                            parent.childCount = parent.childCount + 1
                        if node.parentIndex in v:
                            v[node.parentIndex].append(node.label)
                        else:
                            v[node.parentIndex] = [node.label]
                    for node in medusaNodes:
                        treSel = str(node.selfIndex)+"_"+node.name+"_"+node.objectName+"_"+node.nodeTreeName
                        if node.parentIndex == -1:
                            item = treeList.add()
                            item.label = node.label
                            item.name = node.name
                            item.nodeIndex = node.selfIndex
                            item.childCount = node.childCount
                            item.objectName   = node.objectName
                            item.nodeTreeName = node.nodeTreeName
                            
                            if treSel in treeOldList:
                                if treeOldList[treSel][4]:
                                    item.expanded = True
                                    for n in medusaNodes:
                                        if item.nodeIndex == n.parentIndex:
                                            InsertBeneath(treeList, item.nodeIndex, treeOldList[treSel][3], n)
                        else:
                            if treSel in treeOldList:
                                if treeOldList[treSel][4]:
                                    j = 0
                                    for tr in treeList:
                                        if tr.name == node.name and tr.objectName == node.objectName and tr.nodeTreeName == node.nodeTreeName:
                                            tr.expanded = True
                                            for n in medusaNodes:
                                                if node.selfIndex == n.parentIndex:
                                                    InsertBeneath(treeList, j, treeOldList[treSel][3], n)
                                            break
                                        j = j + 1

            treeOldCpt = len(treeList)
        else:
            medusaNodes = bpy.context.scene.medusaNodes
            treeList = bpy.context.scene.medusaNodesTree
            medusaNodes.clear()
            treeList.clear()

def InsertBeneath( treeList, parentIndex, parentIndent, node,expand=False):
    after_index = parentIndex + 1
    existIndex = -1
    item = treeList.add()
    item.label = node.label
    item.expanded = expand
    item.name = node.name
    item.nodeIndex = node.selfIndex
    item.childCount = node.childCount
    item.objectName   = node.objectName
    item.nodeTreeName = node.nodeTreeName
    item.indent = parentIndent+1
    item_index = len(treeList) -1
    treeList.move(item_index,after_index)

class MedusaNodesTreeItem_Expand(bpy.types.Operator):
    '''Expand or collapse the hierarchy to view the nodes'''
    bl_idname = "medusanodes.medusanodestree_expand"
    bl_label = "Expand / Collapse Hierarchy"
    bl_options = {'UNDO'}
    
    button_id: IntProperty(default=-1)

    def execute(self, context):
        item_index = self.button_id
        item_list = bpy.context.scene.medusaNodesTree
        currentIndex = bpy.context.scene.medusaNodesTree_index

        nodeNa = None
        nodeLa = None
        nodeTr = None
        nodeOb = None
        try:
            item = item_list[currentIndex]
            nodeLa = item.label
            nodeNa = item.name
            nodeTr = item.nodeTreeName
            nodeOb = item.objectName
        except:
            pass
        if item_index < len(item_list):
            item = item_list[item_index]
            item_indent = item.indent
            nodeIndex = item.nodeIndex
            medusaNodes = bpy.context.scene.medusaNodes
            
            if item.expanded:
                item.expanded = False
                nextIndex = item_index+1
                while True:
                    if nextIndex >= len(item_list):
                        break
                    if item_list[nextIndex].indent <= item_indent:
                        break
                    item_list.remove(nextIndex)
            else:
                item.expanded = True
                for n in medusaNodes:
                    if nodeIndex == n.parentIndex:
                        InsertBeneath(item_list, item_index, item_indent, n)
        
        if nodeLa:
            item_list = bpy.context.scene.medusaNodesTree
            for i in range(0,len(item_list)):
                item = item_list[i]
                if item.label == nodeLa and item.name == nodeNa and item.nodeTreeName == nodeTr and item.objectName == nodeOb:
                    bpy.context.scene.medusaNodesTree_index = i
                    break
        return {'FINISHED'}

class MedusaNodesNodeProp_Expand(bpy.types.Operator):
    '''Expand or collapse the Mask Group'''
    bl_idname = "medusanodes.nodeprop_expand"
    bl_label = "Expand / Collapse Mask Group"
    bl_options = {'UNDO'}
    
    objectName : StringProperty()
    nodeTreeName: StringProperty()
    nodeName: StringProperty()
    socketName: StringProperty()

    def execute(self, context):
        if check_valid(self):
            geoNodes = bpy.data.objects[self.objectName].modifiers[self.nodeTreeName].node_group
            socket = geoNodes.nodes[self.nodeName].inputs[self.socketName]
            if "expanded" not in socket:
                socket["expanded"] = False
            else:
                if socket["expanded"] == False:
                    socket["expanded"] = True
                else:
                    socket["expanded"] = False
        return {'FINISHED'}

class MedusaNodesAddTemplates(bpy.types.Operator):
    bl_idname = "medusanodes.templates"
    bl_label = "Templates"
    bl_options = {'UNDO'}
    
    action: StringProperty(default="default")
    
    def execute(self, context):
        action = self.action
        blendFile = os.path.join(assetPath, "presets.blend")
        presetNodeGroup = bpy.data.node_groups.get(action)
        if presetNodeGroup:
            bpy.data.node_groups.remove(presetNodeGroup)
        with bpy.data.libraries.load(blendFile) as (data_from, data_to):
            for group in data_from.node_groups:
                if group == action:
                    data_to.node_groups = [group]
                    break
        presetNodeGroup = bpy.data.node_groups.get(action)
        if presetNodeGroup:
            geoNodes = bpy.context.object.modifiers.active
            geoNodes.node_group = presetNodeGroup
        
        return {'FINISHED'}

class MedusaNodesDisable(bpy.types.Operator):
    '''Enable or Disable the node'''
    bl_idname = "medusanodes.disable"
    bl_label = "Enable / Disable"
    bl_options = {'UNDO'}

    itemName: StringProperty(default="")
    objectName: StringProperty(default="")
    nodeTreeName: StringProperty(default="")

    def execute(self, context):
        if check_valid(self):
            geoNodes = bpy.data.objects[self.objectName].modifiers[self.nodeTreeName]
            activeNode = geoNodes.node_group.nodes[self.itemName]
            attributes = bpy.data.objects[self.objectName].data.curves.data.attributes

            if geoNodes:
                if geoNodes.type == 'NODES':
                    for inp in activeNode.inputs:
                        if inp.name.lower() == "disable":
                            inp.default_value = not inp.default_value
                            if inp.default_value:
                                if "guide" in activeNode.node_tree.name.lower():
                                    #Disable the attribute of "Guide" deformer
                                    try:
                                        attributes['GuidePosition'].name += 'Disabled'
                                        attributes['GuideInfluence'].name += 'Disabled'
                                    except:
                                        pass
                            else:
                                if "guide" in activeNode.node_tree.name.lower():
                                    #Enable the attribute by renaming it back to the same name that way cached "Guide" deformer can work properly
                                    try:
                                        attributes['GuidePositionDisabled'].name = attributes['GuidePositionDisabled'].name[:-8]
                                        attributes['GuideInfluenceDisabled'].name = attributes['GuideInfluenceDisabled'].name[:-8]
                                    except:
                                        pass
                            break

        return {'FINISHED'}

class MedusaNodesHide(bpy.types.Operator):
    '''Show or hide this Medusa Nodes' object'''
    bl_idname = "medusanodes.hide"
    bl_label = "Show / Hide"
    bl_options = {'UNDO'}

    itemName: StringProperty(default="")
    def execute(self, context):
        ob = bpy.data.objects[self.itemName]
        ob.hide_set(not ob.hide_get())
        return {'FINISHED'}

class MedusaNodesSculptToggle(bpy.types.Operator):
    '''Enter Sculpt Mode. Clicking the button again will switch the mode back to the previous one. To be able to sculpt, the 'Generator' should be freezed first'''
    bl_idname = "medusanodes.togglesculpt"
    bl_label = "Sculpt Mode Toggle"
    bl_options = {'UNDO'}

    itemName: StringProperty(default="")
    def execute(self, context):
        global sculptMode
        global sculptModeLstOb
        orOb = bpy.context.object
        
        originalMod = bpy.data.objects[self.itemName].modifiers['MedusaNodes']
        nodes = originalMod.node_group.nodes
        genNode = None
        for n in nodes:
            if "generator" in n.label.lower():
               genNode = n
               generatorLabel = n.label
               cacheState = n.inputs['Cached'].default_value

        if cacheState==False:
            # message=f"To be able to sculpt on the '{orOb.name}' please freeze the '{generatorLabel}' first!"
            # self.report({'WARNING'}, message)
            if genNode:
                bpy.ops.medusanodes.cache_node(objectName=self.itemName, nodeTreeName="MedusaNodes", nodeName=genNode.name, socketName="Cached")
            
        if bpy.context.object.mode != "SCULPT_CURVES":
            sculptModeLstOb = orOb
            sculptMode = True
            ob = bpy.data.objects[self.itemName]
            oldVis = ob.hide_get()
            oldVie = ob.hide_viewport
            ob.hide_set(False)
            ob.hide_viewport = False
            bpy.ops.object.mode_set(mode="OBJECT")
            bpy.ops.object.select_all(action="DESELECT")

            bpy.context.view_layer.objects.active = ob
            ob.select_set(True)
            bpy.ops.object.mode_set(mode="SCULPT_CURVES",toggle=True)
            
            ob.hide_set(oldVis)
            ob.hide_viewport = oldVie
        else:
            sculptMode = False
            try:
                oldVis = sculptModeLstOb.hide_get()
                oldVie = sculptModeLstOb.hide_viewport
                try:
                    sculptModeLstOb.hide_set(False)
                    sculptModeLstOb.hide_viewport = False
                except:
                    pass
                bpy.ops.object.mode_set(mode="OBJECT")
                bpy.ops.object.select_all(action="DESELECT")
                bpy.context.view_layer.objects.active = sculptModeLstOb
                try:
                    sculptModeLstOb.select_set(True)
                    sculptModeLstOb.hide_set(oldVis)
                    sculptModeLstOb.hide_viewport = oldVie
                except:
                    pass
            except:
                pass
    
        return {'FINISHED'}

class MedusaNodesGuides(bpy.types.Operator):
    bl_idname = "medusanodes.guides"
    bl_label = "Guides"
    bl_options = {'UNDO'}

    itemName: StringProperty(default="")
    def execute(self, context):
        return {'FINISHED'}

addedArea = None
currentMode = None
lstObj = None

class MedusaNodesEditMaskGroup(bpy.types.Operator):
    '''Edit this 'Mask Group' in Geometry Nodes' node editor'''
    bl_idname = "medusanodes.edit_mask_group"
    bl_label = "Edit Mask Group"
    bl_options = {'UNDO'}

    groupNodeName: StringProperty(default="")
    def execute(self, context):
        global addedArea
        global lstObj
        global currentMode

        item_index = context.scene.medusaNodesTree_index
        item_list = context.scene.medusaNodesTree

        if len(item_list)>item_index and item_index != -1:
            pass
        else:
            item_index = 0
        try:
            item = item_list[item_index]
            if check_valid(item):
                geoNodes = bpy.data.objects[item.objectName].modifiers[item.nodeTreeName]
                if geoNodes:
                    if geoNodes.type == 'NODES':
                        activeNode = geoNodes.node_group.nodes[self.groupNodeName]
                        if activeNode.type == "GROUP":
                            if currentMode is None:
                                currentMode = bpy.context.mode
                                if "_" in currentMode:
                                    currentMode = "_".join(currentMode.split("_")[::-1])
                                lstObj = bpy.context.object
                            bpy.ops.object.mode_set(mode="OBJECT", toggle=False)
                            bpy.ops.object.select_all(action="DESELECT")
                            oldVis = bpy.data.objects[item.objectName].hide_get()
                            oldVie = bpy.data.objects[item.objectName].hide_viewport
                            bpy.data.objects[item.objectName].hide_set(False)
                            bpy.data.objects[item.objectName].hide_viewport = False
                            bpy.context.view_layer.objects.active = bpy.data.objects[item.objectName]
                            bpy.data.objects[item.objectName].select_set(True)
                            deselectNodes(geoNodes)
                            geoNodes.node_group.nodes.active = activeNode
                            activeNode.select = True
                            if bpy.context.area.type == "VIEW_3D":
                                skip = False
                                for area in bpy.context.screen.areas:
                                    if area.type == 'NODE_EDITOR' and area.spaces[0].tree_type == 'GeometryNodeTree':
                                        skip = True
                                        break
                                if not skip:
                                    start_areas = context.screen.areas[:]
                                    bpy.ops.screen.area_split(direction='HORIZONTAL', factor=0.3)
                                    for area in context.screen.areas:
                                        if area not in start_areas:
                                            addedArea = area
                                            addedArea.type = "NODE_EDITOR"
                                            addedArea.spaces.active.tree_type = 'GeometryNodeTree'
                                            break
                                bpy.context.view_layer.update()
                                bpy.ops.wm.redraw_timer(type='DRAW_WIN_SWAP', iterations=1)
                                for area in bpy.context.screen.areas:
                                    if area.type == 'NODE_EDITOR' and area.spaces[0].tree_type == 'GeometryNodeTree':
                                        with bpy.context.temp_override(window=bpy.context.window,area=area,region=[region for region in area.regions if region.type == 'WINDOW'][0]):
                                            bpy.context.space_data.show_region_toolbar = False
                                            bpy.context.space_data.show_region_ui = False
                                            if bpy.context.active_node == activeNode or bpy.context.space_data.edit_tree != activeNode.node_tree:
                                                try:
                                                    bpy.ops.node.tree_path_parent()
                                                except:
                                                    pass
                                                bpy.ops.node.group_edit()
                                            else:
                                                try:
                                                    bpy.ops.node.tree_path_parent()
                                                except:
                                                    pass
                                                if bpy.context.area == addedArea:
                                                    bpy.ops.object.select_all(action="DESELECT")
                                                    bpy.context.view_layer.objects.active = lstObj
                                                    lstObj.select_set(True)
                                                    bpy.ops.object.mode_set(mode=currentMode, toggle=False)
                                                    bpy.ops.screen.area_close()
                                                    addedArea = None
                                                    currentMode = None
                                                    lstObj = None
                                        break
                            else:
                                if bpy.context.active_node == activeNode or bpy.context.space_data.edit_tree != activeNode.node_tree:
                                    try:
                                        bpy.ops.node.tree_path_parent()
                                    except:
                                        pass
                                    bpy.ops.node.group_edit()
                                else:
                                    try:
                                        bpy.ops.node.tree_path_parent()
                                    except:
                                        pass
                                    try:
                                        bpy.ops.object.select_all(action="DESELECT")
                                        bpy.context.view_layer.objects.active = lstObj
                                        lstObj.select_set(True)
                                        bpy.ops.object.mode_set(mode=currentMode, toggle=False)
                                    except:
                                        pass
                                    lstObj = None
                                    currentMode = None
                            deselectNodes(geoNodes)
                            try:
                                bpy.data.objects[item.objectName].hide_set(oldVis)
                                bpy.data.objects[item.objectName].hide_viewport = oldVie
                            except:
                                pass
        except:
            pass
        return {'FINISHED'}

class MedusaNodesDeleteGroup(bpy.types.Operator):
    '''Remove this 'Mask Group.' The 'Mask Group' will be removed only if it is not linked to a different deformer. In that case, the link will be cut'''
    bl_idname = "medusanodes.delete_group"
    bl_label = "Remove Mask Group"
    bl_options = {'UNDO'}

    groupNodeName: StringProperty(default="")
    def execute(self, context):
        global processedNode
        item_index = context.scene.medusaNodesTree_index
        item_list = context.scene.medusaNodesTree
        
        if len(item_list)>item_index and item_index != -1:
            item = item_list[item_index]
            if check_valid(item):
                geoNodes = bpy.data.objects[item.objectName].modifiers[item.nodeTreeName]
                if geoNodes:
                    if geoNodes.type == 'NODES':
                        activeNode = geoNodes.node_group.nodes[self.groupNodeName]
                        nodes = followLinks(activeNode,inp=None)
                        processedNode = {}
                        k = 0
                        
                        try:
                            bpy.ops.node.tree_path_parent()
                        except:
                            pass
                        
                        for node in reversed(nodes):
                            for subNode in nodes[node]:
                                for out in subNode.outputs:
                                    if len(out.links)>1:
                                        k = 1
                                        break
                                if k == 1:
                                    break
                            if k == 1:
                                break
                        
                        if k == 0:
                            for node in reversed(nodes):
                                for subNode in nodes[node]:
                                    try:
                                        geoNodes.node_group.nodes.remove(subNode)
                                    except Exception as E:
                                        pass
                                try:
                                    geoNodes.node_group.nodes.remove(node)
                                except Exception as E:
                                    pass
                            
                            try:
                                geoNodes.node_group.nodes.remove(activeNode)
                            except Exception as E:
                                pass
                        
        return {'FINISHED'}


class MedusaNodesViewGroup(bpy.types.Operator):
    bl_idname = "medusanodes.view_group"
    bl_label = "View Group"
    bl_options = {'UNDO'}

    groupNodeName: StringProperty(default="")

    def execute(self, context):
        global lstObj
        global currentMode

        item_index = context.scene.medusaNodesTree_index
        item_list = context.scene.medusaNodesTree
        
        if len(item_list)>item_index and item_index != -1:
            item = item_list[item_index]
            if check_valid(item):
                geoNodes = bpy.data.objects[item.objectName].modifiers[item.nodeTreeName]
                if geoNodes:
                    if geoNodes.type == 'NODES':
                        activeNode = geoNodes.node_group.nodes[self.groupNodeName]
                        nodes = bpy.context.object.modifiers[0].node_group.nodes
                        nodeLinks = bpy.context.object.modifiers[0].node_group.links

                        if currentMode is None:
                            currentMode = bpy.context.mode
                            if "_" in currentMode:
                                currentMode = "_".join(currentMode.split("_")[::-1])
                            lstObj = bpy.context.object
                        bpy.ops.object.mode_set(mode="OBJECT", toggle=False)
                        bpy.ops.object.select_all(action="DESELECT")
                        bpy.data.objects[item.objectName].hide_set(False)
                        bpy.data.objects[item.objectName].hide_viewport = False
                        bpy.context.view_layer.objects.active = bpy.data.objects[item.objectName]
                        bpy.data.objects[item.objectName].select_set(True)
                        
                        viewNode = None
                        visualNode = None
                        anyViewerNode = None
                        for node in geoNodes.node_group.nodes:
                            if "visualizer" in node.label.lower():
                                visualNode = node
                            if node.type == "VIEWER":
                                anyViewerNode = node

                            if visualNode and anyViewerNode:
                                break
                        
                        for lnk in activeNode.outputs[0].links:
                            if lnk.to_node.type == "VIEWER":
                                if "MN Viewer" in lnk.to_node.name:
                                    viewNode = lnk.to_node
                                    break
                        
                        if visualNode and viewNode is None:
                            deselectNodes(geoNodes)
                            visualNode.select = True
                            nodes.active = visualNode

                            nodeEditorArea = None
                            for area in bpy.context.screen.areas:
                                if area.type == 'NODE_EDITOR' and area.spaces[0].tree_type == 'GeometryNodeTree':
                                    nodeEditorArea = area
                                    break
                            if nodeEditorArea is None:
                                start_areas = context.screen.areas[:]
                                bpy.ops.screen.area_split(direction='HORIZONTAL', factor=0.99)
                                for area in context.screen.areas:
                                    if area not in start_areas:
                                        nodeEditorArea = area
                                        nodeEditorArea.type = "NODE_EDITOR"
                                        nodeEditorArea.spaces.active.tree_type = 'GeometryNodeTree'
                                        nodeEditorArea.spaces.active.show_region_header = False
                                        bpy.context.scene["MN_AREA"] = str(nodeEditorArea.as_pointer())
                                        break

                            bpy.context.view_layer.update()
                            bpy.ops.wm.redraw_timer(type='DRAW_WIN_SWAP', iterations=1)
                            for area in bpy.context.screen.areas:
                                if area.type == 'NODE_EDITOR' and area.spaces[0].tree_type == 'GeometryNodeTree':
                                    with bpy.context.temp_override(window=bpy.context.window,area=area,region=[region for region in area.regions if region.type == 'WINDOW'][0]):
                                        bpy.ops.node.link_viewer()
                                        if anyViewerNode is None:
                                            new_viewNode = nodes.active 
                                        else:
                                            new_viewNode = anyViewerNode
                                        if new_viewNode:
                                            new_viewNode.name = "MN Viewer"

                                            nodeLinks.new(new_viewNode.inputs[0],visualNode.outputs[0])
                                            nodeLinks.new(new_viewNode.inputs[1],activeNode.outputs[0])
                                        else:
                                            bpy.ops.medusanodes.view_group("INVOKE_DEFAULT",groupNodeName=self.groupNodeName)
                                    break
                        elif viewNode:
                            geoNodes.node_group.nodes.remove(viewNode)
                            try:
                                bpy.ops.object.mode_set(mode="OBJECT", toggle=False)
                                bpy.ops.object.select_all(action="DESELECT")
                                bpy.context.view_layer.objects.active = lstObj
                                lstObj.select_set(True)
                                bpy.ops.object.mode_set(mode=currentMode, toggle=False)
                            except:
                                pass
                            if "MN_AREA" in bpy.context.scene:
                                for area in bpy.context.screen.areas:
                                    if str(area.as_pointer()) == bpy.context.scene["MN_AREA"]:
                                        with bpy.context.temp_override(window=bpy.context.window,area=area,region=[region for region in area.regions if region.type == 'WINDOW'][0]):
                                            bpy.ops.screen.area_close()
                                            break
                                

        return {'FINISHED'}


class MedusaNodesMuteLink(bpy.types.Operator):
    '''Enable or disable the 'Mask Group.' If the 'Mask Group' is disabled, it will expose the original parameter, which can then be edited by the user'''
    bl_idname = "medusanodes.mute_link"
    bl_label = "Enable / Disable the 'Mask Group'"
    bl_options = {'UNDO'}

    socketName: StringProperty(default="")
    def execute(self, context):
        item_index = context.scene.medusaNodesTree_index
        item_list = context.scene.medusaNodesTree
        if len(item_list)>item_index and item_index != -1:
            item = item_list[item_index]
            if check_valid(item):
                geoNodes = bpy.data.objects[item.objectName].modifiers[item.nodeTreeName]
                if geoNodes:
                    if geoNodes.type == 'NODES':
                        activeNode = geoNodes.node_group.nodes[item.name]
                        socket = activeNode.inputs[self.socketName]
                        for link in socket.links:
                            link.is_muted = not link.is_muted
                        activeNode.mute = not activeNode.mute
                        activeNode.mute = not activeNode.mute
    
        return {'FINISHED'}

def swapNodes(nodeLinks,nodeA,nodeB):
    nodeALinksInputs = {}
    nodeBLinksInputs = {}
    
    nodeALinksOutputs = {}
    nodeBLinksOutputs = {}
    
    for inp in nodeA.inputs:
        if inp.is_linked == True:
            nodeALinksInputs[inp.name] = {}
            nodeALinksInputs[inp.name]["inputName"] = inp
            nodeALinksInputs[inp.name]["connectedOutputs"] = []
            for link in inp.links:
                nodeALinksInputs[inp.name]["connectedOutputs"].append(link.from_socket)
    
    for out in nodeA.outputs:
        if out.is_linked == True:
            nodeALinksOutputs[out.name] = {}
            nodeALinksOutputs[out.name]["outputName"] = out
            nodeALinksOutputs[out.name]["connectedInputs"] = []
            for link in out.links:
                nodeALinksOutputs[out.name]["connectedInputs"].append(link.to_socket)
                
    for inp in nodeB.inputs:
        if inp.is_linked == True:
            nodeBLinksInputs[inp.name] = {}
            nodeBLinksInputs[inp.name]["inputName"] = inp
            nodeBLinksInputs[inp.name]["connectedOutputs"] = []
            for link in inp.links:
                nodeBLinksInputs[inp.name]["connectedOutputs"].append(link.from_socket)
    
    for out in nodeB.outputs:
        if out.is_linked == True:
            nodeBLinksOutputs[out.name] = {}
            nodeBLinksOutputs[out.name]["outputName"] = out
            nodeBLinksOutputs[out.name]["connectedInputs"] = []
            for link in out.links:
                nodeBLinksOutputs[out.name]["connectedInputs"].append(link.to_socket)
    oldLinks = []
    for inp in nodeB.inputs:
        for link in inp.links:
            if link.from_node == nodeA:
                oldLinks.append(link)
                try:
                    nodeLinks.new(nodeB.outputs[inp.name],link.from_node.inputs[inp.name])
                except:
                    pass
    inpToConnect = ["splines","emitter geometry","guide curves"]
    for link in oldLinks:
        nodeLinks.remove(link)
    for inp in nodeALinksInputs:
        if inp in nodeBLinksInputs and inp.lower() in inpToConnect:
            for link in nodeALinksInputs[inp]['connectedOutputs']:
                nodeLinks.new(nodeBLinksInputs[inp]["inputName"],link)
    
    for inp in nodeALinksOutputs:
        if inp in nodeBLinksOutputs:
            for link in nodeALinksOutputs[inp]['connectedInputs']:
                if link.node != nodeBLinksOutputs[inp]["outputName"].node:
                    nodeLinks.new(nodeBLinksOutputs[inp]["outputName"],link)
                    
    for inp in nodeBLinksInputs:
        if inp in nodeALinksInputs and inp.lower() in inpToConnect:
            for link in nodeBLinksInputs[inp]['connectedOutputs']:
                if link.node != nodeALinksInputs[inp]["inputName"].node:
                    nodeLinks.new(nodeALinksInputs[inp]["inputName"],link)
    
    for inp in nodeBLinksOutputs:
        if inp in nodeALinksOutputs:
            for link in nodeBLinksOutputs[inp]['connectedInputs']:
                if link.node != nodeALinksOutputs[inp]["outputName"].node:
                    nodeLinks.new(nodeALinksOutputs[inp]["outputName"],link)
    
    nodeALoc = (nodeA.location[0],nodeA.location[1])
    nodeBLoc = (nodeB.location[0],nodeB.location[1])
    
    nodeB.location = nodeALoc
    nodeA.location = nodeBLoc

class MedusaNodesOrder(bpy.types.Operator):
    '''Switch order of the selected deformer'''
    bl_idname = "medusanodes.switch_order"
    bl_label = "Switch order"
    bl_options = {'UNDO'}

    action: StringProperty(default="")
    def execute(self, context):
        item_index = context.scene.medusaNodesTree_index
        item_list = context.scene.medusaNodesTree
        if item_index>=0 and len(item_list)>item_index:
            item = item_list[item_index]
            activeOb = bpy.data.objects[item.objectName]
            geoNodes = activeOb.modifiers[item.nodeTreeName]
            if geoNodes:
                if geoNodes.type == 'NODES':
                    activeNode = geoNodes.node_group.nodes[item.name]
                    activeNodeName = item.name
                    if "mn visual" not in activeNode.node_tree.name.lower() and "mn generator" not in activeNode.node_tree.name.lower():
                        if self.action == "DOWN":
                            nextIndex = item_index + 1
                            if len(item_list)>nextIndex:
                                nextItemF = None
                                for i in range(nextIndex,len(item_list)):
                                    nextItem = item_list[i]
                                    if nextItem.nodeTreeName == item.nodeTreeName and nextItem.objectName == item.objectName:
                                        nextNode = geoNodes.node_group.nodes[nextItem.name]
                                        if "mn visual" not in nextNode.node_tree.name.lower():
                                            nextItemF = nextItem
                                            break
                                if nextItemF:
                                    if "mn generator" not in nextNode.node_tree.name.lower():
                                        nextNode = geoNodes.node_group.nodes[nextItemF.name]
                                        nodeLinks = geoNodes.node_group.links
                                        swapNodes(nodeLinks,nextNode,activeNode)
                                        SetupNodeData()
                                        item_list = context.scene.medusaNodesTree
                                        activeIndex = bpy.context.scene.medusaNodesTree_index - 1
                                        
                                        for i in range(0,len(item_list)):
                                            tempItem = item_list[i]
                                            if tempItem.name == activeNodeName and tempItem.objectName == activeOb.name:
                                                activeIndex = i
                                                break

                                        bpy.context.scene.medusaNodesTree_index = activeIndex
                        elif self.action == "UP":
                            nextIndex = item_index - 1
                            if len(item_list)>nextIndex:
                                nextItemF = None
                                finList = []
                                for i in range(0,len(item_list)):
                                    tempItem = item_list[i]
                                    if tempItem.objectName == item.objectName:
                                        finList.append(tempItem)
                                
                                for i in range(0,len(finList)):
                                    tempItem = finList[i]
                                    if tempItem == item:
                                        if i > 1:
                                            nextNode = geoNodes.node_group.nodes[tempItem.name]
                                            if "mn visual" not in nextNode.node_tree.name.lower():
                                                nextItemF = finList[i-1]
                                                break

                                if nextItemF:
                                    nextNode = geoNodes.node_group.nodes[nextItemF.name]
                                    nodeLinks = geoNodes.node_group.links
                                    swapNodes(nodeLinks,activeNode,nextNode)
                                    SetupNodeData()
                                    
                                    item_list = context.scene.medusaNodesTree
                                    activeIndex = bpy.context.scene.medusaNodesTree_index - 1
                                    
                                    for i in range(0,len(item_list)):
                                        tempItem = item_list[i]
                                        if tempItem.name == activeNodeName and tempItem.objectName == activeOb.name:
                                            activeIndex = i
                                            break
                                    
                                    bpy.context.scene.medusaNodesTree_index = activeIndex
                        alignNodes(geoNodes.node_group)
        return {'FINISHED'}

class MEDUSANODES_UL_TreeItem(bpy.types.UIList):

    def draw_filter(self, context, layout):
        pass
    def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index):
        scene = data
        if self.layout_type in {'DEFAULT', 'COMPACT'}:
            for i in range(item.indent):
                split = layout.split(factor = 0.1)
            col = layout.column()
            
            if item.childCount == 0:
                op = col.operator("medusanodes.medusanodestree_expand", text="", icon='DOT', emboss=False)
                op.button_id = index
                col.enabled = False
            elif item.expanded:
                op = col.operator("medusanodes.medusanodestree_expand", text="", icon='TRIA_DOWN', emboss=False)
                op.button_id = index
            else:
                op = col.operator("medusanodes.medusanodestree_expand", text="", icon='TRIA_RIGHT', emboss=False)
                op.button_id = index
            col = layout.column()
            if item.objectName in bpy.data.objects:
                actOb = bpy.data.objects[item.objectName]
                if item.nodeTreeName in actOb.modifiers:
                    geoNodes = actOb.modifiers[item.nodeTreeName]
                    activeNode = geoNodes.node_group.nodes[item.name]
                    iconName = activeNode.node_tree.name.lower()[3:]

                    icon_id = 0
                    icon = 'NONE'
                    if "MedusaHair" in actOb:
                        if iconName in guidepcoll:
                            icon_id = guidepcoll[iconName].icon_id
                        if 'generator' in activeNode.node_tree.name.lower() and activeNode.inputs['Cached'].default_value:
                            icon_id = guidepcoll[iconName +' freeze'].icon_id
                        if 'noise' in activeNode.node_tree.name.lower() and activeNode.inputs['Cached'].default_value:
                            icon_id = guidepcoll[iconName +' freeze'].icon_id
                        if 'curl' in activeNode.node_tree.name.lower() and activeNode.inputs['Cached'].default_value:
                            icon_id = guidepcoll[iconName +' freeze'].icon_id
                        if 'curves to splines' in activeNode.node_tree.name.lower() and activeNode.inputs['Cached'].default_value:
                            icon_id = guidepcoll[iconName +' freeze'].icon_id
                    else:
                        if iconName in pcoll:
                            icon_id = pcoll[iconName].icon_id
                        if 'generator' in activeNode.node_tree.name.lower() and activeNode.inputs['Cached'].default_value:
                            icon_id = pcoll[iconName +' freeze'].icon_id
                        if 'noise' in activeNode.node_tree.name.lower() and activeNode.inputs['Cached'].default_value:
                            icon_id = pcoll[iconName +' freeze'].icon_id
                        if 'curl' in activeNode.node_tree.name.lower() and activeNode.inputs['Cached'].default_value:
                            icon_id = pcoll[iconName +' freeze'].icon_id
                        if 'guide' in activeNode.node_tree.name.lower() and activeNode.inputs['Cached'].default_value:
                            icon_id = pcoll[iconName +' freeze'].icon_id
                        if 'curves to splines' in activeNode.node_tree.name.lower() and activeNode.inputs['Cached'].default_value:
                            icon_id = pcoll[iconName +' freeze'].icon_id

                            

                    #add some color swatches or blender internal icons
                    # if iconName == 'generator':
                    #     icon_id = 0
                    #     icon='SEQUENCE_COLOR_01'

                    # if iconName == 'visualizer':
                    #     icon_id = 0
                    #     icon='SEQUENCE_COLOR_04'
                    # if iconName == 'visualizer guide':
                    #     icon_id = 0
                    #     icon='SEQUENCE_COLOR_04'
                    
                    #col.prop(item, "label", icon='SEQUENCE_COLOR_05', emboss=False, text=""
                            

                    #Grey out nodes below 'MN Dynamics'. 
                    enableItem = True
                    medusaNodesTree = context.scene.medusaNodesTree
                    #if the frame not at the start
                    if context.scene.frame_start != context.scene.frame_current:
                        if 'guide'in item.objectName.lower():

                            for node in medusaNodesTree:
                                if 'dynamics' in node.label.lower() and node.objectName == item.objectName and geoNodes.node_group.nodes[node.name].inputs['Disable'].default_value == False:
                                    enableItem = True
                                    if item.nodeIndex < node.nodeIndex:
                                        enableItem = False
                                    if item.objectName.lower() == item.label.lower():
                                        enableItem = True

                        else:
                            enableItem = True
                            for node in medusaNodesTree:
                                if 'dynamics' in node.label.lower() and node.objectName == item.objectName and geoNodes.node_group.nodes[node.name].inputs['Disable'].default_value == False:
                                    enableItem = True
                                    if item.nodeIndex < node.nodeIndex:
                                        enableItem = False
                                    if item.objectName.lower() == item.label.lower():
                                        enableItem = True
                    else:
                        enableItem = True

                    col.enabled = enableItem
                    col.prop(item, "label", icon_value= icon_id, icon = icon, emboss=False, text='')
                    
                    col = layout.column()
                    grid = col.row(align=True)
                    if not enableItem:
                        grid.enabled = enableItem
                    if check_valid(item):
                        geoNodes = bpy.data.objects[item.objectName].modifiers[item.nodeTreeName]
                        if geoNodes:
                            if geoNodes.type == 'NODES':
                                activeNode = geoNodes.node_group.nodes[item.name]

                                if "mn visual" in activeNode.node_tree.name.lower():
                                    ob = bpy.data.objects[item.objectName]
                                    for node in geoNodes.node_group.nodes:
                                        if node.type =='GROUP' and node.node_tree.name.lower() == 'mn generator':
                                            # if node.inputs['Cached'].default_value:
                                            row = grid.row()
                                            row.enabled = True
                                            if actOb.mode == 'SCULPT_CURVES':
                                                row.operator("medusanodes.togglesculpt", text="", icon='SCULPTMODE_HLT', emboss=True, depress=True).itemName = item.objectName
                                            else:
                                                row.operator("medusanodes.togglesculpt", text="", icon='SCULPTMODE_HLT', emboss=False, depress=False).itemName = item.objectName
                                            # else:  
                                            #     row = grid.row()
                                            #     row.enabled = False
                                            #     row.operator("medusanodes.togglesculpt", text="", icon='SCULPTMODE_HLT', emboss=False, depress=False).itemName = item.objectName   
                                    if ob.hide_get():
                                        grid.operator("medusanodes.hide", text="", icon='HIDE_ON', emboss=False, depress=False).itemName = item.objectName
                                    else:
                                        grid.operator("medusanodes.hide", text="", icon='HIDE_OFF', emboss=False, depress=False).itemName = item.objectName
                                
                                if "mn clump" in activeNode.node_tree.name.lower() or "mn guide" in activeNode.node_tree.name.lower() or "mn braid" in activeNode.node_tree.name.lower():
                                    grid.separator()
                                    #grid.operator_menu_enum(MedusaNodeTreeGuidesMenu.bl_idname, 'guideItems', text="",icon='SETTINGS').index = index

                                    grid.label(text='', icon='BLANK1')

                                if "mn visual" in activeNode.node_tree.name.lower():
                                    if bpy.data.objects[item.objectName].hide_viewport:
                                        grid.operator("medusanodes.hide_viewport", icon='RESTRICT_VIEW_ON', text="", emboss=False, depress=False).objectName = item.objectName
                                    else:
                                        grid.operator("medusanodes.hide_viewport", icon='RESTRICT_VIEW_OFF', text="", emboss=False, depress=False).objectName = item.objectName
                                else:
                                    for inp in activeNode.inputs:
                                        if inp.name.lower() == "disable":
                                            if inp.default_value:
                                                op = grid.operator("medusanodes.disable", text="", icon='RESTRICT_VIEW_ON', emboss=False, depress=False)
                                                op.itemName = item.name
                                                op.objectName = item.objectName
                                                op.nodeTreeName = item.nodeTreeName
                                            else:
                                                op = grid.operator("medusanodes.disable", text="", icon='RESTRICT_VIEW_OFF', emboss=False, depress=False)
                                                op.itemName = item.name
                                                op.objectName = item.objectName
                                                op.nodeTreeName = item.nodeTreeName
                                            break

class SCENE_PT_MedusaNodesMain(bpy.types.Panel):
    bl_label = "Medusa Nodes Menu"
    bl_idname = "SCENE_PT_medusanodes_main"
    bl_space_type = 'NODE_EDITOR'
    bl_region_type = "UI"
    bl_category = "Medusa Nodes"

    # @classmethod
    # def poll(cls, context):
    #     snode = context.space_data
    #     return snode.tree_type == 'GeometryNodeTree'


    # def draw_header(self, context: bpy.types.Context):
    #     self.layout.label(text="", icon_value=pcoll["medusa_nodes_logo"].icon_id)
    
    def draw(self ,context):
        layout = self.layout
        row = layout.column_flow(columns = 3,align=True)
       
        row1 = row.row()
        row1.scale_y = 2
        row1.alignment = 'CENTER'
        row1.label(text = '')
        
        row2 = row.row()
        row2.alignment = 'CENTER'
        row2.template_icon(icon_value=pcoll["medusa_nodes_logo"].icon_id, scale=2)
        
        row3 = row.row(align = True)
        row3.alignment = 'RIGHT'
        
        version_box = row3.box()
        #version_box.separator(factor =-0.1 )
        version_box.scale_y = .34
        version_box.enabled = False
        
        version_row = version_box.row(align=True)
        version_row.alignment = 'RIGHT'
        version_row.label(text="Medusa Nodes")
      
        second_row = version_box.row(align=True)
        second_row.alignment = 'RIGHT'
        second_row.label(text="v1.1.0-beta")
 
        col = layout.column(align=True)
        row = col.row(align=True)
        row.operator("medusanodes.main_new_groom", icon="STRANDS")
        row = col.row(align=True)
        row.operator("medusanodes.main_dup_groom", icon="DUPLICATE")
        row = col.row(align=True)
        row.operator("medusanodes.main_del_groom", icon="TRASH")
        row = col.row(align=True)
        # row = layout.row(align=True)
        # row.prop(context.scene, "use_medusa_hair_object")


class SCENE_PT_MedusaNodesMain_Viewport(SCENE_PT_MedusaNodesMain):
    bl_label = "Medusa Nodes Menu"
    bl_idname = "SCENE_PT_medusanodes_main_view"
    bl_space_type = 'VIEW_3D'

class MedusaNodesMainNewGroomFunc(bpy.types.Operator):
    '''Create new groom with the defined parameters'''
    bl_idname = "medusanodes.main_new_groom_func"
    bl_label = "Create Groom"
    bl_options = {'UNDO'}

    groomName : StringProperty(default="",
        name="",
        description="Groom to duplicate Name")
    partName : StringProperty(default="",
        name="",
        description="Part to duplicate Name")
    seed : IntProperty(default=0,
        name="Seed",
        description="Generator seed")
    controlPointsCount : IntProperty(default=16,
        name="Control Point Count",
        description="Generator control point count")
    density : FloatProperty(default=100,
        name="Density",
        description="Generator density")
    
    x: bpy.props.IntProperty()
    y: bpy.props.IntProperty()

    def invoke(self, context, event):
        self.x = event.mouse_x
        self.y = event.mouse_y
        return self.execute(context)

    def execute(self, context):
        scalpOb = context.object
        groomCol = None

        if scalpOb.scale[0] != 1.0:
            self.report({'WARNING'}, "Make sure that the mesh object's scale is 1.0! Otherwise you will encounter various issues with the addon")

        scalpObPolygons=scalpOb.data.polygons

        #calculate geometry area
        area = 0.0
        for p in scalpObPolygons:
            area += p.area
        areaFactor = 1 / area
        #calculate dimensions   
        dimensions = scalpOb.dimensions
        dimensions = dimensions[0]+dimensions[1]+dimensions[2]

        hairNodeData = {
            "Seed":self.seed,
            "Control Point Count":self.controlPointsCount,
            "":self.density,
            "Density":self.density,
            "Spline Length":dimensions * 0.1,
            "Hair Thickness":dimensions * 0.0015,
            }

        newGroomName = self.groomName
        newGroomPartName = self.partName
        if newGroomPartName == "":
            newGroomPartName = "Part01"
        if len(context.object.data.uv_layers)==0:
            context.object.data.uv_layers.new(name="surface_uv_coordinate")
        else:
            context.object.data.uv_layers[0].name ="surface_uv_coordinate"
        scalpOb = context.object
        groomCol = None

        partIndex = -1

        for ob in bpy.data.objects:
            if ob.type == "CURVES":
                if newGroomName in ob.name:
                    nameParts = ob.name.split("_")
                    if len(nameParts)>1:
                        if newGroomName == "_".join(nameParts[:-1]):
                            namePrt = nameParts[-1]
                            if newGroomPartName in namePrt.lower():
                                namePrtI = namePrt.lower().replace(newGroomPartName,"")
                                if namePrtI.isdigit():
                                    if int(namePrtI)>partIndex:
                                        partIndex = int(namePrtI)
                                else:
                                    partIndex = 0
        if partIndex != -1:
            partStrInde = str(partIndex+1).zfill(2)
            if partStrInde not in newGroomPartName and not newGroomPartName[-2:].isdigit():
                newGroomPartName = newGroomPartName+partStrInde

        for col in bpy.data.collections:
            if col.name == newGroomName:
                groomCol = col
                break
        if groomCol == None:
            groomCol = bpy.data.collections.new(name=newGroomName)
            bpy.context.scene.collection.children.link(groomCol)
        bpy.ops.object.mode_set(mode="OBJECT")
        bpy.ops.object.select_all(action="DESELECT")
        scalpObDisabled = scalpOb.hide_viewport
        scalpObHidden = scalpOb.hide_get()

        scalpOb.hide_set(False)
        scalpOb.hide_viewport = False
        bpy.context.view_layer.objects.active = scalpOb
        scalpOb.select_set(True)
        newGroomName = groomCol.name
        bpy.ops.object.curves_empty_hair_add(align='WORLD', scale=(1, 1, 1))
        
        scalpOb.hide_set(scalpObHidden)
        scalpOb.hide_viewport = scalpObDisabled
        hairOb = bpy.context.object
        for col in hairOb.users_collection:
            try:
                col.objects.unlink(hairOb)
            except Exception as E:
                pass
        try:
            groomCol.objects.link(hairOb)
        except:pass

        hairOb.name  = newGroomName + "_" + newGroomPartName
        setupHairOb(hairOb,scalpOb,hairNodeData)
        bpy.context.window.cursor_warp(self.x, self.y-100)

        #expand the hierarchy when the new groom is created
        bpy.ops.medusanodes.medusanodestree_expand()

        return {'FINISHED'}

def checkExistingPart(self,context):
    newGroomPartName = context.scene.newGroomPartName

    partIndex = -1
    for ob in bpy.data.objects:
        if ob.type == "CURVES":
            nameParts = ob.name.split("_")
            if context.scene.newGroomName in ob.name and newGroomPartName in ob.name and "guide" not in ob.name:
                if len(nameParts)>1:
                    namePrt = nameParts[-1]
                    if newGroomPartName.lower() in namePrt.lower():
                        namePrtI = namePrt.lower().replace(newGroomPartName,"")[-2:]
                        if namePrtI.isdigit():
                            if int(namePrtI)>partIndex:
                                partIndex = int(namePrtI)
                        else:
                            partIndex = 0
    if partIndex != -1:
        partStrInde = str(partIndex+1).zfill(2)
        if newGroomPartName[-2:] != partStrInde and not newGroomPartName[-2:].isdigit():
            context.scene.newGroomPartName = newGroomPartName+partStrInde
        elif newGroomPartName[-2:] != partStrInde:
            context.scene.newGroomPartName = newGroomPartName[:-2]+partStrInde

class MedusaNodesMainNewGroom(bpy.types.Operator):
    '''Create the new 'Medusa Nodes' groom'''
    bl_idname = "medusanodes.main_new_groom"
    bl_label = "New Groom"
    bl_options = {'UNDO'}

    seed : IntProperty(default=0,
        name = "Seed",
        description = "Distribution seed of the new groom. Choose a different value to get a different distirbution")
    controlPointsCount : IntProperty(default=16,
        name = "Control Point Count",
        description = "'Control Point Count' of the new groom. Amount of points on individual spline (hair folicle)")
    density : FloatProperty(default = 1000,
        name = "Density",
        description = "'Density' value of the new groom. The value doesn't represent exact hair count as it depends on the Emitter Geometry's area size")
    hairCount : FloatProperty(default = 0,
        name = "Approximate Hair Count",
        description = "Rough count approximation of the distributed splines (hair folicles)")
    activeNameProp : BoolProperty(default = False,
        name = "Active Name Property",
        description = "")

    def draw(self, context):
        layout = self.layout
        layout.use_property_decorate = False
        layout.separator(factor=1.5)
       
        row = layout.row()
        row.alignment = 'CENTER'
        row.label(text="New 'Medusa Nodes' Groom", icon_value=pcoll["medusa_nodes_logo"].icon_id) # ' icon = 'STRANDS''

        layout.separator(factor=1)

        row1 = layout.row(align = True)
        col = row1.column(align = True)

        row = col.row()
        row.separator(factor=2)
        row.label(text="Groom's Name")
        #row.separator(factor=2)

        row = col.row()
        row.separator(factor=2)
        row.prop(context.scene, 'newGroomName')
        #row.separator(factor=2)

        row1.separator(factor=1.5)
        col = row1.column(align = True)
        #layout.separator(factor=0.5)

        row = col.row()
        #row.separator(factor=2)
        row.label(text="Groom Part's Name")
        row.separator(factor=2)
        row = col.row()
        #row.separator(factor=2)          
        newGroomProp= row.prop(context.scene, 'newGroomPartName')
        row.separator(factor=2)
       

        layout.separator(factor=0.5)

        
        row = layout.row()
        row.separator(factor=2)
        col = row.column(align = True)
        row.separator(factor=2)
        row = col.row()
        row.label(text='The new groom will be named:')
        row = col.row()
        row.label(text=('"{}_{}"').format(context.scene.newGroomName, context.scene.newGroomPartName))
        

        layout.separator(factor=0.5)

        row = layout.row()
        row.separator(factor=2)
        row.label(text="Groom's Parameters")
        row.separator(factor=2)
        row = layout.row()
        row.separator(factor=2)
        
        box = row.box()
        box.prop(self, 'density') 
        box.prop(self, 'controlPointsCount')
        box.prop(self, 'seed')
        row.separator(factor=2)

        layout.separator(factor=0.5)

        #Dynamically update the hairCount
        if context.object:
            if context.object.type == "MESH":
                factor = areaFactor(context.object)
                self.hairCount = self.density / factor
    
        row = layout.row()
        row.separator(factor=2)
        row.label(text = 'Approximate Calculation')
        
        row = layout.row()
        #row.alignment = 'CENTER'
        row.separator(factor=2)
        box = row.box()
        box.emboss = 'NONE'
        
        boxCol = box.column(align=True)
        #boxCol.alignment = 'CENTER'A
        boxCol.label(text='Point Count ≈ {:,}'.format(int(self.hairCount * self.controlPointsCount)), icon = 'OUTLINER_OB_POINTCLOUD')
        boxCol.label(text='Spline Count ≈ {:,}'.format(int(self.hairCount)), icon = 'OUTLINER_OB_CURVES')
        row.separator(factor=2)
        #row.separator_spacer()
        layout.separator(factor=2)
        row = layout.row()
        row.scale_y= 2.0

        row.separator(factor=2)
        op = row.operator("medusanodes.main_new_groom_func")
        row.separator(factor=2)

        op.groomName = context.scene.newGroomName
        op.partName = context.scene.newGroomPartName
        op.seed = self.seed
        op.controlPointsCount = self.controlPointsCount
        op.density = self.density

        layout.separator(factor=2.5)

    def execute(self, context):
        newGroomName = context.scene.newGroomName

        groomIndex = -1
        groomCpt = 0

        for ob in bpy.data.objects:
            if ob.type == "CURVES":
                nameParts = ob.name.split("_")
                if newGroomName in ob.name and "guide" not in ob.name:
                    groomCpt = groomCpt + 1
                    if len(nameParts)>0:
                        groomPrt = nameParts[0][-2:]
                        if groomPrt.isdigit():
                            if int(groomPrt)>groomIndex:
                                groomIndex = int(groomPrt)
                        else:
                            groomIndex = 0

        if groomIndex != -1:
            groomStrInde = str(groomIndex+1).zfill(2)
            if newGroomName[-2:] != groomStrInde and not newGroomName[-2:].isdigit():
                context.scene.newGroomName = newGroomName+groomStrInde
            elif newGroomName[-2:] != groomStrInde:
                context.scene.newGroomName = newGroomName[:-2]+groomStrInde

        return {'FINISHED'}
    
    def invoke(self,context,event):
        #self.execute(context)
        if context.object:
                
            if context.object.type == "MESH":

                if len(context.object.data.uv_layers) != 0:

                    context.scene.render.hair_type = 'STRIP'
                    context.scene.render.hair_subdiv = 2 
                    
                    #Geometry Area Factor
                    factor = areaFactor(context.object)
                    self.density = factor * 10000
                    self.hairCount = self.density / factor
                    
                    self.execute(context)
                    return context.window_manager.invoke_popup(self)

                else:
                    self.report({'WARNING'}, "Make sure that the mesh object has proper UVs first!")
                    return {'FINISHED'}   
            else:
                self.report({'WARNING'}, 'Only mesh objects can be used for hair distribution, select the mesh object first!')
                return {'FINISHED'}
        else:
            self.report({'WARNING'}, 'Select the mesh object first!')
            return {'FINISHED'}

def copyColl_objects(from_col, to_col, linked, dupe_lut):
    for o in from_col.objects:
        dupe = o.copy()
        if not linked and o.data:
            dupe.data = dupe.data.copy()
        dupe.name = o.name.replace(from_col.name,to_col.name)
        to_col.objects.link(dupe)
        dupe_lut[o] = dupe
        bpy.context.view_layer.objects.active = dupe
        dupe.select_set(True)

def copyColl(parent, collection, linked=False):
    dupe_lut = defaultdict(lambda : None)
    
    bpy.ops.object.select_all(action="DESELECT")
    def _copy(parent, collection, linked=False):
        copyColl_objects(collection, parent, linked, dupe_lut)
        for c in collection.children:
            _copy(parent, c, linked)

    _copy(parent, collection, linked)
    for o, dupe in tuple(dupe_lut.items()):
        parent = dupe_lut[o.parent]
        if parent:
            dupe.parent = parent

class MedusaNodesMainDuplGroomFunc(bpy.types.Operator):
    '''Duplicate entire groom or only the part'''
    bl_idname = "medusanodes.main_dup_groom_func"
    bl_label = "Duplicate Groom"
    bl_options = {'UNDO'}

    groomToDuplicate : StringProperty(default="",
        name="",
        description="Groom to duplicate Name")
    partToDuplicate : StringProperty(default="",
        name="",
        description="Part to duplicate Name")
    duplOnlyGroom : BoolProperty(default=False,
        name="Duplicate Only Groom Part",
        description="Duplicate Only Groom Part")
    newGroomName : StringProperty(default="Groom02",
        name="",
        description="New Name of the duplicated groom")
    newPartName : StringProperty(default="Part02",
        name="",
        description="New Name of the duplicated part")

    def execute(self, context):
        col = bpy.data.collections.get(self.groomToDuplicate)
        if col:
            if not self.duplOnlyGroom and self.newGroomName != "":
                newCol = bpy.data.collections.new(name=self.newGroomName)
                bpy.context.scene.collection.children.link(newCol)
                # newCol.name = self.newGroomName
                copyColl(newCol, col)
                mainGroomHairs = []
                groomGuides = []
                for ob in newCol.objects:
                    if "MedusaGroom" in ob:
                        mainGroomHairs.append(ob)
                    elif ob.type == "CURVES":
                        if "MedusaHair" in ob:
                            groomGuides.append(ob)
                    for mod in ob.modifiers:
                        if mod.type == "NODES" and mod.name == "MedusaNodes":
                            mod.node_group = mod.node_group.copy()
                            mod.node_group.name = ob.name
                            for node in mod.node_group.nodes:
                                if node.type == "GROUP" and "mask group" in node.label.lower():
                                    node.node_tree = node.node_tree.copy()
                print(groomGuides)
                for guide in groomGuides:
                    groomName = guide.name.split("_")[0]
                    guideName = guide.name.split("_")[-1]
                    for groom in mainGroomHairs:
                        if groomName in groom.name:
                            for mod in groom.modifiers:
                                if mod.type == "NODES" and mod.name == "MedusaNodes":
                                    hSplineNode = None
                                    for node in mod.node_group.nodes:
                                        if node.type == "GROUP" and "mask group" in node.label.lower():
                                            node.node_tree = node.node_tree.copy()
                                        if node.type == "GROUP" and "generator" in node.node_tree.name.lower():
                                            hSplineNode = node
                                    for gInp in mod.node_group.interface.items_tree:
                                        # if gInp.name.lower() == "guide curves":
                                        if gInp.name.lower() == guideName.lower():
                                            mod[gInp.identifier] = guide
                                            guide["MedusaHair"] = groom
                                            for guid_mod in guide.modifiers:
                                                if guid_mod.type == "NODES" and guid_mod.name == "MedusaNodes":
                                                    guideGenNode = None
                                                    for guid_node in guid_mod.node_group.nodes:
                                                        if guid_node.type == "GROUP" and "generator" in guid_node.node_tree.name.lower():
                                                            guideGenNode = guid_node
                                                            break
                                                    if guideGenNode and hSplineNode:
                                                        d = None
                                                        var1 = None
                                                        if guid_mod.node_group.animation_data:
                                                            if len(guid_mod.node_group.animation_data.drivers)>0:
                                                                d = guid_mod.node_group.animation_data.drivers[0]
                                                                var1 = d.driver.variables[0]
                                                        if d is None:
                                                            d = guideGenNode.inputs["Control Point Count"].driver_add("default_value")
                                                            var1 = d.driver.variables.new()
                                                            var1.name = "AVERAGE"
                                                        var1.targets[0].id_type = 'NODETREE'
                                                        var1.targets[0].id = mod.node_group
                                                        var1.targets[0].data_path = 'nodes["'+hSplineNode.name+'"].inputs["Control Point Count"].default_value'
                                                        d.driver.expression = var1.name
                                    break
                            break
            elif self.partToDuplicate != "":
                groomGuides = []
                for o in col.objects:
                    if self.partToDuplicate in o.name.rsplit('_')[1] and self.groomToDuplicate in o.name:
                        dupe = o.copy()
                        dupe.data = dupe.data.copy()
                        cleanName = o.name.replace(self.partToDuplicate,self.newPartName)
                        dupe.name = cleanName
                        col.objects.link(dupe)
                        hSplineNode = None
                        for mod in dupe.modifiers:
                            if mod.type == "NODES" and mod.name == "MedusaNodes":
                                mod.node_group = mod.node_group.copy()
                                mod.node_group.name = dupe.name
                                for node in mod.node_group.nodes:
                                    if node.type == "GROUP" and "mask group" in node.label.lower():
                                        node.node_tree = node.node_tree.copy()
                                    if node.type == "GROUP" and "generator" in node.node_tree.name.lower():
                                        hSplineNode = node
                                for socket in mod.keys():
                                    sockOb = mod[socket]
                                    if sockOb:
                                        if "MedusaHair" in sockOb:
                                            dupeGuide = sockOb.copy()
                                            dupeGuide.data = dupeGuide.data.copy()
                                            cleanName = sockOb.name.replace(self.partToDuplicate,self.newPartName)
                                            dupeGuide.name = cleanName
                                            for guide_mod in dupeGuide.modifiers:
                                                if guide_mod.type == "NODES" and guide_mod.name == "MedusaNodes":
                                                    guide_mod.node_group = guide_mod.node_group.copy()
                                                    guide_mod.node_group.name = dupeGuide.name
                                                    for guide_node in guide_mod.node_group.nodes:
                                                        if guide_node.type == "GROUP" and "mask group" in guide_node.label.lower():
                                                            guide_node.node_tree = guide_node.node_tree.copy()
                                            col.objects.link(dupeGuide)
                                            mod[socket] = dupeGuide
                                            dupeGuide["MedusaHair"] = dupe
                                            for guid_mod in dupeGuide.modifiers:
                                                if guid_mod.type == "NODES" and guid_mod.name == "MedusaNodes":
                                                    guideGenNode = None
                                                    for guid_node in guid_mod.node_group.nodes:
                                                        if guid_node.type == "GROUP" and "generator" in guid_node.node_tree.name.lower():
                                                            guideGenNode = guid_node
                                                            break
                                                    if guideGenNode and hSplineNode:
                                                        d = None
                                                        var1 = None
                                                        if guid_mod.node_group.animation_data:
                                                            if len(guid_mod.node_group.animation_data.drivers)>0:
                                                                d = guid_mod.node_group.animation_data.drivers[0]
                                                                var1 = d.driver.variables[0]
                                                        if d is None:
                                                            d = guideGenNode.inputs["Control Point Count"].driver_add("default_value")
                                                            var1 = d.driver.variables.new()
                                                            var1.name = "AVERAGE"
                                                        var1.targets[0].id_type = 'NODETREE'
                                                        var1.targets[0].id = mod.node_group
                                                        var1.targets[0].data_path = 'nodes["'+hSplineNode.name+'"].inputs["Control Point Count"].default_value'
                                                        d.driver.expression = var1.name
                        break
        else:
            self.report({'WARNING'}, 'No groom named "'+ self.groomToDuplicate +'" has been found! Enter the groom name you would like to duplicate!')
            return {'FINISHED'} 

        return {'FINISHED'}

def sceneGroomsItems(self, context):
    Grooms = []
    for ob in bpy.data.objects:
        if ob.type == "CURVES":
            if "MedusaGroom" in ob:
                groomName = ob.name.split("_")[0]
                groomItem = (groomName,groomName,'')
                if groomItem not in Grooms:
                    Grooms.append(groomItem)
    return Grooms

def sceneGroomsPartItems(self, context):
    Parts = []
    for ob in bpy.data.objects:
        if ob.type == "CURVES":
            if "MedusaGroom" in ob:
                partName = ob.name.split("_")[1]
                partItem = (partName,partName,'')
                if partItem not in Parts:
                    Parts.append(partItem)
    return Parts

def sceneGroomsPartItems2(self, context):
    Parts = []
    Parts.append(("All","All",""))
    for ob in bpy.data.objects:
        if ob.type == "CURVES":
            if "MedusaGroom" in ob:
                partName = ob.name.split("_")[1]
                partItem = (partName,partName,'')
                if partItem not in Parts:
                    Parts.append(partItem)
    return Parts


class MedusaNodesMainDuplGroom(bpy.types.Operator):
    '''Duplicate the 'Medusa Nodes' groom.'''
    bl_idname = "medusanodes.main_dup_groom"
    bl_label = "Duplicate Groom"
    bl_options = {'UNDO'}

    groomsToDup : EnumProperty(name="",
        description= "List of all Medusa Nodes' grooms",
        items=sceneGroomsItems)
    newGroomName : StringProperty(default="Groom02",
        name="",
        description="New name of the duplicated groom")
    duplOnlyGroom : BoolProperty(default=False,
        name="Duplicate Only Groom Part",
        description="Duplicate Only Groom Part")
    partsToDup : EnumProperty(name="",
        description= "List of the selected groom's parts",
        items=sceneGroomsPartItems)
    newPartName : StringProperty(default="Part02",
        name="",
        description="New name of the duplicated part")

    def draw(self, context):
        layout = self.layout
        layout.use_property_decorate = False
        layout.label(text='Duplicate Groom')

        col = layout.column()
        row = col.row()
        row.label(text="Choose Groom to Duplicate")
        row.label(text="New Groom Name")
        col = layout.column()
        row = col.row()
        row.prop(self, 'groomsToDup')
        row.prop(self, 'newGroomName')

        t = False
        if self.duplOnlyGroom:
            t = True
        row = layout.row()
        row.prop(self, 'duplOnlyGroom')

        col = layout.column()
        row = col.row()
        row.label(text="Choose Part to Duplicate")
        row.label(text="New Part Name")
        col = layout.column()
        row = col.row()
        row.prop(self, 'partsToDup')
        row.prop(self, 'newPartName')
        row.enabled = True
        layout.separator(factor=1.0)

        row = layout.row()
        row.scale_y= 1.5 
        op = row.operator("medusanodes.main_dup_groom_func")
        op.groomToDuplicate = self.groomsToDup
        op.partToDuplicate = self.partsToDup
        op.newGroomName = self.newGroomName
        op.newPartName = self.newPartName
        op.duplOnlyGroom = self.duplOnlyGroom

    def execute(self, context):
        return {'FINISHED'}
    
    def invoke(self,context,event):
        return context.window_manager.invoke_popup(self)


class MedusaNodesMainDelGroomFunc(bpy.types.Operator):
    '''Delete entire groom or only the part'''
    bl_idname = "medusanodes.main_del_groom_func"
    bl_label = "Delete Groom"
    bl_options = {'UNDO'}

    groomToDelete : StringProperty(default="",
        name="",
        description="Groom to duplicate Name")
    partToDelete : StringProperty(default="",
        name="",
        description="Part to duplicate Name")

    def groupDepth(self,nodeGrp):
        for subN in nodeGrp.node_tree.nodes:
            if subN.type == "GROUP":
                if subN.node_tree.users == 1:
                    self.groupDepth(subN)
        bpy.data.node_groups.remove(nodeGrp.node_tree)

    def execute(self, context):
        if self.groomToDelete != "":
            part = ""
            if self.partToDelete != "All":
                part = self.partToDelete
            objScene = bpy.data.objects
            cleanName = self.groomToDelete
            k = {}
            for mat in bpy.data.materials:
               k[mat.name] = mat.users
                
            for o in objScene:
                if "MedusaHair" in o or "MedusaGroom" in o:
                    if cleanName in o.name and part in o.name:
                        cols = o.users_collection
                        for mod in o.modifiers:
                            if mod.type == "NODES" and "medusanodes" in mod.name.lower():
                                for node in mod.node_group.nodes:
                                    if node.type == "GROUP":
                                        if node.node_tree.users == 1:
                                            self.groupDepth(node)
                                mod.node_group.use_fake_user = False
                                if mod.node_group.users == 1:
                                    bpy.data.node_groups.remove(mod.node_group)
                        try:
                            bpy.data.hair_curves.remove(o.data)
                        except:
                            pass
                        try:
                            bpy.data.objects.remove(o)
                        except:
                            pass
                        for col in cols:
                            if len(col.objects) == 0:
                                bpy.data.collections.remove(col)
                        for node in bpy.data.node_groups:
                            if node.users == 0:
                                bpy.data.node_groups.remove(node)

            for mat in bpy.data.materials:
                if mat.name in k:
                    if mat.users != k[mat.name]:
                        if mat.users == 0:
                            bpy.data.materials.remove(mat)
                    elif mat.users == 0 and "groom" in mat.name.lower():
                        bpy.data.materials.remove(mat)

        return {'FINISHED'}


class MedusaNodesMainDelGroom(bpy.types.Operator):
    '''Delete 'Medusa Nodes' groom.'''
    bl_idname = "medusanodes.main_del_groom"
    bl_label = "Delete Groom"
    bl_options = {'UNDO'}

    groomsToDel : EnumProperty(name="",
        description= "List of grooms to duplicate",
        items=sceneGroomsItems)
    partsToDel : EnumProperty(name="",
        description= "List of parts to duplicate",
        items=sceneGroomsPartItems2)

    def draw(self, context):
        layout = self.layout
        layout.use_property_decorate = False
        layout.label(text='Delete Groom or Part')

        col = layout.column()
        row = col.row()
        row.label(text="Choose Groom To Delete")
        col = layout.column()
        row = col.row()
        row.prop(self, 'groomsToDel')

        col = layout.column()
        row = col.row()
        row.label(text="Choose Part To Delete")
        col = layout.column()
        row = col.row()
        row.prop(self, 'partsToDel')

        layout.separator(factor=1.0)

        row = layout.row()
        row.scale_y= 1.5
        op = row.operator("medusanodes.main_del_groom_func")
        op.groomToDelete = self.groomsToDel
        op.partToDelete = self.partsToDel

    def execute(self, context):
        return {'FINISHED'}

    def invoke(self,context,event):
        return context.window_manager.invoke_popup(self)


class MedusaNodesPaintMode(bpy.types.Operator):
    '''Switch to the 'Paint Mode' to paint on the this texture map by clicking on it. Click on it again to exit the 'Paint Mode'.'''
    bl_idname = "medusanodes.paint_mode"
    bl_label = "Paint Mode Toggle"
    bl_options = {'UNDO'}

    socketName: StringProperty(default="")
    groupNodeName: StringProperty(default="")
    isInTextureMode: BoolProperty(default=False)

    def execute(self, context):
        global paintModeLstOb
        item_index = context.scene.medusaNodesTree_index
        item_list = context.scene.medusaNodesTree
        item = item_list[item_index]  
        if check_valid(item):
            geoNodes = bpy.data.objects[item.objectName].modifiers[item.nodeTreeName]
            if geoNodes:
                if geoNodes.type == 'NODES':
                    activeNode = geoNodes.node_group.nodes[self.groupNodeName]
                    for gInp in geoNodes.node_group.interface.items_tree:
                        if gInp.name.lower() == "emitter object" or gInp.name.lower() == "emitter geometry":
                            scalpOb = geoNodes[gInp.identifier]
                            break
        if bpy.context.object.mode == "TEXTURE_PAINT":
            bpy.ops.object.mode_set(mode="TEXTURE_PAINT",toggle=True)
            if paintModeLstOb:
                oldVis = paintModeLstOb.hide_get()
                oldVie = paintModeLstOb.hide_viewport

                #so the hair object would not stay selected
                bpy.ops.object.select_all(action='DESELECT')

                paintModeLstOb.hide_set(False)
                paintModeLstOb.hide_viewport = False
                paintModeLstOb.select_set(True)
                bpy.context.view_layer.objects.active = paintModeLstOb
                if "lastMode" in paintModeLstOb:
                    if paintModeLstOb["lastMode"] != "":
                        bpy.ops.object.mode_set(mode=paintModeLstOb["lastMode"])
                        paintModeLstOb["lastMode"] = ""
                paintModeLstOb.hide_set(oldVis)
                paintModeLstOb.hide_viewport = oldVie
                scalpOb["hairOb"] = paintModeLstOb.name
                scalpOb["paintMode"] = False
                #for the depressed status of the button
                self.isInTextureMode= True
        else:
            paintModeLstOb = bpy.context.object
            if scalpOb:
                paintModeLstOb["lastMode"] = bpy.context.object.mode
                bpy.ops.object.mode_set(mode="OBJECT")
                scalpOb["hairOb"] = paintModeLstOb.name
                scalpOb["paintMode"] = True
                oldVis = scalpOb.hide_get()
                oldVie = scalpOb.hide_viewport

                #so the hair object would not stay selected
                bpy.ops.object.select_all(action='DESELECT')

                scalpOb.hide_set(False)
                scalpOb.hide_viewport = False
                scalpOb.select_set(True)
                
                bpy.context.view_layer.objects.active = scalpOb
                bpy.ops.object.mode_set(mode="TEXTURE_PAINT",toggle=True)
                bpy.ops.wm.tool_set_by_id(name="builtin_brush.Draw")

                #Paint conditions
                bpy.context.scene.tool_settings.image_paint.brush.curve_preset = 'CUSTOM'
               
                if 'region map' in activeNode.outputs[0].links[0].to_socket.name.lower():
                    bpy.context.scene.tool_settings.image_paint.brush.curve_preset = 'CONSTANT'

                    #Region map needs 'sharp' borders, hence the 'closest' interpolation is used.
                    for node in activeNode.node_tree.nodes:
                        if node.type == 'IMAGE_TEXTURE':
                            node.interpolation = 'Closest'
 
                bpy.context.scene.tool_settings.image_paint.mode = "IMAGE"
                bpy.context.scene.tool_settings.image_paint.canvas = activeNode.inputs[self.socketName].default_value
                scalpOb.hide_set(oldVis)
                scalpOb.hide_viewport = oldVie
                #for the depressed status of the button
                self.isInTextureMode= False
        return {'FINISHED'}

class MedusaNodesHideViewport(bpy.types.Operator):
    '''Enable or disable this Medusa Nodes' object in viewport'''
    bl_idname = "medusanodes.hide_viewport"
    bl_label = "Enable / Disable in Viewport"
    bl_options = {'UNDO'}

    objectName: StringProperty(default="")

    def execute(self, context):
        ob = bpy.data.objects.get(self.objectName)
        if ob:
            ob.hide_viewport = not ob.hide_viewport
        return {'FINISHED'}

class SCENE_PT_MedusaNodesMainRigging(bpy.types.Panel):
    bl_label = "Rigging"
    bl_idname = "SCENE_PT_medusanodes_mainrigging"
    bl_space_type = 'NODE_EDITOR'
    bl_region_type = "UI"
    bl_category = "Medusa Nodes"
    bl_parent_id = "SCENE_PT_medusanodes_main"
    bl_options = {"DEFAULT_CLOSED"}

    @classmethod
    def poll(cls, context):
        snode = context.space_data
        return snode.tree_type == 'GeometryNodeTree'
    
    def draw(self ,context):
        pass

class SCENE_PT_MedusaNodesMainRiggingView(SCENE_PT_MedusaNodesMainRigging):
    bl_idname = "SCENE_PT_medusanodes_mainrigging_view"
    bl_space_type = 'VIEW_3D'
    bl_parent_id = "SCENE_PT_medusanodes_hierarchy_vie"

class SCENE_PT_MedusaNodesMainRenVie_curves(bpy.types.Panel):
    bl_label = "Curves"
    bl_space_type = 'NODE_EDITOR'
    bl_region_type = "UI"
    bl_category = "Medusa Nodes"
    bl_parent_id = "SCENE_PT_medusanodes_mainrenvie"
    bl_options = {'DEFAULT_CLOSED'}
    COMPAT_ENGINES = {'CYCLES'}

    @classmethod
    def poll(cls, context):
        return context.engine in cls.COMPAT_ENGINES

    def draw(self, context):
        layout = self.layout
        layout.use_property_split = True
        layout.use_property_decorate = False

        scene = context.scene
        ccscene = scene.cycles_curves

        col = layout.column()
        col.prop(ccscene, "shape", text="Shape")
        if ccscene.shape == 'RIBBONS':
            col.prop(ccscene, "subdivisions", text="Curve Subdivisions")

class SCENE_PT_MedusaNodesHierarchyOptions(bpy.types.Panel):
    bl_label = "Hierarchy Options"
    bl_idname = "SCENE_PT_medusanodes_hierarchy_options"
    bl_space_type = 'NODE_EDITOR'
    bl_region_type = "UI"
    bl_category = "Medusa Nodes"
    bl_parent_id = "SCENE_PT_medusanodes_main"
    bl_options = {'DEFAULT_CLOSED'}

    def draw(self, context):
        layout = self.layout
        layout.use_property_split = True
        layout.use_property_decorate = False

        col = layout.column()
        col.prop(context.scene, "use_medusa_hair_object")
  
class SCENE_PT_MedusaNodesHierarchyOptionsView(SCENE_PT_MedusaNodesHierarchyOptions):
    bl_idname = "SCENE_PT_medusanodes_hierarchy_options_vie"
    bl_space_type = 'VIEW_3D'
    bl_parent_id = "SCENE_PT_medusanodes_main_view"   

class SCENE_PT_MedusaNodesMainRenVie_curvesView(SCENE_PT_MedusaNodesMainRenVie_curves):
    bl_space_type = 'VIEW_3D'
    bl_parent_id = "SCENE_PT_medusanodes_mainrenvie_view"

class SCENE_PT_MedusaNodesMainRenVie_viewport_display(bpy.types.Panel):
    bl_label = "Viewport Display"
    bl_space_type = 'NODE_EDITOR'
    bl_region_type = "UI"
    bl_category = "Medusa Nodes"
    bl_parent_id = "SCENE_PT_medusanodes_mainrenvie"
    bl_options = {'DEFAULT_CLOSED'}

    def draw(self, context):
        draw_curves_settings(self, context)

class SCENE_PT_MedusaNodesMainRenVie_viewport_displayView(SCENE_PT_MedusaNodesMainRenVie_viewport_display):
    bl_space_type = 'VIEW_3D'
    bl_parent_id = "SCENE_PT_medusanodes_mainrenvie_view"

class SCENE_PT_MedusaNodesMainRenVie(bpy.types.Panel):
    bl_label = "Viewport & Rendering"
    bl_idname = "SCENE_PT_medusanodes_mainrenvie"
    bl_space_type = 'NODE_EDITOR'
    bl_region_type = "UI"
    bl_category = "Medusa Nodes"
    bl_parent_id = "SCENE_PT_medusanodes_main"
    bl_options = {"DEFAULT_CLOSED"}

    # @classmethod
    # def poll(cls, context):
    #     snode = context.space_data
    #     return snode.tree_type == 'GeometryNodeTree'
    
    def draw(self ,context):
        pass

class SCENE_PT_MedusaNodesMainRenVieView(SCENE_PT_MedusaNodesMainRenVie):
    bl_idname = "SCENE_PT_medusanodes_mainrenvie_view"
    bl_space_type = 'VIEW_3D'
    bl_parent_id = "SCENE_PT_medusanodes_main_view"

class RENDER_PG_bn_image_settings(bpy.types.PropertyGroup):

    items = [('PNG', 'PNG', ""),
            ('JPEG', 'JPEG', ""),
            ('TARGA', 'TGA', ""),
            ('TIFF', 'TIF', ""),
            ('DPX', 'DPX', ""),
            ('BMP', 'BMP', ""),
            ('OPEN_EXR', 'OpenEXR', ""),
            ('OPEN_EXR_MULTILAYER', 'OpenEXR MultiLayer', ""),
    ]

    file_format: EnumProperty(
        name="Image format",
        description="Choose which file format that images are saved to ",
        items=items,
        default="PNG"
        )


    exr_codec_items = [
            ('DWAA', 'DWAA (lossy)', ""),
            ('ZIPS', 'ZIPS (Lossless)', ""),
            ('RLE', 'RLE (Lossless', ""),
            ('PIZ', 'PIZ (Lossless', ""),
            ('ZIP', 'ZIP (Lossless', ""),
            ('PXR24', 'Pxr24 (lossy)', ""),
    ]

    exr_codec: EnumProperty(
    name="Exr codec",
    description="Compression mode for EXR",
    items=exr_codec_items,
    default = 'ZIP'
    )

    tiff_codec_items = [
            ('PACKBITS', 'Pack bits', ""),
            ('LZW', 'LZW', ""),
            ('DEFLATE', 'DEFLATE', ""),
            ('NONE', 'None', ""),
    ]

    tiff_codec: EnumProperty(
    name="Tiff codec",
    description="Compression mode for TIFF",
    items=tiff_codec_items,
    default = 'LZW'
    )

    jpeg2000_codec_items = [
            ('JP2', 'JP2', ""),
            ('J2K', 'J2K', ""),
    ]
    
    jpeg2k_codec: EnumProperty(
    name="JPEG 2000 codec",
    description="Compression mode for JPEG 2000",
    items=jpeg2000_codec_items
    )

    def item_callback(self, context):
        if self.file_format in ['BMP', 'JPEG', 'CINEON', 'HDR', 'AVI_JPEG', 'AVI_RAW', 'FFMPEG']:
            return [
                ('RGB', 'RGB', ''),
                ('BW', 'BW', '')
            ]
        else:
            return [
                ('RGB', 'RGB', ''),
                ('RGBA', 'RGBA', ''),
                ('BW', 'BW', ''),
            ]



    color_mode: EnumProperty(
    name="Color",
    description= "Choose BW for saving grayscale images, \
        RGB for saving red, green and blue channels, \
        and RGB for saving red, green, blue and alpha channels",
    items=item_callback,

    )   

    def item_callback(self, context):
        if self.file_format in ['BMP', 'IRIS', 'JPEG', 'TARGA', 'TARGA_RAW', 'AVI_JPEG', 'AVI_RAW', 'FFMPEG']:
            return [('8', '8', "")
                ]
        elif self.file_format in ['PNG', 'TIFF', ]:
            return [('16', '16', ""),
                ('8', '8', "")
                ]
        elif  self.file_format in ['CINEON',]:
            return [('10', '10', "")]
        elif self.file_format in ['OPEN_EXR_MULTILAYER', 'OPEN_EXR', ]:
            return [('16', '16', ""),
                ('32', '32', "")
                ]
        elif self.file_format in ['EXR']:
            return [('32', '32', "")
            ]
        elif self.file_format in ['DPX']:
            return [('8', '8', ""),
                ('10', '10', ""),
                ('12', '12', ""),
                ('16', '16', "")
                ]
        else:
            return [('16', '16', ""),
                    ('8', '8', ""),
                    ('32', '32', "")
                ]
                

    color_depth: EnumProperty(
        name="Color depth",
        description= "Bit depth per channel",
        items=item_callback,
    )     
    
    compression: IntProperty(
    name="Compression",
    description= "Amount of time to determine best compression: "\
        "0= no compression with fast file output, \
        100 = maximum lossless compression with slow file output",
    subtype = 'PERCENTAGE',
    min = 0,
    max = 100,
    default = 15
    )

    quality: IntProperty(
    name="Compression",
    description= "Quality for image formats that support lossy compression",
    subtype = 'PERCENTAGE',
    min = 0,
    max = 100,
    default = 90
    ) 

fstT = 0
def updateTexOptions():
    global fstT
    medusaNodesSaveOptions = bpy.context.scene.medusaNodesSaveOptions

    import inspect
    frm = inspect.stack()[1]
    mod = inspect.getmodule(frm[0])
    name = mod.__name__
    package = mod.__package__

    pref = bpy.context.preferences.addons[__package__].preferences
    if pref is not None:
        if fstT == 0:
            medusaNodesSaveOptions.autoTexBak = pref.autoTexBak
            medusaNodesSaveOptions.useAltTexPath = pref.useAltTexPath
            medusaNodesSaveOptions.texPath = pref.texPath
            medusaNodesSaveOptions.image_settings.file_format = pref.image_settings.file_format
            medusaNodesSaveOptions.image_settings.color_mode = pref.image_settings.color_mode
            medusaNodesSaveOptions.image_settings.color_depth = pref.image_settings.color_depth
            medusaNodesSaveOptions.image_settings.compression = pref.image_settings.compression
            medusaNodesSaveOptions.image_settings.quality = pref.image_settings.quality
            medusaNodesSaveOptions.image_settings.tiff_codec = pref.image_settings.tiff_codec
            medusaNodesSaveOptions.image_settings.exr_codec = pref.image_settings.exr_codec
            fstT = 1

class MedusaNodesSaveOptions(PropertyGroup):
    autoTexBak: BoolProperty(
        name="Automatic Texture Backup",
        description="Automatic Texture Backup",
        default = False)
    useAltTexPath: BoolProperty(
        name="Use Alternative Texture Path",
        description="Use Alternative Texture Path",
        default = False)
    texPath: StringProperty(
        name="",
        description="",
        default = "")
    image_settings: bpy.props.PointerProperty(type = RENDER_PG_bn_image_settings)

    image_name: StringProperty(
        name = "Texture Name",
        description="",
        default = "")
    
class SCENE_PT_MedusaNodesSaveOptions(bpy.types.Panel):
    bl_label = "Medusa Nodes"
    bl_idname = "SCENE_PT_medusaNodessaveoptions"
    bl_space_type = 'FILE_BROWSER'
    bl_region_type = 'TOOL_PROPS'
    bl_category = "Medusa Nodes"
    # bl_options = {"HIDE_HEADER"}

    @classmethod
    def poll(cls, context):
        return True

    def draw(self, context):
        layout = self.layout
        medusaNodesSaveOptions = bpy.context.scene.medusaNodesSaveOptions
        row = layout.row()
        row.prop(medusaNodesSaveOptions,"autoTexBak")

        txt = "Note: By default, textures connected to the groom will be saved in the 'textures' folder next to the .blend file."
        chars = int(context.region.width / 7)
        wrapper = textwrap.TextWrapper(width=chars)
        box = layout.box()
        box.alignment = 'EXPAND'
        text_lines = wrapper.wrap(text=txt)
        for text_line in text_lines:
            row = box.row(align = True)
            row.label(text=text_line)
        row = layout.row()
        row.prop(medusaNodesSaveOptions,"useAltTexPath")

        row = layout.row()
        row.enabled = False
        if medusaNodesSaveOptions.useAltTexPath:
            row.enabled = True
        row.prop(medusaNodesSaveOptions,"texPath")

class SCENE_PT_MedusaNodesSaveOptionsIS(bpy.types.Panel):
    bl_label = "Image file settings"
    bl_idname = "SCENE_PT_medusaNodessaveoptionsis"
    bl_space_type = 'FILE_BROWSER'
    bl_region_type = 'TOOL_PROPS'
    bl_category = "Medusa Nodes"
    bl_parent_id = 'SCENE_PT_medusaNodessaveoptions'

    @classmethod
    def poll(cls, context):
        return True

    def draw(self, context):
        layout = self.layout
        col = layout.column()
        medusaNodesSaveOptions = bpy.context.scene.medusaNodesSaveOptions
        image_settings = medusaNodesSaveOptions.image_settings

        grid = col.grid_flow(row_major = True, columns = 2,)

        grid.label(text='File Format')
        grid.prop(image_settings, "file_format", text = "")

        grid.label(text='Color')
        grid.prop(image_settings, "color_mode", text = "")

        grid.label(text='Color Depth')
        grid.prop(image_settings, "color_depth", text = "")


        if image_settings.file_format in ['PNG']:
            grid.label(text='Compression')
            grid.prop(image_settings, "compression", text = "") 

        if image_settings.file_format in ['JPEG', 'JPEG2000', 'AVI_JPEG']:
            grid.label(text='Quality')
            grid.prop(image_settings, "quality", text = "")

        if image_settings.file_format in ['TIFF']:
            grid.label(text='Compression')
            grid.prop(image_settings, "tiff_codec", text = "") 

        if image_settings.file_format in ['OPEN_EXR', 'OPEN_EXR_MULTILAYER']:
            grid.label(text='Compression')
            grid.prop(image_settings, "exr_codec", text = "")

activeNodeGroupName = ""
activeNodeObjectName = ""
activeNodeTreeGroupName = ""

class SCENE_PT_MedusaNodesHierarchy(bpy.types.Panel):
    bl_label = "Medusa Nodes Hierarchy"
    bl_idname = "SCENE_PT_medusanodes_hierarchy"
    bl_space_type = 'NODE_EDITOR'
    bl_region_type = "UI"
    bl_category = "Medusa Nodes"

    # def draw_header(self, context: bpy.types.Context):
    #     self.layout.label(text="", icon='OUTLINER')

    def draw(self, context):
        global activeNodeGroupName
        global activeNodeObjectName
        global activeNodeTreeGroupName
        scn = context.scene
        layout = self.layout
        col = layout.row(align=True)

        # pcoll = preview_collections["main"]
        # col.operator("medusanodes.templates", icon_value=pcoll["temp1"].icon_id, text="").action = 'preset1'
        # col.operator("medusanodes.templates", icon_value=pcoll["temp2"].icon_id, text="").action = 'preset2'
        # col.operator("medusanodes.templates", icon_value=pcoll["temp3"].icon_id, text="").action = 'preset3'
        # col.operator("medusanodes.templates", icon_value=pcoll["temp4"].icon_id, text="").action = 'preset4'
        # col.operator("medusanodes.templates", icon_value=pcoll["temp5"].icon_id, text="").action = 'preset5'


        row = layout.row()
        templateList = row.template_list(
            "MEDUSANODES_UL_TreeItem",
            "",
            scn,
            "medusaNodesTree",
            scn,
            "medusaNodesTree_index",
            sort_reverse=False,
            sort_lock=False,
            rows=5,
            type = 'DEFAULT'
        )

        col = row.column(align=True)
        col.menu("MEDUSANODE_MT_deformers_menu", icon='ADD', text="")
        col.operator("medusanodes.delete_deformer", icon='REMOVE', text="")
        col.separator()
        col.menu("MEDUSANODE_MT_options_menu", icon='DOWNARROW_HLT', text="")
        col.separator()

        col.operator("medusanodes.switch_order", icon='TRIA_UP', text="").action = 'UP'
        col.operator("medusanodes.switch_order", icon='TRIA_DOWN', text="").action = 'DOWN'



class MedusaNodesNotifications(bpy.types.Operator):
    bl_idname = "medusanodes.notifications"
    bl_label = "notification"

    def execute(self, context):
        message="Freeze the Generator first!"
        self.report({'WARNING'},message)
        return {'FINISHED'}


def msgbus_callback():
    medusaNodes = bpy.context.scene.medusaNodesTree
    item_index = bpy.context.scene.medusaNodesTree_index
    item_list = bpy.context.scene.medusaNodesTree
    if item_index>=0:
        if len(item_list)>item_index:
            item = medusaNodes[item_index]
            item_index = bpy.context.scene.medusaNodesTree_index
            if check_valid(item):
                geoNodes = bpy.data.objects[item.objectName].modifiers[item.nodeTreeName]
                node = geoNodes.node_group.nodes[item.name]

                if "mn visualizer" in node.node_tree.name.lower():
                    node.inputs["Hair Material"].default_value = bpy.data.objects[item.objectName].active_material

subPanels = []

class MedusaNodesNewTexture(bpy.types.Operator):
    '''Create new texture map'''
    bl_label = "New Texture"
    bl_idname = "medusanodes.change_resolution"
    bl_options = {'UNDO'}

    #divide: BoolProperty(default = False)
    #propertyToMultiply: StringProperty(default = '')
    def execute(self, context):
        mapResolutionX = getattr(context.scene, "mapResolutionX")
        mapResolutionY = getattr(context.scene, "mapResolutionY")
        if self.divide:
            mapResolutionX = mapResolutionX / 2
            mapResolutionY = mapResolutionY / 2 
        else:
            mapResolutionX = mapResolutionX * 2
            mapResolutionY = mapResolutionY * 2

        setattr(context.scene, "mapResolutionX", int(mapResolutionX))
        setattr(context.scene, "mapResolutionY", int(mapResolutionY))

        return {'FINISHED'}
    
class MedusaNodesChangeResolution(bpy.types.Operator):
    '''Multiply or divide the resolution of the texture map by 2'''
    bl_label = "Multiply or divide by 2"
    bl_idname = "medusanodes.change_resolution"
    bl_options = {'UNDO'}

    divide: BoolProperty(default = False)
    #propertyToMultiply: StringProperty(default = '')
    def execute(self, context):
        mapResolutionX = getattr(context.scene, "mapResolutionX")
        mapResolutionY = getattr(context.scene, "mapResolutionY")
        if self.divide:
            mapResolutionX = mapResolutionX / 2
            mapResolutionY = mapResolutionY / 2 
        else:
            mapResolutionX = mapResolutionX * 2
            mapResolutionY = mapResolutionY * 2

        setattr(context.scene, "mapResolutionX", int(mapResolutionX))
        setattr(context.scene, "mapResolutionY", int(mapResolutionY))

        return {'FINISHED'}
    

def get_scene_collection(collection, name):
    
    if collection.name == name:
        return collection
    for child_collection in collection.children:
        found = get_scene_collection(child_collection, name)
        if found:
            return found
        
    return None


#should update when the hierarchy gets build
def update_curve_object_list(colName):
    try:
        collection = bpy.data.collections[colName]
        collection.curve_object_list.clear()

        for o in collection.objects:
            item = collection.curve_object_list.add()
            item.name = o.name
            item.hidden = o.hide_get()
            item.disabled = o.hide_viewport
            
        sceneCol = bpy.context.view_layer.layer_collection
        viewlayerCol = get_scene_collection(sceneCol,colName)
        if collection:
            item = collection.curve_object_list.add()
            item.name = viewlayerCol.name
            item.hidden = viewlayerCol.hide_viewport #viewlayer dependet(eye icon)
            item.disabled = collection.hide_viewport #global (monitor icon)
    except:
        pass
   

class curveObjectPropGroup(bpy.types.PropertyGroup):
    name: StringProperty()
    hidden : BoolProperty(default = False)
    disabled : BoolProperty(default = False)
    
    
class MedusaNodesAddCurveObject(bpy.types.Operator):
    bl_idname = "medusanodes.add_curve_object"
    bl_label = "Add Curve Object"
    
    def execute(self, context):
        item_index = context.scene.medusaNodesTree_index
        item_list = context.scene.medusaNodesTree
        item = item_list[item_index]  
        if check_valid(item):
            node_ob = bpy.data.objects[item.objectName]
            # geoNodes = bpy.data.objects[item.objectName].modifiers[item.nodeTreeName]
            bpy.ops.object.mode_set(mode='OBJECT')
            partIndex = -1
            partStrInde = ""
            for ob in bpy.data.objects:
                if ob.type == "CURVE":
                    if item.objectName+'_Curves' in ob.name:
                        nameParts = ob.name.split("_")
                        if len(nameParts)>1:
                            namePrt = nameParts[-1]
                            namePrtI = namePrt.lower()[-2:]
                            if namePrtI.isdigit():
                                if int(namePrtI)>partIndex:
                                    partIndex = int(namePrtI)
                            else:
                                partIndex = 0
            if partIndex != -1:
                partStrInde = str(partIndex+1).zfill(2)
            
            curveData = bpy.data.curves.new('curveData', 'CURVE')
            curveData.dimensions = '3D'
            curve = bpy.data.objects.new(item.objectName+'_Curves'+partStrInde, curveData)
            
            colName = item.objectName+'_Curves'
            collection = bpy.data.collections.get(colName)
            if collection is None:
                collection = bpy.data.collections.new(colName)
            collection.objects.link(curve)
            try:
                node_ob.users_collection[0].children.link(collection)
            except:
                pass
            collection.curve_object_index = len(collection.curve_object_list)-1
            #update props
            update_curve_object_list(colName)
    
        return {'FINISHED'}
    
class MedusaNodesRemoveCurveObject(bpy.types.Operator):
    bl_idname = "medusanodes.remove_curve_object"
    bl_label = "Remove Curve Object"

    index : IntProperty()
    
    def execute(self, context):
        item_index = context.scene.medusaNodesTree_index
        item_list = context.scene.medusaNodesTree
        item = item_list[item_index]  
        if check_valid(item):
            ob = bpy.data.objects[item.objectName]
            colName = item.objectName+'_Curves'
            collection = bpy.data.collections[colName]
            if self.index>=0 and self.index<len(collection.objects):
                obj = collection.objects[self.index]
                objects = bpy.data.objects
                objects.remove(obj, do_unlink=True)
                
                #update props
                update_curve_object_list(colName)
                try:
                    collection.curve_object_index = len(collection.curve_object_list)-2
                except:
                    pass
                # try:
                #     #set for context
                #     context.view_layer.objects.active = collection.objects[self.index-1]
                # except:
                #     pass
                bpy.context.view_layer.objects.active = ob
                # ob.select_set(True)
        
        return {'FINISHED'}

class MedusaNodesEditCurveObject(bpy.types.Operator):
    bl_idname = "object.edit_curve_object"
    bl_label = "Edit Curve Object"

    index : bpy.props.IntProperty()
    drawCurve : bpy.props.BoolProperty(default=False)
    
    def execute(self, context):
        global lstObj
        sceneCol = context.view_layer.layer_collection
        selectedObjectName = bpy.context.object.name
        currentMode = bpy.context.mode
        item_index = context.scene.medusaNodesTree_index
        item_list = context.scene.medusaNodesTree
        item = item_list[item_index]  
        if check_valid(item):
            ob = bpy.data.objects[item.objectName]
            colName = item.objectName+'_Curves'
            collection = bpy.data.collections[colName]
            obj = collection.objects[self.index]

            #set object to active
            #context.view_layer.objects.active = obj
            #obj.select_set(True)
            
            item = collection.curve_object_list[self.index]
            itemCol = collection.curve_object_list[colName]
            
            layerCollection = get_scene_collection(sceneCol, colName)
            
            if item.hidden:
                obj.hide_set(False)
            if item.disabled:
                obj.hide_viewport = False
            
            layerCollection.hide_viewport = False  
            collection.hide_viewport = False

            if bpy.context.view_layer.objects.active == obj:
                if 'EDIT' in currentMode:
                    if self.drawCurve and 'draw' not in context.workspace.tools.from_space_view3d_mode('EDIT_CURVE').idname:
                        bpy.ops.wm.tool_set_by_id(name="builtin.draw")
                        bpy.context.tool_settings.curve_paint_settings.depth_mode = 'SURFACE'
                        bpy.context.tool_settings.curve_paint_settings.use_stroke_endpoints = True
                    else:
                        bpy.ops.object.mode_set(mode='OBJECT')
                        obj.select_set(False)
                        #revert visibility
                        obj.hide_set(item.hidden)
                        obj.hide_viewport = item.disabled
                        collection.hide_viewport = itemCol.disabled #global (monitor icon)
                        layerCollection.hide_viewport = itemCol.hidden #viewlayer dependet(eye icon)
                        bpy.context.view_layer.objects.active = lstObj
                        lstObj.select_set(True)
    
                else:
                    
                    lstObj = bpy.context.object
                    obj.select_set(True)
                    bpy.ops.object.mode_set(mode='EDIT')
                    if self.drawCurve:
                        bpy.ops.wm.tool_set_by_id(name="builtin.draw")
                        bpy.context.tool_settings.curve_paint_settings.depth_mode = 'SURFACE'
                        bpy.context.tool_settings.curve_paint_settings.use_stroke_endpoints = True
                    else:
                        bpy.ops.wm.tool_set_by_id(name="builtin.select_box")
            
            else:
                lstObj = bpy.context.object
                bpy.ops.object.mode_set(mode="OBJECT")
                bpy.ops.object.select_all(action="DESELECT")
                bpy.context.view_layer.objects.active = obj
                obj.select_set(True)
                
                
                bpy.ops.object.mode_set(mode='EDIT')
                if self.drawCurve:
                    bpy.ops.wm.tool_set_by_id(name="builtin.draw")
                    bpy.context.tool_settings.curve_paint_settings.depth_mode = 'SURFACE'
                    bpy.context.tool_settings.curve_paint_settings.use_stroke_endpoints = True
                else:
                    bpy.ops.wm.tool_set_by_id(name="builtin.select_box")

        return {'FINISHED'}
    
class MEDUSANODES_UL_CurveObjectCollection(bpy.types.UIList):
    def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index):
        obj = bpy.data.objects.get(item.name)
        if obj:
            row = layout.row(align = True)
            row.label(icon = "OUTLINER_OB_CURVE", text="")
            row.prop(obj, "name", text="", emboss=False)

            drawMode = False
            alreadyEdit = False

            if 'EDIT' in obj.mode:
                if 'draw' not in context.workspace.tools.from_space_view3d_mode('EDIT_CURVE').idname:
                    alreadyEdit = True
                if 'draw' in context.workspace.tools.from_space_view3d_mode('EDIT_CURVE').idname:
                    drawMode = True

            operator = row.operator('object.edit_curve_object', icon= 'EDITMODE_HLT',text = "", depress = alreadyEdit )
            operator.index = index
            operator.drawCurve = False

            operator = row.operator('object.edit_curve_object', icon= 'GREASEPENCIL',text = "", depress = drawMode )
            operator.index = index
            operator.drawCurve = True

class SCENE_PT_MedusaNodesProperties(bpy.types.Panel):
    bl_label = "Medusa Nodes Properties"
    bl_idname = "SCENE_PT_medusanodes_properties"
    bl_space_type = 'NODE_EDITOR'
    bl_region_type = "UI"
    bl_category = "Medusa Nodes"

    bpy.types.Scene.mapResolutionX = IntProperty(
        name="",
        description="",
        default = 1024)
    bpy.types.Scene.mapResolutionY = IntProperty(
        name="",
        description="",
        default = 1024)
    
    # def draw_header(self, context: bpy.types.Context):
    #     self.layout.label(text="", icon='PROPERTIES')
    
    def draw(self, context):
        global activeNodeGroupName
        global activeNodeObjectName
        global activeNodeTreeGroupName
        global subPanels
        global needToUpdate
        layout = self.layout
        #layout = layout.box()#TEST
        needToUpdate = False
        subPanels = []

        item_index = context.scene.medusaNodesTree_index
        item_list = context.scene.medusaNodesTree

        if item_index>=0:# and len(item_list)>item_index:
            # try:
                item = None
                if len(item_list)>item_index:
                    medusaNodes = context.scene.medusaNodesTree
                    item = medusaNodes[item_index]
                    activeNodeGroupName = item.name
                    activeNodeObjectName = item.objectName
                    activeNodeTreeGroupName = item.nodeTreeName
                else:
                    for barNode in context.scene.medusaNodes:
                        if barNode.name == activeNodeGroupName and barNode.nodeTreeName == activeNodeTreeGroupName and barNode.objectName == activeNodeObjectName:
                            item = barNode
                            break
                if check_valid(item):
                    geoNodes = bpy.data.objects[item.objectName].modifiers[item.nodeTreeName]
                    inpTypes = ["VALUE","INT","BOOLEAN","RGBA"]
                    if geoNodes:
                        if geoNodes.type == 'NODES':
                            if item.name in geoNodes.node_group.nodes:
                                deformerCached = True
                                node_ref = geoNodes.node_group.nodes[item.name]
                                inputsToIgnore = []
                                nodePanelData = {}
                                insideGroup = ""
                                enableItem = True
                                if context.scene.frame_start != context.scene.frame_current:
                                    if 'guide'in item.objectName.lower():
                                        for node in medusaNodes:
                                            if 'dynamics' in node.label.lower() and node.objectName == item.objectName and geoNodes.node_group.nodes[node.name].inputs['Disable'].default_value == False:
                                                enableItem = True
                                                if item.nodeIndex < node.nodeIndex:
                                                    enableItem = False
                                                if item.objectName.lower() == item.label.lower():
                                                    enableItem = True
                                    else:
                                        enableItem = True
                                        for node in medusaNodes:
                                            if 'dynamics' in node.label.lower() and node.objectName == item.objectName and geoNodes.node_group.nodes[node.name].inputs['Disable'].default_value == False:
                                                enableItem = True
                                                if item.nodeIndex < node.nodeIndex:
                                                    enableItem = False
                                                if item.objectName.lower() == item.label.lower():
                                                    enableItem = True
                                else:
                                    enableItem = True
                                for inp in node_ref.inputs:
                                    if "###" in inp.name:
                                        if node_ref.name not in nodePanelData:
                                            nodePanelData[node_ref.name] = {}
                                        if inp.identifier not in nodePanelData[node_ref.name]:
                                            nodePanelData[node_ref.name][inp.identifier] = {}
                                            nodePanelData[node_ref.name][inp.identifier]["title"] = inp.name.replace("###","")
                                            nodePanelData[node_ref.name][inp.identifier]["item"] = item
                                            nodePanelData[node_ref.name][inp.identifier]["inputs"] = []
                                        insideGroup = inp.identifier
                                    else:
                                        if insideGroup != "":
                                            nodePanelData[node_ref.name][insideGroup]["inputs"].append(inp.name)
                                            inputsToIgnore.append(inp.name)
                                for p in nodePanelData:
                                    for subP in nodePanelData[p]:
                                        subPart = p+"_"+subP
                                        id = "SCENE_PT_Medusanodeshierarchy_subpanel_"+subPart.lower().replace(".","_")
                                        subpanel_class = type("SCENE_PT_MedusaSubPanel", (bpy.types.Panel,), {
                                            "bl_idname": id,
                                            "bl_label": nodePanelData[p][subP]["title"],
                                            "bl_parent_id": "SCENE_PT_medusanodes_properties_vie",
                                            "bl_space_type": 'VIEW_3D',
                                            "bl_region_type": 'UI',
                                            "bl_category": 'Medusa Nodes',
                                            "draw": draw_medusa_subpanel,
                                            "data":nodePanelData[p][subP]
                                        })
                                        subPanels.append(subpanel_class)
                                        id = "SCENE_PT_Medusanodeshierarchy_subpanel_node_"+subPart.lower().replace(".","_")
                                        subpanel_class = type("SCENE_PT_MedusaSubPanel", (bpy.types.Panel,), {
                                            "bl_idname": id,
                                            "bl_label": nodePanelData[p][subP]["title"],
                                            "bl_parent_id": "SCENE_PT_medusanodes_properties",
                                            "bl_space_type": 'NODE_EDITOR',
                                            "bl_region_type": 'UI',
                                            "bl_category": 'Medusa Nodes',
                                            "draw": draw_medusa_subpanel,
                                            "data":nodePanelData[p][subP]
                                        })
                                        subPanels.append(subpanel_class)
                                
                                if "mn generator" not in node_ref.node_tree.name.lower():
                                    for node in geoNodes.node_group.nodes:
                                        if node.type == "GROUP":
                                            if "mn generator" in node.node_tree.name.lower():
                                                for inp in node.inputs:
                                                    if inp.name.lower() == "cached":
                                                        if not inp.default_value:
                                                            deformerCached = False
                                
                                #Deformer Icon 
                                if item.objectName in bpy.data.objects:
                                    actOb = bpy.data.objects[item.objectName]
                                    if item.nodeTreeName in actOb.modifiers:
                                        geoNodes = actOb.modifiers[item.nodeTreeName]
                                        activeNode = geoNodes.node_group.nodes[item.name]
                                        iconName = activeNode.node_tree.name.lower()[3:]

                                        icon_id = 0
                                        icon = 'NONE'
                                        if "MedusaHair" in actOb:
                                            if iconName in guidepcoll:
                                                icon_id = guidepcoll[iconName].icon_id
                                            if 'generator' in activeNode.node_tree.name.lower()  and activeNode.inputs['Cached'].default_value:
                                                icon_id = guidepcoll[iconName +' freeze'].icon_id
                                            if 'noise' in activeNode.node_tree.name.lower() and activeNode.inputs['Cached'].default_value:
                                                icon_id = guidepcoll[iconName +' freeze'].icon_id
                                            if 'curl' in activeNode.node_tree.name.lower() and activeNode.inputs['Cached'].default_value:
                                                icon_id = guidepcoll[iconName +' freeze'].icon_id
                                        else:
                                            if iconName in pcoll:
                                                icon_id = pcoll[iconName].icon_id
                                            if 'generator' in activeNode.node_tree.name.lower() and activeNode.inputs['Cached'].default_value:
                                                icon_id = pcoll[iconName +' freeze'].icon_id
                                            if 'noise' in activeNode.node_tree.name.lower() and activeNode.inputs['Cached'].default_value:
                                                icon_id = pcoll[iconName +' freeze'].icon_id
                                            if 'curl' in activeNode.node_tree.name.lower() and activeNode.inputs['Cached'].default_value:
                                                icon_id = pcoll[iconName +' freeze'].icon_id
                                            if 'guide' in activeNode.node_tree.name.lower() and activeNode.inputs['Cached'].default_value:
                                                icon_id = pcoll[iconName +' freeze'].icon_id
                                        
                                        row = layout.grid_flow(row_major=True, columns=3, even_columns=False, even_rows=False, align=False)
                                    
                                        sub_row = row.row()
                                        sub_row.alignment = 'CENTER'
                                        sub_row.label(text="")

                                        sub_row = row.row()
                                        sub_row.alignment = 'CENTER'
        
                                        if enableItem: 
                                            sub_row.label(text=item.label, icon_value= icon_id, icon = icon)
                                        else:
                                            sub_row.label(text=item.label + " (Cached)", icon_value= icon_id, icon = icon)

                                        if "mn clump" in activeNode.node_tree.name.lower() or "mn guide" in activeNode.node_tree.name.lower() or "mn braid" in activeNode.node_tree.name.lower():
                                            sub_row = row.row()
                                            sub_row.alignment = 'RIGHT'
                                            #sub_row.label(text = '', icon_value = guidepcoll['visualizer'].icon_id)
                                            sub_row.operator_menu_enum(MedusaNodeTreeGuidesMenu.bl_idname, 'guideItems', text="Guide Object").index = item_index #icon='SETTINGS'
                                        else:
                                            sub_row = row.row()
                                            sub_row.alignment = 'CENTER'
                                            sub_row.label(text="")
                                        layout.context_pointer_set("node", node_ref)

                                        if context.screen.is_animation_playing:
                                            animationIcon = 'PAUSE'
                                            animationLabel = 'Pause Simulation'
                                            animationDepress = True
                                        else:
                                            animationIcon = 'PLAY'
                                            animationLabel = 'Run Simulation'
                                            animationDepress = False

                                        #Dynamics animation buttons
                                        if "mn dynamics" in node_ref.node_tree.name.lower():
                                            #for node in geo Nodes.node_group.nodes:
                                            row = layout.row(align= True)
                                            row.scale_y = 1.45
                                            row.operator('screen.animation_play',icon = animationIcon , text = animationLabel, depress=animationDepress)
                                            row.scale_x = 2
                                            row.operator("screen.frame_jump", text="", icon='SNAP_FACE').end = False
                                            
                                            #show caching progress bar
                                            # if context.screen.is_animation_playing:
                                            #     progress_bar = layout.row()
                                            #     progress_bar.prop(context.scene, "progress", text="Caching")

                                        layout = layout.column(align=True)#TEST: align props

                                        # if hasattr(node_ref, "draw_buttons_ext"):
                                        #     node_ref.draw_buttons_ext(context, layout)
                                        # elif hasattr(node_ref, "draw_buttons"):
                                        #     node_ref.draw_buttons(context, layout)


                                        



                                        value_inputs = [socket for socket in node_ref.inputs if self.show_socket_input(socket)]
                                        if value_inputs:
                                            cachedState = False
                                            poissonState = False
                                            exactAmountState = False
                                            onVerticesState = False
                                            curveCollection = None
                                            for socket in value_inputs:
                                                if socket.name.lower() == "cached":
                                                    if socket.default_value == True:
                                                        cachedState = True
                                                        break
                                                if activeNode.node_tree.name.lower() == 'mn generator':
                                                    if activeNode.inputs['Poisson Disk Distribution'].default_value ==True:
                                                        poissonState = True
                                                    if activeNode.inputs['Exact Amount Distribution'].default_value ==True:
                                                        exactAmountState = True
                                                    if activeNode.inputs['Distribute on Vertices'].default_value ==True:
                                                        onVerticesState = True

                                                if activeNode.node_tree.name.lower() == 'mn curves to splines':
                                                    if activeNode.inputs['Curve Collection'].default_value:
                                                        curveCollection = activeNode.inputs['Curve Collection'].default_value


                                            for socket in value_inputs:
                                                if socket.name not in inputsToIgnore:
                                                    if socket.name.lower() != "disable" and socket.name.lower() != "cached" and "###" not in socket.name.lower():
                                                        fromNode = None
                                                        for link in geoNodes.node_group.links:
                                                            if link.to_node == node_ref:
                                                                if link.to_socket == socket and socket.type in inpTypes:
                                                                    fromNode = link.from_node
                                                                    break
                                                        if fromNode:
                                                            floatCurvesNodes = {}
                                                            
                                                            if fromNode.type == "GROUP":
                                                                for snode in fromNode.node_tree.nodes:
                                                                    if snode.type == "CURVE_FLOAT":
                                                                        floatCurvesNodes[snode.label] = snode
                                                                    elif snode.type == "VALTORGB":
                                                                        floatCurvesNodes[snode.label] = snode
                                                            value_inputs_from = [socketF for socketF in fromNode.inputs if self.show_socket_input(socketF)]
                                                            row = layout.row()
                                                            if cachedState:
                                                                if node_ref.node_tree.name in inputsDisableData:
                                                                    if socket.name in inputsDisableData[node_ref.node_tree.name]:
                                                                        row.enabled = False
                                                            if not enableItem:
                                                                row.enabled = enableItem
                                                            
                                                            if "expanded" not in socket:
                                                                op = row.operator("medusanodes.nodeprop_expand", text="", icon='TRIA_RIGHT', emboss=False)
                                                            else:
                                                                if not socket["expanded"]:
                                                                    op = row.operator("medusanodes.nodeprop_expand", text="", icon='TRIA_RIGHT', emboss=False)
                                                                else:
                                                                    op = row.operator("medusanodes.nodeprop_expand", text="", icon='TRIA_DOWN', emboss=False)
                                                            op.objectName= item.objectName
                                                            op.nodeTreeName  = item.nodeTreeName
                                                            op.nodeName  = node_ref.name
                                                            op.socketName = socket.name
                                                            
                                                            if socket.links[0].is_muted:
                                                                socket.draw(
                                                                    context,
                                                                    row,
                                                                    node_ref,
                                                                    iface_(socket.label if socket.label else socket.name, socket.bl_rna.translation_context),
                                                                )
                                                            else:
                                                                row.label(text=socket.name)

                                                            # row.menu("MEDUSANODE_MT_masks_group_menu", icon='PLUGIN', text="")
                                                            op = row.operator("medusanodes.invokemaskmenu", icon='PLUGIN', text="")
                                                            op.groupNodeSocketName = socket.name
                                                            op.nodeGroup = node_ref.node_tree.name
                                                            op.groupNodeName = fromNode.name
                                                            row.scale_x=1.2
                                                            row.scale_y=1.2

                                                            if "expanded" in socket:
                                                                if socket["expanded"]:
                                                                    box = layout.box()
                                                                    box = box.column(align=True)
                                                                    row = box.row(align=True)
                                                                    if cachedState:
                                                                        if node_ref.node_tree.name in inputsDisableData:
                                                                            if socket.name in inputsDisableData[node_ref.node_tree.name]:
                                                                                row.enabled = False
                                                                    if not enableItem:
                                                                        row.enabled = enableItem
                                                                    import re         
                                                                    strippedDigits = re.sub("[0-9.]",'',fromNode.node_tree.name)
                                                                    maskGroupLabel = strippedDigits.replace('MN ','')
                                                                    if ' Mask Group ' in maskGroupLabel:
                                                                        maskGroupIcon = maskGroupLabel.replace(' Mask Group ','')
                                                                    else:
                                                                        maskGroupIcon = maskGroupLabel.replace(' Mask Group','')
                                                                    if maskGroupIcon.lower() in pcoll:
                                                                        row.label(text= maskGroupLabel, icon_value=pcoll[maskGroupIcon.lower()].icon_id)
                                                                    elif maskGroupIcon.lower() == "curve" :
                                                                        row.label(text= maskGroupLabel, icon='FCURVE')
                                                                    elif maskGroupIcon.lower() == "texture" :
                                                                        row.label(text= maskGroupLabel, icon='TEXTURE')
                                                                    elif maskGroupIcon.lower() == "noise" :
                                                                        row.label(text= maskGroupLabel, icon='MOD_NOISE')
                                                                    else:
                                                                        row.label(text= maskGroupLabel)

                                                                    viewNode = None
                                                                    for lnk in fromNode.outputs[0].links:
                                                                        if lnk.to_node.type == "VIEWER":
                                                                            if "MN Viewer" in lnk.to_node.name:
                                                                                viewNode = lnk.to_node
                                                                                break
                                                                    # if viewNode is None:
                                                                    #     row.operator("medusanodes.view_group", icon="HIDE_OFF",text="").groupNodeName = fromNode.name
                                                                    # else:
                                                                    #     row.operator("medusanodes.view_group", icon="HIDE_ON",text="").groupNodeName = fromNode.name

                                                                    if socket.links[0].is_muted:
                                                                        row.operator("medusanodes.mute_link", icon='RESTRICT_VIEW_ON', text="").socketName = socket.name
                                                                    else:
                                                                        row.operator("medusanodes.mute_link", icon='RESTRICT_VIEW_OFF', text="").socketName = socket.name
                                                                    row.operator("medusanodes.edit_mask_group", icon='NODETREE', text="").groupNodeName = fromNode.name
                                                                    row.operator("medusanodes.delete_group", icon='TRASH', text="").groupNodeName = fromNode.name
                                                                    row.scale_x=1.2
                                                                    row.scale_y=1.2

                                                                    l = 0
                                                                    for sub_socket in value_inputs_from:
                                                                        if sub_socket.name.lower() != "disable" and socket.type in inpTypes:
                                                                            if str(l+1) in floatCurvesNodes:
                                                                                boxV = box.box()
                                                                                if socket.links[0].is_muted:
                                                                                    boxV.enabled = False
                                                                                else:
                                                                                    boxV.enabled = True
                                                                                if not enableItem:
                                                                                    boxV.enabled = enableItem
                                                                                floatCurvesNode = floatCurvesNodes[str(l+1)]
                                                                                value_inputs_from = [socketF for socketF in floatCurvesNode.inputs if self.show_socket_input(socketF)]
                                                                                if hasattr(floatCurvesNode, "draw_buttons_ext"):
                                                                                    floatCurvesNode.draw_buttons_ext(context, boxV)
                                                                                elif hasattr(floatCurvesNode, "draw_buttons"):
                                                                                    floatCurvesNode.draw_buttons(context, boxV)
                                                                            
                                                                            row = box.row(align = True)
                                                                            if socket.links[0].is_muted:
                                                                                row.enabled = False
                                                                            else:
                                                                                row.enabled = True
                                                                            '''
                                                                            # texture settings
                                                                            if cachedState:
                                                                                if node_ref.node_tree.name in inputsDisableData:
                                                                                    if socket.name in inputsDisableData[node_ref.node_tree.name]:
                                                                                        row.enabled = False
                                                                            if sub_socket.type == "IMAGE":

                                                                                for socket in fromNode.inputs:
                                                                                    if "expanded" not in socket:
                                                                                        op = row.operator("medusanodes.nodeprop_expand", text="", icon='PREFERENCES')
                                                                                    else:
                                                                                        if not socket["expanded"]:
                                                                                            op = row.operator("medusanodes.nodeprop_expand", text="", icon='PREFERENCES',)
                                                                                        else:
                                                                                            op = row.operator("medusanodes.nodeprop_expand", text="", icon='PREFERENCES', depress=True)
                                                                                    op.objectName= item.objectName
                                                                                    op.nodeTreeName  = item.nodeTreeName
                                                                                    op.nodeName  = fromNode.name
                                                                                    op.socketName = socket.name

                                                                                sub_socket.draw(
                                                                                    context,
                                                                                    row,
                                                                                    fromNode,
                                                                                    iface_(sub_socket.label if sub_socket.label else sub_socket.name, sub_socket.bl_rna.translation_context))
                                                                                
                                                                                row = row.row()
                                                                                status = False
                                                                                if "paintMode" in bpy.context.object:
                                                                                    if bpy.context.object["paintMode"]:
                                                                                        status = True
                                                                                if bpy.context.mode != "PAINT_TEXTURE":
                                                                                    status = False
                                                                                else:
                                                                                    status = True

                                                                                if socket["expanded"]:
                                                                                    boxRow = box.row()

                                                                                    boxRow = box.row() # main box which contains the texture setup buttons and the arrow key
                                                                                    subBox = boxRow.box() # row is filled with a box
                                                                                    subBoxRow = subBox.row(align=True, heading='', heading_ctxt='', translate=True) # the box has its own row inside
                                                                                    boxRow.scale_x = 1
                                                                                    boxRow.scale_y = 1
                                                                                
                                                                                    subBoxRow.scale_x = 0.25
                                                                                    subBoxRow.operator("image.new", text = "New", icon = 'ADD')
                                                                                    #subBoxRow.template_ID(fromNodeinput, 'default_value', unlink='image.open', filter='ALL', live_icon=False, text='', text_ctxt='', translate=True)
                                                                                    subBoxRow.scale_x = 0.5
                                                                                    subBoxRow.prop(bpy.context.scene.medusaNodesSaveOptions, "image_name", text = "")
                                                                                    subBoxRow.scale_x = 0.25
                                                                                    subBoxRow.prop(bpy.context.scene, "mapResolutionX")
                                                                                    #row.prop(bpy.context.scene, "mapResolutionY")
                                                                                    subBoxRow.scale_x = 1
                                                                                    subBoxRow.operator("medusanodes.change_resolution", icon='ADD', text="",).divide=False
                                                                                    subBoxRow.operator("medusanodes.change_resolution", icon='REMOVE', text="").divide=True
                                                                                else:
                                                                                    pass
                                                                                #Paint mode button size increase
                                                                                row.scale_x = 1.5
                                                                                row.scale_y = 2
                                                                                paintOp = row.operator("medusanodes.paint_mode", icon='BRUSH_DATA', text="",depress=status)
                                                                                paintOp.groupNodeName = fromNode.name
                                                                                paintOp.socketName = sub_socket.name
                                                                                if sub_socket.default_value == None:
                                                                                    row.enabled = False
                                                                                if cachedState:
                                                                                    if node_ref.node_tree.name in inputsDisableData:
                                                                                        if sub_socket.name in inputsDisableData[node_ref.node_tree.name]:
                                                                                            row.enabled = False
                                                                            else:
                                                                                sub_socket.draw(
                                                                                    context,
                                                                                    row,
                                                                                    fromNode,
                                                                                    iface_(sub_socket.label if sub_socket.label else sub_socket.name, sub_socket.bl_rna.translation_context))
                                                                        '''
                                                                        #################################start of comment: '''
                                                                            sub_socket.draw(
                                                                                context,
                                                                                row,
                                                                                fromNode,
                                                                                iface_(sub_socket.label if sub_socket.label else sub_socket.name, sub_socket.bl_rna.translation_context),
                                                                            )                                                                                                                               
                                                                            if cachedState:
                                                                                if node_ref.node_tree.name in inputsDisableData:
                                                                                    if socket.name in inputsDisableData[node_ref.node_tree.name]:
                                                                                        row.enabled = False
                                                                            if sub_socket.type == "IMAGE":
                                                                                row = row.row()
                                                                                row.scale_x=1.1
                                                                                row.scale_y=1.1                                                                        

                                                                                status = False
                                                                                if "paintMode" in bpy.context.object:
                                                                                    if bpy.context.object["paintMode"]:
                                                                                        status = True
                                                                                if bpy.context.mode != "PAINT_TEXTURE":
                                                                                    status = False
                                                                                else:
                                                                                    status = True                                                                        
                                                                                paintOp = row.operator("medusanodes.paint_mode", icon='BRUSH_DATA', text="",depress=status)
                                                                                paintOp.groupNodeName = fromNode.name
                                                                                paintOp.socketName = sub_socket.name
                                                                                if sub_socket.default_value == None:
                                                                                    row.enabled = False
                                                                                if cachedState:
                                                                                    if node_ref.node_tree.name in inputsDisableData:
                                                                                        if sub_socket.name in inputsDisableData[node_ref.node_tree.name]:
                                                                                            row.enabled = False   
                                                                        #################################End of comment: '''
                                                                        l = l + 1
                                                                    for index in floatCurvesNodes:
                                                                        if index.isdigit():
                                                                            if int(index) > l:
                                                                                boxV = box.box()
                                                                                if socket.links[0].is_muted:
                                                                                    boxV.enabled = False
                                                                                else:
                                                                                    boxV.enabled = True
                                                                                if not enableItem:
                                                                                    boxV.enabled = enableItem
                                                                                floatCurvesNode = floatCurvesNodes[index]
                                                                                value_inputs_from = [socketF for socketF in floatCurvesNode.inputs if self.show_socket_input(socketF)]
                                                                                if hasattr(floatCurvesNode, "draw_buttons_ext"):
                                                                                    floatCurvesNode.draw_buttons_ext(context, boxV)
                                                                                elif hasattr(floatCurvesNode, "draw_buttons"):
                                                                                    floatCurvesNode.draw_buttons(context, boxV)
                                                                    
                                                                    row = box.row()            
                                                        elif socket.is_linked == False:
                                                            row = layout.row(align=True)  
                                                            if socket.name.lower() != "hair material" and socket.name.lower() != "root normal material":
                                                                socket.draw(
                                                                    context,
                                                                    row,
                                                                    node_ref,
                                                                    iface_(socket.label if socket.label else socket.name, socket.bl_rna.translation_context),
                                                                )
                                                            elif socket.name.lower() == "hair material":
                                                                row = layout.row()
                                                                split = row.split(factor=0.3)
                                                                c = split.row()
                                                                c.label(icon_value=layout.icon(bpy.data.objects[item.objectName].active_material))
                                                                c.label(text="Hair Material")
                                                                split = split.split()
                                                                c = split.column()
                                                                c.template_ID(bpy.data.objects[item.objectName], "active_material",live_icon=False)
                                                            if cachedState:
                                                                if node_ref.node_tree.name in inputsDisableData:
                                                                    if socket.name in inputsDisableData[node_ref.node_tree.name]:
                                                                        row.enabled = False
                                                            if not enableItem:
                                                                row.enabled = enableItem
                                                            if socket.type in inpTypes and socket.type != "BOOLEAN":
                                                                op = row.operator("medusanodes.invokemaskmenu", icon='PLUGIN', text="")
                                                                op.groupNodeSocketName = socket.name
                                                                op.nodeGroup = node_ref.node_tree.name
                                                                row.scale_x=1.2
                                                                row.scale_y=1.2
    
                                                    if activeNode.node_tree.name.lower() == 'mn generator':
                                                        if poissonState:
                                                            if socket.name.lower() == "exact amount distribution" or socket.name.lower() == "emitter geometry to hair" or socket.name.lower() == "distribute on vertices":
                                                                row.enabled = False
                                                        if exactAmountState:
                                                            if socket.name.lower() == "poisson disk distribution" or socket.name.lower() == "emitter geometry to hair" or socket.name.lower() == "distribute on vertices":
                                                                row.enabled = False
                                                        if onVerticesState:
                                                            if socket.name.lower() == "poisson disk distribution" or socket.name.lower() == "emitter geometry to hair" or socket.name.lower() == "exact amount distribution":
                                                                row.enabled = False
                                                                
                                                    if socket.name.lower() == "cached":
                                                        row = layout.row()
                                                        if not enableItem:
                                                            row.enabled = enableItem
                                                        opText = 'Freeze'
                                                        cacheIcon = 'FREEZE'
                                                        if socket.default_value == True:
                                                            opText = 'Unfreeze'
                                                            cacheIcon = 'LOCKED'
                                                            row.alert = True

                                                        #Put the button in the box, disable the emboss to make it "borderless"  
                                                        box = row.box()
                                                        box.emboss = 'NONE'

                                                        #Warning row
                                                        spline_count, point_count = getHairCount(bpy.data.objects[item.objectName])
                                                        warningRow = layout.row()
                                                        if not cachedState and spline_count == 0:
                                                            warningRow.template_reports_banner()

                                                        if cachedState:
                                                            box.emboss = 'NONE'
                                                            box.alert=True

                                                        operator='medusanodes.cache_node'
                                                        op = box.operator(operator,icon =cacheIcon, text =opText)

                                                        op.objectName= item.objectName
                                                        op.nodeTreeName  = item.nodeTreeName
                                                        op.nodeName  = node_ref.name
                                                        op.socketName = socket.name
                                                        layout.separator(factor=1.0)

                                                        if not deformerCached:
                                                            row.enabled = False
                                                        # deps = bpy.context.evaluated_depsgraph_get()
                                                        # eval_obj = bpy.data.objects[item.objectName].evaluated_get(deps)
                                                        # if bpy.data.objects[item.objectName].type == "CURVES":
                                                        #     deps = bpy.context.evaluated_depsgraph_get()
                                                        #     eval_obj = bpy.data.objects[item.objectName].evaluated_get(deps)
                                                        #     if len(eval_obj.data.curves.data.points)==0:
                                                        #         row.enabled = False
                                                    
                                                    if socket.name.lower() == "curve collection":
                                                        if curveCollection:
                                                            collection = bpy.data.collections.get(curveCollection.name)

                                                            layout.separator()
                                                            row = layout.row()

                                                            if activeNode.inputs['Cached'].default_value == True:
                                                                row.enabled = False
                                                            
                                                            row.template_list("MEDUSANODES_UL_CurveObjectCollection", "", collection, "objects", collection, "curve_object_index",
                                                            sort_reverse=False,
                                                            sort_lock=False,
                                                            rows=3,)
                                                            
                                                            col = row.column(align=True)
                                                            col.operator('medusanodes.add_curve_object', icon= 'ADD',text = "", depress = False )
                                                            col.operator('medusanodes.remove_curve_object', icon= 'REMOVE',text = "", depress = False ).index = collection.curve_object_index
                                                            col.separator()
                                                            layout.separator()
            # except Exception as E:
            #     print(E)
        needToUpdate = True
    def show_socket_input(self, socket):
        return hasattr(socket, 'draw') and socket.enabled
 
    def write_image_name(self, socket):
        bpy.context.scene.medusaNodesSaveOptions.image_name = socket.name
        print(socket.name)


def draw_medusa_subpanel(self, context):  
    global activeNodeGroupName
    global activeNodeObjectName
    global activeNodeTreeGroupName
    layout = self.layout
    
    data = self.data
    item = data["item"]
    if check_valid(item):
        geoNodes = bpy.data.objects[item.objectName].modifiers[item.nodeTreeName]

        inpTypes = ["VALUE","INT","BOOLEAN","RGBA"]
        enableItem = True
        medusaNodesTree = context.scene.medusaNodesTree
        #if the frame not at the start
        if context.scene.frame_start != context.scene.frame_current:
            if 'guide'in item.objectName.lower():
                for node in medusaNodesTree:
                    if 'dynamics' in node.label.lower() and node.objectName == item.objectName and geoNodes.node_group.nodes[node.name].inputs['Disable'].default_value == False:
                        enableItem = True
                        if item.nodeIndex < node.nodeIndex:
                            enableItem = False
                        if item.objectName.lower() == item.label.lower():
                            enableItem = True
            else:
                enableItem = True
                for node in medusaNodesTree:
                    if 'dynamics' in node.label.lower() and node.objectName == item.objectName and geoNodes.node_group.nodes[node.name].inputs['Disable'].default_value == False:
                        enableItem = True
                        if item.nodeIndex < node.nodeIndex:
                            enableItem = False
                        if item.objectName.lower() == item.label.lower():
                            enableItem = True
        else:
            enableItem = True
        if geoNodes:
            if geoNodes.type == 'NODES':
                if item.name in geoNodes.node_group.nodes:
                    deformerCached = True
                    node_ref = geoNodes.node_group.nodes[item.name]
                    
                    if "mn generator" not in node_ref.node_tree.name.lower():
                        for node in geoNodes.node_group.nodes:
                            if node.type == "GROUP":
                                if "mn generator" in node.node_tree.name.lower():
                                    for inp in node.inputs:
                                        if inp.name.lower() == "cached":
                                            if not inp.default_value:
                                                deformerCached = False
                                                
                    layout = layout.column(align=True)#TEST: align props
                    row = layout.row(align=True)
                    value_inputs = [socket for socket in node_ref.inputs if hasattr(socket, 'draw') and socket.enabled]
                    value_inputs_grouped = data["inputs"]

                    if value_inputs:
                        cachedState = False
                        followGuideState = False
                        followPreviousClump = False
                        followPreviousClumpCell = False
                        for socket in value_inputs:
                            if socket.name.lower() == "cached":
                                if socket.default_value == True:
                                    cachedState = True
                                    break
                            if node_ref.node_tree.name.lower() == 'mn clump':
                                if node_ref.inputs['Follow Guides'].default_value == True:
                                    followGuideState = True
                                if node_ref.inputs['Follow Previous Clump'].default_value == True:
                                    followPreviousClump = True
                                if node_ref.inputs['Follow Previous Clump Cell'].default_value == True:
                                    followPreviousClumpCell = True

                        for socket_name in value_inputs_grouped:
                            if socket_name in node_ref.inputs:
                                socket = node_ref.inputs[socket_name]
                                if socket.name.lower() != "disable" and socket.name.lower() != "cached":
                                    fromNode = None
                                    for link in geoNodes.node_group.links:
                                        if link.to_node == node_ref:
                                            if link.to_socket == socket and socket.type in inpTypes:
                                                fromNode = link.from_node
                                                break

                                    if fromNode:
                                        floatCurvesNodes = {}
                                        
                                        if fromNode.type == "GROUP":
                                            for snode in fromNode.node_tree.nodes:
                                                if snode.type == "CURVE_FLOAT":
                                                    floatCurvesNodes[snode.label] = snode
                                                elif snode.type == "VALTORGB":
                                                    floatCurvesNodes[snode.label] = snode
                                        value_inputs_from = [socketF for socketF in fromNode.inputs if hasattr(socketF, 'draw') and socketF.enabled]
                                        row = layout.row()
                                        if cachedState:
                                            if node_ref.node_tree.name in inputsDisableData:
                                                if socket.name in inputsDisableData[node_ref.node_tree.name]:
                                                    row.enabled = False

                                        if "expanded" not in socket:
                                            op = row.operator("medusanodes.nodeprop_expand", text="", icon='TRIA_RIGHT', emboss=False)
                                        else:
                                            if not socket["expanded"]:
                                                op = row.operator("medusanodes.nodeprop_expand", text="", icon='TRIA_RIGHT', emboss=False)
                                            else:
                                                op = row.operator("medusanodes.nodeprop_expand", text="", icon='TRIA_DOWN', emboss=False)
                                        op.objectName= item.objectName
                                        op.nodeTreeName  = item.nodeTreeName
                                        op.nodeName  = node_ref.name
                                        op.socketName = socket.name
                                        
                                        if socket.links[0].is_muted:
                                            socket.draw(
                                                context,
                                                row,
                                                node_ref,
                                                iface_(socket.label if socket.label else socket.name, socket.bl_rna.translation_context),
                                            )
                                        else:
                                            row.label(text=socket.name)
                                        # row.menu("MEDUSANODE_MT_masks_group_menu", icon='PLUGIN', text="")
                                        op = row.operator("medusanodes.invokemaskmenu", icon='PLUGIN', text="")
                                        op.groupNodeSocketName = socket.name
                                        op.nodeGroup = node_ref.node_tree.name
                                        op.groupNodeName = fromNode.name
                                        row.scale_x=1.2
                                        row.scale_y=1.2

                                        if "expanded" in socket:
                                            if socket["expanded"]:
                                                box = layout.box()
                                                row = box.row(align=True)
                                                if cachedState:
                                                    if node_ref.node_tree.name in inputsDisableData:
                                                        if socket.name in inputsDisableData[node_ref.node_tree.name]:
                                                            row.enabled = False
                                                row.label(text="  "+fromNode.node_tree.name)

                                                viewNode = None
                                                for lnk in fromNode.outputs[0].links:
                                                    if lnk.to_node.type == "VIEWER":
                                                        if "MN Viewer" in lnk.to_node.name:
                                                            viewNode = lnk.to_node
                                                            break
                                                if viewNode is None:
                                                    row.operator("medusanodes.view_group", icon="HIDE_OFF",text="").groupNodeName = fromNode.name
                                                else:
                                                    row.operator("medusanodes.view_group", icon="HIDE_ON",text="").groupNodeName = fromNode.name
                                                
                                                if socket.links[0].is_muted:
                                                    row.operator("medusanodes.mute_link", icon='RESTRICT_VIEW_ON', text="").socketName = socket.name
                                                else:
                                                    row.operator("medusanodes.mute_link", icon='RESTRICT_VIEW_OFF', text="").socketName = socket.name
                                                
                                                row.operator("medusanodes.edit_mask_group", icon='NODETREE', text="").groupNodeName = fromNode.name
                                                row.operator("medusanodes.delete_group", icon='TRASH', text="").groupNodeName = fromNode.name
                                                row.scale_x=1.2
                                                row.scale_y=1.2

                                                l = 0
                                                for sub_socket in value_inputs_from:
                                                    if sub_socket.name.lower() != "disable" and socket.type in inpTypes:
                                                        if str(l+1) in floatCurvesNodes:
                                                            boxV = box.box()
                                                            if socket.links[0].is_muted:
                                                                boxV.enabled = False
                                                            else:
                                                                boxV.enabled = True
                                                            if not enableItem:
                                                                boxV.enabled = enableItem
                                                            floatCurvesNode = floatCurvesNodes[str(l+1)]
                                                            value_inputs_from = [socketF for socketF in floatCurvesNode.inputs if hasattr(socketF, 'draw') and socketF.enabled]
                                                            if hasattr(floatCurvesNode, "draw_buttons_ext"):
                                                                floatCurvesNode.draw_buttons_ext(context, boxV)
                                                            elif hasattr(floatCurvesNode, "draw_buttons"):
                                                                floatCurvesNode.draw_buttons(context, boxV)
                                                        
                                                        row = box.row()
                                                        if socket.links[0].is_muted:
                                                            row.enabled = False
                                                        else:
                                                            row.enabled = True
                                                        sub_socket.draw(
                                                            context,
                                                            row,
                                                            fromNode,
                                                            iface_(sub_socket.label if sub_socket.label else sub_socket.name, sub_socket.bl_rna.translation_context),
                                                        )
                                                        if cachedState:
                                                            if node_ref.node_tree.name in inputsDisableData:
                                                                if socket.name in inputsDisableData[node_ref.node_tree.name]:
                                                                    row.enabled = False
                                                        if sub_socket.type == "IMAGE":
                                                            row = row.row()
                                                            row.scale_x=1.1
                                                            row.scale_y=1.1

                                                            status = False
                                                            if "paintMode" in bpy.context.object:
                                                                if bpy.context.object["paintMode"]:
                                                                    status = True
                                                            if bpy.context.mode != "PAINT_TEXTURE":
                                                                status = False
                                                            else:
                                                                status = True
                                                            paintOp = row.operator("medusanodes.paint_mode", icon='BRUSH_DATA', text="",depress=status)
                                                            paintOp.groupNodeName = fromNode.name
                                                            paintOp.socketName = sub_socket.name
                                                            if sub_socket.default_value == None:
                                                                row.enabled = False
                                                            if cachedState:
                                                                if node_ref.node_tree.name in inputsDisableData:
                                                                    if sub_socket.name in inputsDisableData[node_ref.node_tree.name]:
                                                                        row.enabled = False
                                                    l = l + 1
                                                for index in floatCurvesNodes:
                                                    if index.isdigit():
                                                        if int(index) > l:
                                                            boxV = box.box()
                                                            if socket.links[0].is_muted:
                                                                boxV.enabled = False
                                                            else:
                                                                boxV.enabled = True
                                                            if not enableItem:
                                                                boxV.enabled = enableItem
                                                            floatCurvesNode = floatCurvesNodes[index]
                                                            
                                                            value_inputs_from = [socketF for socketF in floatCurvesNode.inputs if hasattr(socketF, 'draw') and socketF.enabled]
                                                            if hasattr(floatCurvesNode, "draw_buttons_ext"):
                                                                floatCurvesNode.draw_buttons_ext(context, boxV)
                                                            elif hasattr(floatCurvesNode, "draw_buttons"):
                                                                floatCurvesNode.draw_buttons(context, boxV)
                                                
                                                row = box.row()
                                    elif socket.is_linked == False:
                                        row = layout.row(align=True)
                                        if socket.name.lower() != "hair material" and socket.name.lower() != "root normal material":
                                            socket.draw(
                                                context,
                                                row,
                                                node_ref,
                                                iface_(socket.label if socket.label else socket.name, socket.bl_rna.translation_context),
                                            )
                                        elif socket.name.lower() == "hair material":
                                            row = layout.row()
                                            split = row.split(factor=0.3)
                                            c = split.row()
                                            c.label(icon_value=layout.icon(bpy.data.objects[item.objectName].active_material))
                                            c.label(text="Hair Material")
                                            split = split.split()
                                            c = split.column()
                                            c.template_ID(bpy.data.objects[item.objectName], "active_material",live_icon=False)
                                        if cachedState:
                                            if node_ref.node_tree.name in inputsDisableData:
                                                if socket.name in inputsDisableData[node_ref.node_tree.name]:
                                                    row.enabled = False
                                        if not enableItem:
                                            row.enabled = enableItem
                                        if socket.type in inpTypes and socket.type != "BOOLEAN":
                                            op = row.operator("medusanodes.invokemaskmenu", icon='PLUGIN', text="")
                                            op.groupNodeSocketName = socket.name
                                            op.nodeGroup = node_ref.node_tree.name
                                            row.scale_x=1.2
                                            row.scale_y=1.2

                                if 'clump' in node_ref.node_tree.name.lower():
                                    if followGuideState:
                                        if socket.name.lower() == "follow previous clump" or socket.name.lower() == "follow previous clump cell":
                                            row.enabled = False
                                    if followPreviousClump:
                                        if socket.name.lower() == "follow guides":
                                            row.enabled = False
                                    if followPreviousClumpCell:
                                        if socket.name.lower() == "follow guides":
                                            row.enabled = False

                                if socket.name.lower() == "cached":
                                    row = layout.row()
                                    if not enableItem:
                                        row.enabled = enableItem
                                    opText = 'Freeze'
                                    cacheIcon = 'FREEZE'
                                    if socket.default_value == True:
                                        opText = 'Unfreeze'
                                        cacheIcon = 'LOCKED'
                                        row.alert = True

                                    #Put the button in the box ,disable the emboss to make it "borderless"
                                    box = row.box()
                                    box.emboss = 'NONE'

                                    #Warning row
                                    spline_count, point_count = getHairCount(bpy.data.objects[item.objectName])
                                    warningRow = layout.row()
                                    if not cachedState and spline_count == 0:
                                        warningRow.template_reports_banner()

                                    if cachedState:
                                        box.emboss = 'NONE'
                                        box.alert=True

                                    operator='medusanodes.cache_node'
                                    op = box.operator(operator,icon =cacheIcon, text =opText)

                                    op.objectName= item.objectName
                                    op.nodeTreeName  = item.nodeTreeName
                                    op.nodeName  = node_ref.name
                                    op.socketName = socket.name

                                    if not deformerCached:
                                        row.enabled = False


class SCENE_PT_MedusaNodesHierarchy_Viewport(SCENE_PT_MedusaNodesHierarchy):
    bl_idname = "SCENE_PT_medusanodes_hierarchy_vie"
    bl_space_type = 'VIEW_3D'

class SCENE_PT_MedusaNodesHierarchy_Properties(SCENE_PT_MedusaNodesProperties):
    bl_idname = "SCENE_PT_medusanodes_properties_vie"
    bl_space_type = 'VIEW_3D'

def prefOnChange(self,context):
    medusaNodesSaveOptions = bpy.context.scene.medusaNodesSaveOptions
    medusaNodesSaveOptions.autoTexBak = self.autoTexBak
    medusaNodesSaveOptions.useAltTexPath = self.useAltTexPath
    medusaNodesSaveOptions.texPath = self.texPath
    medusaNodesSaveOptions.image_settings.file_format = self.image_settings.file_format
    medusaNodesSaveOptions.image_settings.color_mode = self.image_settings.color_mode
    medusaNodesSaveOptions.image_settings.color_depth = self.image_settings.color_depth
    medusaNodesSaveOptions.image_settings.compression = self.image_settings.compression
    medusaNodesSaveOptions.image_settings.quality = self.image_settings.quality
    medusaNodesSaveOptions.image_settings.tiff_codec = self.image_settings.tiff_codec
    medusaNodesSaveOptions.image_settings.exr_codec = self.image_settings.exr_codec

class SCENE_PT_MedusaNodesPrefs(bpy.types.AddonPreferences):
    bl_idname = __package__

    autoTexBak: BoolProperty(
        name="Automatic Texture Backup",
        description="Automatic Texture Backup",
        default = False,
        update=prefOnChange)

    useAltTexPath: BoolProperty(
        name="Use Alternative Texture Path",
        description= "Use alternative texture path for saving the images on your hard drive",
        default = False,
        update=prefOnChange)

    texPath: StringProperty(
        name="",
        description="",
        default = "",
        update=prefOnChange)

    image_settings: bpy.props.PointerProperty(type = RENDER_PG_bn_image_settings,update=prefOnChange)


    # addon updater preferences from `__init__`, be sure to copy all of them
    auto_check_update: bpy.props.BoolProperty(
        name = "Auto-check for Update",
        description = "If enabled, auto-check for updates using an interval",
        default = False,
    )

    updater_interval_months: bpy.props.IntProperty(
        name='Months',
        description = "Number of months between checking for updates",
        default=0,
        min=0
    )
    updater_interval_days: bpy.props.IntProperty(
        name='Days',
        description = "Number of days between checking for updates",
        default=7,
        min=0,
    )
    updater_interval_hours: bpy.props.IntProperty(
        name='Hours',
        description = "Number of hours between checking for updates",
        default=0,
        min=0,
        max=23
    )
    updater_interval_minutes: bpy.props.IntProperty(
        name='Minutes',
        description = "Number of minutes between checking for updates",
        default=0,
        min=0,
        max=59
    )

    def draw(self, context):
        layout = self.layout
        row = layout.row()
        row.prop(self,"autoTexBak")

        txt = 'Note: By default textures connected to the groom will be saved in "textures" folder next to the .blend file.'
        chars = int(context.region.width / 6) 
        wrapper = textwrap.TextWrapper(width=chars)
        box = layout.box()
        box.alignment = 'EXPAND'
        text_lines = wrapper.wrap(text=txt)
        for text_line in text_lines:
            row = box.row(align = True)
            row.label(text=text_line)
        row = layout.row()
        row.prop(self,"useAltTexPath")

        row = layout.row()
        row.enabled = False
        if self.useAltTexPath:
            row.enabled = True
        row.prop(self,"texPath")

        col = layout.column()
        image_settings = self.image_settings

        grid = col.grid_flow(row_major = True, columns = 2,)
        grid.label(text='File Format')
        grid.prop(image_settings, "file_format", text = "")
        grid.label(text='Color')
        grid.prop(image_settings, "color_mode", text = "")
        grid.label(text='Color Depth')
        grid.prop(image_settings, "color_depth", text = "")
        if image_settings.file_format in ['PNG']:
            grid.label(text='Compression')
            grid.prop(image_settings, "compression", text = "") 
        if image_settings.file_format in ['JPEG', 'JPEG2000', 'AVI_JPEG']:
            grid.label(text='Quality')
            grid.prop(image_settings, "quality", text = "")
        if image_settings.file_format in ['TIFF']:
            grid.label(text='Compression')
            grid.prop(image_settings, "tiff_codec", text = "") 
        if image_settings.file_format in ['OPEN_EXR', 'OPEN_EXR_MULTILAYER']:
            grid.label(text='Compression')
            grid.prop(image_settings, "exr_codec", text = "")
        # #Auto-updater
        addon_updater_ops.update_settings_ui(self,context)

def subpanel_update():
    global needToUpdate
    if needToUpdate:
        global subPanels

        panel_class_names = []
        for cls in bpy.types.Panel.__subclasses__():
            if "MedusaSubPanel" in cls.__name__ and cls.is_registered:
                panel_class_names.append(cls)

        for classO in panel_class_names:
            try:
                bpy.utils.unregister_class(classO)
            except:
                pass

        for pan in subPanels:
            bpy.utils.register_class(pan)
        needToUpdate = False
    return 0.1

@persistent
def nodesUpdate(scene):
    global sculptMode
    global lstCount
    global lstData

    correctBrush = False
    if bpy.context.mode == 'SCULPT_CURVES':
        contextSculptBrush = bpy.context.tool_settings.curves_sculpt.brush
        for brush in brushList:
            if brush.lower() in contextSculptBrush.name.lower():
                #print(contextSculptBrush.name)
                correctBrush = True
    if correctBrush or bpy.context.mode == "EDIT_CURVES":
        exist = False
        activeOb = bpy.context.active_object
        emitterGeo = None
        for mod in activeOb.modifiers:
            if mod.type == "NODES" and "medusanodes" in mod.name.lower():
                exist = True
                emitterGeo = mod["Socket_2"]
                break
        if exist:
            deps = bpy.context.evaluated_depsgraph_get()
            eval_obj = activeOb.evaluated_get(deps)
            point_count = len(eval_obj.data.curves.data.points)
            sel = np.zeros([len(eval_obj.data.curves.data.points)*3], dtype="f")
            eval_obj.data.curves.data.points.foreach_get("position",sel)
            if lstCount == -1:
                lstCount = point_count
            if lstData is None:
                lstData = sel
            updateNode = False
            if point_count != lstCount:
                lstData = sel
                lstCount = point_count
                updateNode = True
            else:
                if not np.array_equal(sel,lstData):
                    lstData = sel
                    updateNode = True
            if updateNode:
                addApplyUpdateTree(activeOb,emitterGeo)
    # update_curve_object_list()
   
    try:
        if fstT == 0:
            updateTexOptions()
        if hasattr(bpy.context,"object"):
                obj = bpy.context.object
                if obj and sculptMode == False:
                    if len(obj.modifiers)>0:
                        geoNodes = obj.modifiers.active
                        if geoNodes == None:
                            medusaNodes = bpy.context.scene.medusaNodes
                            treeList = bpy.context.scene.medusaNodesTree
                            medusaNodes.clear()
                            treeList.clear()
                        elif scene.medusaNodesActiveOb!=obj.name:
                            SetupNodeData()
                        elif geoNodes:
                            if geoNodes.type == 'NODES':
                                SetupNodeData()
                            elif geoNodes.node_group is not None:
                                if scene.medusaNodesActiveNodeTree!=geoNodes.node_group.name:
                                    SetupNodeData()
                    elif bpy.context.scene.lastMedusaHairOb in bpy.data.objects and not bpy.context.scene.use_medusa_hair_object:
                        SetupNodeData()
                    else:
                        if "hairOb" in obj:
                            if obj["paintMode"] == False:
                                medusaNodes = bpy.context.scene.medusaNodes
                                treeList = bpy.context.scene.medusaNodesTree
                                medusaNodes.clear()
                                treeList.clear()
                        else:
                            medusaNodes = bpy.context.scene.medusaNodes
                            treeList = bpy.context.scene.medusaNodesTree
                            medusaNodes.clear()
                            treeList.clear()
                elif sculptMode == False:
                    medusaNodes = bpy.context.scene.medusaNodes
                    treeList = bpy.context.scene.medusaNodesTree
                    medusaNodes.clear()
                    treeList.clear()
                if obj and sculptMode:
                    if obj.mode != "SCULPT_CURVES":
                        sculptMode = False  
    
    except Exception as E:
        print("################")
        print(E)

global manSave
manSave = 0

@persistent
def blendFileSaved(scene):
    global manSave
    if manSave == 0:
        medusaNodesSaveOptions = bpy.context.scene.medusaNodesSaveOptions
        objectsToProcess = []

        format = bpy.context.scene.render.image_settings.file_format
        colorMode = bpy.context.scene.render.image_settings.color_mode
        depth = bpy.context.scene.render.image_settings.color_depth
        compr = bpy.context.scene.render.image_settings.compression
        quali = bpy.context.scene.render.image_settings.quality
        tiffCodec = bpy.context.scene.render.image_settings.tiff_codec
        exrCodec = bpy.context.scene.render.image_settings.exr_codec

        bpy.context.scene.render.image_settings.file_format = medusaNodesSaveOptions.image_settings.file_format
        bpy.context.scene.render.image_settings.color_mode = medusaNodesSaveOptions.image_settings.color_mode
        bpy.context.scene.render.image_settings.color_depth = medusaNodesSaveOptions.image_settings.color_depth
        bpy.context.scene.render.image_settings.compression = medusaNodesSaveOptions.image_settings.compression
        bpy.context.scene.render.image_settings.quality = medusaNodesSaveOptions.image_settings.quality
        bpy.context.scene.render.image_settings.tiff_codec = medusaNodesSaveOptions.image_settings.tiff_codec
        bpy.context.scene.render.image_settings.exr_codec = medusaNodesSaveOptions.image_settings.exr_codec

        extensio = bpy.context.scene.render.file_extension
        for ob in bpy.data.objects:
            if len(ob.modifiers)>0:
                for mod in ob.modifiers:
                    if mod.type == "NODES":
                        for node in mod.node_group.nodes:
                            if node.type == "GROUP":
                                if "mn" in node.node_tree.name.lower():
                                    if ob not in objectsToProcess:
                                        objectsToProcess.append(ob)
                                    break
        
        if not medusaNodesSaveOptions.useAltTexPath:
            texturePath = os.path.join(bpy.path.abspath("//"),"textures","Medusa Nodes")
        else:
            texturePath = os.path.join(bpy.path.abspath(medusaNodesSaveOptions.texPath),"textures","Medusa Nodes")
        
        createBackup = False
        if medusaNodesSaveOptions.autoTexBak:
            for ob in bpy.data.objects:
                if len(ob.modifiers)>0:
                    for mod in ob.modifiers:
                        if mod.type == "NODES":
                            for node in mod.node_group.nodes:
                                for inp in node.inputs:
                                    if inp.type == "IMAGE":
                                        im = inp.default_value
                                        if im.is_dirty:
                                            createBackup = True
                                            break
        
        if medusaNodesSaveOptions.autoTexBak and createBackup:
            for ob in objectsToProcess:
                if len(ob.modifiers)>0:
                    for mod in ob.modifiers:
                        if mod.type == "NODES":
                            name = ob.name.split("_")
                            if len(name)>1:
                                date = datetime.datetime.now().strftime("_%Y.%m.%d_%H-%M")
                                groomPath = os.path.join(texturePath,"backup",name[0]+date)
                                partPath = os.path.join(groomPath,name[1])
                            else:
                                date = datetime.datetime.now().strftime("_%Y.%m.%d_%H-%M")
                                groomPath = os.path.join(texturePath,"backup",name[0]+date)
                                partPath = os.path.join(groomPath,name[0])

                                
                            for node in mod.node_group.nodes:
                                for inp in node.inputs:
                                    if inp.type == "IMAGE":
                                        im = inp.default_value
                                        if im.has_data:
                                            fileName = os.path.basename(im.filepath)
                                            if fileName == "":
                                                fileName = im.name
                                            if im.packed_file:
                                                im.unpack(method='USE_LOCAL')
                                            if extensio not in fileName.lower():
                                                fileName = fileName + extensio
                                            os.makedirs(partPath,exist_ok=True)
                                            im.filepath_raw =os.path.join(partPath,fileName)
                                            im.save()
                                            #im.save_render(os.path.join(partPath,fileName)) #this saves the color management as well, which is not ideal
                                            im.filepath = os.path.join(partPath,fileName)
                                            im.source = 'FILE'

        else:
            for ob in objectsToProcess:
                if len(ob.modifiers)>0:
                    for mod in ob.modifiers:
                        if mod.type == "NODES":
                            name = ob.name.split("_")
                            if len(name)>1:
                                groomPath = os.path.join(texturePath,name[0])
                                partPath = os.path.join(groomPath,name[1])
                            else:
                                date = datetime.datetime.now().strftime("_%Y.%m.%d_%H-%M")
                                groomPath = os.path.join(texturePath,"backup",name[0]+date)
                                partPath = os.path.join(groomPath,name[0])
                            
                            for node in mod.node_group.nodes:
                                for inp in node.inputs:
                                    if inp.type == "IMAGE":
                                        im = inp.default_value
                                        if im:
                                            if im.has_data:
                                                fileName = os.path.basename(im.filepath)
                                                if fileName == "":
                                                    fileName = im.name
                                                if im.packed_file:
                                                    im.unpack(method='USE_LOCAL')
                                                if extensio not in fileName.lower():
                                                    fileName = fileName + extensio
                                                os.makedirs(partPath,exist_ok=True)
                                                im.filepath_raw = os.path.join(partPath,fileName)
                                                im.save()
                                                #im.save_render(os.path.join(partPath,fileName)) #this saves the color management as well, which is not ideal
                                                im.filepath = os.path.join(partPath,fileName)
                                                im.source = 'FILE'
                                                

        bpy.context.scene.render.image_settings.file_format = format
        bpy.context.scene.render.image_settings.color_mode = colorMode
        bpy.context.scene.render.image_settings.color_depth = depth
        bpy.context.scene.render.image_settings.compression = compr
        bpy.context.scene.render.image_settings.quality = quali
        bpy.context.scene.render.image_settings.tiff_codec = tiffCodec
        bpy.context.scene.render.image_settings.exr_codec = exrCodec
        
        if manSave == 0:
            manSave = 1
            bpy.ops.wm.save_mainfile()
    else:
        manSave = 0
